#include "AdjGraph.h"
#include <iostream>
#include <fstream>

#include <math.h>
#include <time.h>
#include "Cluster.h"
using namespace std;
//======vertex=======
Vertex::Vertex(int num)
{
   mNode=num;
   mAdjacentList=new vector<Vertex* >();
}

void
Vertex::insertAdjacent(Vertex* n)
{
   mAdjacentList->push_back(n);
}

Vertex::~Vertex()
{
  delete mAdjacentList;
}

//----------Edge--------------


Edge::Edge(Vertex* s, Vertex* t)
{
   mSource=s;
   mTarget=t;
}
Edge::~Edge()
{

}


AdjGraph::AdjGraph(double alpha)
{
  mVertices=new vector<Vertex *>();
  mAlpha=alpha;
  mClusters=NULL;
}

AdjGraph::AdjGraph(double alpha, int numNode)
{
  mVertices=new vector<Vertex *>(numNode);
  mAlpha=alpha;
  for(int i=0; i<numNode; i++)
  {
    (*mVertices)[i]=new Vertex(i);
  }
  mClusters=NULL;
}


AdjGraph::~AdjGraph()
{
   int size=mVertices->size();
   for(int i=0; i<size; i++)
   {
     delete (*mVertices)[i];
   }
   delete mVertices;
}

void
AdjGraph::randomPermuteVertex()
{
   srand(time(NULL));
   int size=mVertices->size();
   for(int i=0; i<size-1; i++)
   {
      Vertex* v=(*mVertices)[i];
      int s=(random()%(size-1));
      Vertex* t=(*mVertices)[s];
      (*mVertices)[i]=t;
      (*mVertices)[s]=v;
   }
   int* mapping=new int[size];
   for(int i=0; i<size; i++)
   {
       mapping[(*mVertices)[i]->getNumber()]=i;
      (*mVertices)[i]->setNumber(i);
   }
   
   for(int i=0; i<mClusters->size(); i++)
   {
     vector<int>* cluster=(*mClusters)[i];
     for(int j=0; j<cluster->size(); j++)
     {
        (*cluster)[j]=mapping[(*cluster)[j]];
     }
   }
}

void 
AdjGraph::readGraph(char* aFileName)
{
   mNames=new map<string, int>();
   mInverseNames=new map<int, string>();

   ifstream input(aFileName);
   if(!input)
   {
      cerr<<"Can not open file "<<aFileName<<endl;
      exit(0);
   }
   char line[400];
   char* token; 
   while(!input.eof())
   {
      input.getline(line, 400); 
      token=strtok(line, "\t ");
      if(token==NULL) continue;
      string name1=token;
      string name2=strtok(NULL, "\t \n");
      //cout<<name1<<" ....."<<name2<<endl;
      addEdge(name1, name2);
   }    
}


void 
AdjGraph::depthFS(vector<int>* cluster, int start_v, int base, int size)
{
   cluster->push_back(start_v+base);
   visited[start_v]=true;
   for(int i=start_v+1; i<size; i++)
   {
      if(edge_matrix[start_v][i]==0) continue;
      if(visited[i]) continue;
      depthFS(cluster, i, base, size);
   }
}
void
AdjGraph::randomGraph2(int numNode)
{
    visited=new bool[numNode];
    int* first_=new int[numNode];
    int* second_=new int[numNode];

    for(int i=0; i<numNode; i++)
    {
       mVertices->push_back(new Vertex(i));

       visited[i]=false;
       first_[i]=-1;
       second_[i]=-1;
    }//for
    

    for(int i=0; i<numNode-2; i++)
    {
      int first=0;
      int second=0;
      while(first==second)
      {
         first=random()%(numNode-i-1)+i+1;
         second=random()%(numNode-i-1)+i+1;
      }//while
      first_[i]=first;
      second_[i]=second;

      (*mVertices)[i]->insertAdjacent((*mVertices)[first]);
      (*mVertices)[i]->insertAdjacent((*mVertices)[second]);

      (*mVertices)[first]->insertAdjacent((*mVertices)[i]);
      (*mVertices)[second]->insertAdjacent((*mVertices)[i]);
    }//for
    (*mVertices)[numNode-1]->insertAdjacent((*mVertices)[numNode-2]);
    (*mVertices)[numNode-2]->insertAdjacent((*mVertices)[numNode-1]);
    first_[numNode-2]=numNode-1;
    mClusters=new vector<vector<int>*>();
    for(int i=0; i<numNode-2; i++)
    {
       if(visited[i]) continue;
       if(first_[i]==-1 || visited[first_[i]]) continue;
       if(second_[i]==-1 || visited[second_[i]]) continue;

       int f=first_[i];
       int s=second_[i];
       if(f<s)
       {
          if(first_[f]!=s && second_[f]!=s) continue;
       }
       else
       {
         if(first_[s]!=f && second_[s]!=f) continue;
       }
       vector<int>* new_c=new vector<int>();
       new_c->push_back(i);
       new_c->push_back(first_[i]);
       new_c->push_back(second_[i]);
       mClusters->push_back(new_c);
       visited[i]=true;
       if(first_[i]!=-1) visited[first_[i]]=true;
       if(second_[i]!=-1) visited[second_[i]]=true;
    }//for


    for(int i=0; i<numNode-2; i++)
    {
       if(visited[i]) continue;
       if((first_[i]==-1 || visited[first_[i]]) && (second_[i]==-1 || visited[second_[i]])) continue;
       
       vector<int>* new_c=new vector<int>();
       new_c->push_back(i);
       
       if((first_[i]!=-1 && !visited[first_[i]]))
       {
         new_c->push_back(first_[i]);
	 visited[first_[i]]=true;
       }
       if(new_c->size()<2) 
       { if((second_[i]!=-1 && !visited[second_[i]]))
        { 
         new_c->push_back(second_[i]);
         visited[second_[i]]=true; 
	}
       }
       mClusters->push_back(new_c);
       visited[i]=true;
    }//for 

    double credit=0;
    for(int i=0; i<mClusters->size(); i++)
    {
       if((*mClusters)[i]->size()==3)
          credit+=mAlpha*(*mClusters)[i]->size();
       else credit+=mAlpha;
    }
    randomPermuteVertex();
    cout<<"Heuristic Credit is"<<credit<<endl;
}
void
AdjGraph::randomGraph(int numNode, int largestClique, double prob_remove, double percent_true_edge)
{
  for(int i=0; i<numNode; i++)
  {
     mVertices->push_back(new Vertex(i));
  }//for
  int index=0;
  double credit=0;
  double correct_edge=0;
  double total_edge=0;
  
  edge_matrix=new int *[largestClique];
  for(int i=0; i<largestClique; i++)
  {
     edge_matrix[i]=new int[largestClique];
  }//for
  visited=new bool[largestClique];
  
  
  
  
  mClusters=new vector<vector<int>* >();
  
  
  while(index<numNode-1)
  {
     int size=random()%(largestClique-2)+2;
     if(index+size>numNode) size=numNode-index;
     //credit-=(0.5*size*(size-1));
     for(int i=0; i<size; i++)
     {
        for(int j=0; j<size; j++)
	{
	  edge_matrix[i][j]=0;
	}
	visited[i]=false;
     }
     
     
     for(int i=0; i<size; i++)
     {
       for(int j=i+1; j<size; j++)
       {
          double prob=random()*1.0/RAND_MAX;
          if(prob>prob_remove) 
          {
            (*mVertices)[i+index]->insertAdjacent((*mVertices)[j+index]);
            (*mVertices)[j+index]->insertAdjacent((*mVertices)[i+index]);

            edge_matrix[i][j]=1;
            edge_matrix[j][i]=1;
//	    credit+=(1+mAlpha);
            correct_edge+=1.0;
            total_edge+=1.0;
            //cout<<(i+index)<<" "<<(j+index)<<endl;
          }
       }
     } 

     vector<int>* cluster=new vector<int>();
     for(int i=0; i<size; i++)
     {
        if(visited[i]) continue;
	depthFS(cluster, i,  index, size);
        if(cluster->size()!=0)
	{
	  mClusters->push_back(cluster);
	  int s=cluster->size();
	  double e=0;
	  for(int k=0; k<s; k++)
	  {
	    for(int j=k+1; j<s; j++)
	    {
	       if(edge_matrix[(*cluster)[k]-index][(*cluster)[j]-index]!=0)
	       {
	         e+=1;
	       }
	    }
	  }
	  credit+=((1+mAlpha)*e-0.5*s*(s-1));
	  cluster=new vector<int>();
	}
	
     }
     delete cluster;

     index=index+size; 
  }//while
  
  double prob_add=2*correct_edge*(1.0/percent_true_edge-1)/((numNode-largestClique)*(numNode-largestClique-1));
  if(prob_add==1) prob_add=0.9;
  if(prob_add<0)  prob_add=0;
  for(int i=0; i<numNode; i++)
  {  
     for(int j=i+largestClique; j<numNode; j++)
     {
        double prob=random()*1.0/RAND_MAX;
        if(prob<prob_add)
        {
            (*mVertices)[i]->insertAdjacent((*mVertices)[j]);
            (*mVertices)[j]->insertAdjacent((*mVertices)[i]);      
            total_edge+=1;    
        } 

     }
  }    
  randomPermuteVertex();
  cout<<"Clique edge percent"<<(correct_edge/total_edge)<<" "<<correct_edge<<" "<<((int)total_edge)<<" "<<prob_add<<endl;
  cout<<"Expected credit is:"<<credit<<endl;

}

void
AdjGraph::generateGridGraph(int numNode, int largestClique, double prob_remove, double prob_add)
{
  int size=(int)sqrt(numNode);

  for(int i=0; i<numNode; i++)
  {
     mVertices->push_back(new Vertex(i));
  }//for
   
//  randomPermuteVertex();
  for(int i=0; i<size; i++)
  {
    for(int j=0; j<size; j++)
    {
       int this_index=i*size+j;
       if(j<size-1)
       {
         (*mVertices)[this_index]->insertAdjacent((*mVertices)[this_index+1]);
         (*mVertices)[this_index+1]->insertAdjacent((*mVertices)[this_index]);
       }
       int next_row_index=this_index+size;
       if(i<size-1)
       {
         (*mVertices)[this_index]->insertAdjacent((*mVertices)[next_row_index]);
         (*mVertices)[next_row_index]->insertAdjacent((*mVertices)[this_index]);
       }  
    }
  }
  mClusters=new vector<vector<int> *>();
  randomPermuteVertex();
  cout<<"Expected credit is "<<((size/2)*(size/2)*10)<<endl;
}
void
AdjGraph::insertEdge(int i, int j, EdgeWeightType w)
{
    (*mVertices)[i]->insertAdjacent((*mVertices)[j]);
    (*mVertices)[j]->insertAdjacent((*mVertices)[i]); 
}

void
AdjGraph::print()
{
  cout<<"Adj relations"<<endl;
  for(int i=0; i<mVertices->size(); i++)
  {
     cout<<"vertex "<<i<<" has neighbors: "<<endl;
     vector<Vertex* >* adj=(*mVertices)[i]->getAdjacentList(); 
     cout<<"\t\t";
     for(int k=0; k<adj->size(); k++)
     {
       cout<<(*adj)[k]->getNumber()<<" ";;
     }
     cout<<endl;
  }
}


void
AdjGraph::addEdge(string name1, string name2)
{
    map<string, int>::iterator it1=mNames->find(name1);
    int first_index, second_index;
    if(it1==mNames->end())
    {
       first_index=mNames->size();
       (*mNames)[name1]=first_index;
       (*mInverseNames)[first_index]=name1;
    }
    else
    {
       first_index=it1->second;
    }
    map<string, int>::iterator it2=mNames->find(name2);
    if(it2==mNames->end())
    {
        second_index=mNames->size();
       (*mNames)[name2]=second_index;
       (*mInverseNames)[second_index]=name2;
    }
    else
    {
       second_index=it2->second;
    }
    if(first_index>=mVertices->size())
    {
       mVertices->push_back(new Vertex(first_index)); 
    }
    if(second_index>=mVertices->size())
    {
       mVertices->push_back(new Vertex(second_index));
    }
    if(first_index>=second_index) return; 
    (*mVertices)[first_index]->insertAdjacent((*mVertices)[second_index]);
    (*mVertices)[second_index]->insertAdjacent((*mVertices)[first_index]);
}

void
AdjGraph::output(vector<Cluster*>* clusters)
{
   int s=clusters->size();
   for(int i=0; i<s; i++)
   {
     Cluster* c=(*clusters)[i];
     vector<int>* indices=c->getVertexNumbers(); 
     int s2=indices->size();
     cout<<"cluster \t"<<i<<"\t:";
     for(int j=0; j<s2; j++)
     {
        cout<<(*mInverseNames)[(*indices)[j]]<<" \t";
     }
     cout<<endl;
   }
}

