#ifndef _ADJ_GRAPH_H_
#define _ADJ_GRAPH_H_
#include <vector>
#include <map>
#include <string>
#include <stdlib.h>
#include <string.h>

using namespace std;
typedef int VerDataType; 
typedef float EdgeWeightType;
class Cluster;
class Vertex
{
   protected: 
    int mNode; 
    VerDataType mData; 
    vector<Vertex *>* mAdjacentList;
    
   public: 
     Vertex(int num);
     ~Vertex();
    int getNumber() { return mNode; }
    void setData(VerDataType d) { mData=d; }
    void setNumber(int i) {  mNode=i;}
    void insertAdjacent(Vertex* n);
    vector<Vertex *>* getAdjacentList() { return mAdjacentList; }
    VerDataType getData() { return mData; }
};

class Edge
{
   protected:
     Vertex* mSource;
     Vertex* mTarget;
   public:
     Edge(Vertex* s, Vertex* t);
     ~Edge();
     Vertex* getSource() { return mSource; }
     Vertex* getTarget() { return mTarget; }

};

class AdjGraph
{
    protected:
      vector<Vertex *>* mVertices;
      void  randomPermuteVertex();
      double mAlpha; 
      vector<vector<int> *>* mClusters;
      map<string, int>* mNames;
      map<int, string>* mInverseNames;
      int** edge_matrix;
      bool* visited;
  public:
      AdjGraph(double alpha);
      AdjGraph(double alpha, int numNode);
      ~AdjGraph();
      void readGraph(char* aFileName);
      vector<Vertex *>* getVertices() { return mVertices;  } 
      void randomGraph(int numNode, int largestClique, double prob_remove, double prob_add);
      void generateGridGraph(int numNode, int largestClique, double prob_remove, double prob_add);
      void insertEdge(int i, int j, EdgeWeightType w);
      int getNumVertices() { return mVertices->size(); }
      void print();
      void randomGraph2(int num_node);
      vector<vector<int> *>* getClusters() { return mClusters; }
      void output(vector<Cluster*>* clusters);
      void addEdge(string name1, string name2);
      void depthFS(vector<int>* cluster, int start_v, int base, int size);
};

#endif 
