#include "Cluster.h"
#include "ArrayList.h"
#include <iostream>
#include "ClusterGraph.h"
using namespace std;

Cluster::Cluster()
{
   mVertices=new ArrayList<ClusterVertex *>();
   mAdjacentClusters=new ArrayList<ClusterCounterElement *>();
   mAdjacentHash=new SimpleHash<Cluster*, ClusterCounterElement *>(ClusterGraph::HASH_TABLE_SIZE);

   mCredit=0;
   mVisit=true;
   mRemoveBadAltered=true;
   mSplitAltered=true;
   mSwapAltered=true;
   mAltered=true;
   mMoveSingleAltered=true;

   mDynamicList  =NULL;
   mDynOutList   =NULL;
   mDynTable     =NULL;
   mTracePointers=NULL;
   mAdjMatrix    =NULL;
}

Cluster::Cluster(vector<ClusterVertex *>* vertices, int size)
{
   mVertices=new ArrayList<ClusterVertex *>(vertices);
   mAdjacentClusters=new ArrayList<ClusterCounterElement *>();
   mAdjacentHash=new SimpleHash<Cluster*, ClusterCounterElement *>(size);
   mCredit=0;
   mVisit=true;
   mRemoveBadAltered=true;
   mSplitAltered=true;
   mSwapAltered=true;
   mAltered=true;
   mMoveSingleAltered=true;
   
   mDynamicList  =NULL;
   mDynOutList   =NULL;
   mDynTable     =NULL;
   mTracePointers=NULL;
   mAdjMatrix    =NULL;
}


Cluster::~Cluster()
{
   delete mVertices;
   delete mAdjacentClusters;
   delete mAdjacentHash; 
}

int 
Cluster::getNumVertices() 
{ return mVertices->getSize(); }

void
Cluster::moveTo(ClusterVertex* v, Cluster* c, double alpha)
{
   mRemoveBadAltered=true;
   mSplitAltered=true;
   mSwapAltered=true;
   mAltered=true;
   c->mRemoveBadAltered=true;
   c->mSplitAltered=true;
   c->mSwapAltered=true;
   c->mAltered=true;
   
   remove(v);
   c->insert(v);
   v->setCluster(c);
   AdjacentElement* in_list=v->getInList();
   AdjacentElement* new_in_list=v->getOutList(c);
   
   ArrayList<AdjacentElement *>* out_list=v->getOutList();
   vector<AdjacentElement *> *  out_list_v=out_list->getCopy();
   int size=out_list->getSize();
   for(int i=0; i<size; i++)
   {
     AdjacentElement* ae=(*out_list_v)[i];
     ClusterCounterElement* old_cce=ae->getClusterCounterElement();
     
     if(old_cce->getCounter()-ae->getSize()==0)
     {
        remove(old_cce);
        delete old_cce;	
     }
     else
     {
       changeCCECounter(old_cce, -ae->getSize());
     }
     if(ae->getCluster()==c) continue; 
     ClusterCounterElement* new_cce=c->getAdjacentCluster(ae->getCluster());
     if(new_cce==NULL)
     {
        new_cce=new ClusterCounterElement(ae->getCluster(), ae->getSize());
	new_cce->setCluster(ae->getCluster());
	
	c->insert(new_cce);
     }
     else
     {
	c->changeCCECounter(new_cce, ae->getSize());
     }
     ae->setClusterCounterElement(new_cce);
   }
   delete out_list_v;
   //==========end updating of the cluster counter list
   //
   //======

   if(new_in_list==NULL) new_in_list=new AdjacentElement(v);
   else
   {
      //v->getOutList()->remove(new_in_list);
      v->removeOutList(new_in_list);
   }

   int edges_deleted=in_list->getSize();
   int edges_add=(new_in_list->getSize());
   changeScore(-(alpha+1)*edges_deleted+getSize());  
   c->changeScore((alpha+1)*edges_add-c->getSize()+1);  
   //cout<<"move vertex "<<v->getID()<<endl;  
   adjustOutEdges(v, c); 
   removeInEdges(in_list, c); 
   insertInEdges(new_in_list->getEdges());
   new_in_list->setCluster(c);
   v->setInList(new_in_list);
   if(in_list->getSize()==0)
   {
     delete in_list;
   }
   else
   {
     in_list->setCluster(this);
     v->addToOutList(in_list);    
   }
}

void
Cluster::removeInEdges(AdjacentElement* in, Cluster* to)
{
   ArrayList<EdgeEnd *>* edges_end_a=in->getEdges();
   vector<EdgeEnd *>* edges=edges_end_a->getCopy();
   int size=edges_end_a->getSize();
   //cout<<"---==== adjacent size "<<size<<endl;
   for(int i=0; i<size; i++)
   {
      EdgeEnd* end=(*edges)[i];
      ClusterEdge* e= end->getEdge();
      EdgeEnd* theOtherEnd;
      if(e->getSource()==end)
      {
          theOtherEnd=e->getTarget();
      }
      else
      { 
          theOtherEnd=e->getSource();
          if(e->getTarget()!=end) cout<<"ha ha"<<endl;
      }
      
      ClusterVertex* vertex=theOtherEnd->getVertex();
      //cout<<"it is neighbor of vertex: "<<vertex->getID()<<endl;
      //vertex->getInList()->printNeighbor();
      //cout<<"index "<<theOtherEnd->getIndex()<<endl;
      vertex->removeInVertex(theOtherEnd);
      //vertex->getInList()->printNeighbor();
      vertex->insertOutVertex(theOtherEnd, to);
   }//for
   delete edges;

}

void
Cluster::insertInEdges(ArrayList<EdgeEnd* >* ends)
{
   int size=ends->getSize();
   vector<EdgeEnd* >* v_ends=ends->getCopy();
   for(int i=0; i<size; i++)
   {
      EdgeEnd* end=(*v_ends)[i]; 
      ClusterEdge* e= end->getEdge();

      EdgeEnd* theOtherEnd;
      if(e->getSource()==end)
      {
          theOtherEnd=e->getTarget();
      }
      else
      {
          theOtherEnd=e->getSource();
      }

      ClusterVertex* vertex=theOtherEnd->getVertex(); 
      vertex->removeOutVertex(theOtherEnd);  
      vertex->insertInVertex(theOtherEnd);  
   } 
   delete v_ends;
}
void
Cluster::adjustOutEdges(ClusterVertex* v, Cluster* c)
{
   ArrayList<AdjacentElement *>* outs=v->getOutList();
   int size=outs->getSize();
   vector<AdjacentElement *>*  lists=outs->getCopy();
   for(int i=0; i<size; i++)
   {
      AdjacentElement* e=(*lists)[i];
      if(e->getCluster()==c)
      {
        continue;
      }
      ArrayList<EdgeEnd* >* ends=e->getEdges();
      int size2=ends->getSize();
      vector<EdgeEnd *>* ends_v=ends->getCopy();
      for(int j=0; j<size2; j++)
      {
        EdgeEnd* end=((*ends_v)[j]);
        ClusterEdge*    edge=end->getEdge();
        EdgeEnd* theOtherEnd;
        if(end==edge->getSource())
        {
           theOtherEnd=edge->getTarget();
        }
        else 
        {
           theOtherEnd=edge->getSource();
        }
        ClusterVertex* cv=theOtherEnd->getVertex();
        cv->removeOutVertex(theOtherEnd);
	cv->insertOutVertex(theOtherEnd, c);
      }
      delete ends_v;
   }  
   delete lists;

}

vector<vector<ClusterVertex *> *>*
Cluster::partition()
{
   vector<vector<ClusterVertex *> *> * result=new vector<vector<ClusterVertex *> *>(); 
   int size=mVertices->getSize();
   vector<ClusterVertex* >* vers=mVertices->getData();
   for(int i=0; i<size; i++)
   {
      (*vers)[i]->mLocalVisited=false;     
   }
   
   for(int i=0; i<size; i++)
   {
      if((*vers)[i]->mLocalVisited) continue;
      vector<ClusterVertex* >* component=new vector<ClusterVertex *>();
      depthFirstSearch((*vers)[i], component); 
      result->push_back(component);
   }
   return result;
}

void
Cluster::removeBadVertices(Cluster* c0)
{
  if(!mRemoveBadAltered) return;
  createRandomList();
  createMatrix(NULL, NULL);
  int num_list=1<<(mRandomList->size());

  double best_c=-1000;
  int    best_index=0; 
  double best_ave=-1000;
  int    best_index_ave=0; 
 
  int index=0;
  int num_node=0;
  while(index<num_list)
  {
    double c=badCredit(index, num_node);
    if(getSize()>2*num_node)
    {
       if((mCredit+c)/(getSize()-num_node)>best_ave)
       {
	  best_ave=(mCredit+c)/(getSize()-num_node);
	  best_index_ave=index;
       } 
    }
    if(c>best_c)
    {
       best_index=index;
       best_c=c;
    }//if
    index++;
  }
  if(best_c>=0 && best_index!=0)
  {
     int l=mRandomList->size();
     for(int i=0, in=best_index; i<l; i++, in/=2)
     {
        if(in%2==1)
	{
	  moveTo((*mRandomList)[i], c0, ClusterGraph::mAlpha); 
	}	
     }
  }
  else if(best_ave>(mCredit/getSize()))
  {
     int l=mRandomList->size();
     for(int i=0, in=best_index_ave; i<l; i++, in/=2)
     {
        if(in%2==1)
	{
	  moveTo((*mRandomList)[i], c0, ClusterGraph::mAlpha); 
	}	
     }
  
  }
  else
  {
     mRemoveBadAltered=false;
  }
  destroyMatrix(); 
  delete mRandomList;
}

Cluster*
Cluster::splitCluster()
{
  if(!mSplitAltered) return NULL;
  if(getSize()>50) return NULL;
  createRandomList();
  int size=getSize(); 
  createMatrix(NULL, NULL);
  int num_list=1<<(mRandomList->size());
  int index=0;

  double best_c=-1000;
  int    best_index=0; 
  while(index<num_list)
  {
    double c=splitCredit(index);
    if(c>best_c)
    {
       best_index=index;
       best_c=c;
    }//if
    index++;
  }
  Cluster* rev=NULL;
  if(best_c>=0 && best_index!=0)
  {
     rev=new Cluster();
     int l=mRandomList->size();
     double old_c=mCredit;
     
    // consistencyCheck();
     
   /*for(int i=0; i<size; i++)
   {
     ClusterVertex* v=mVertices->getData(i);
     AdjacentElement* in=v->getInList();
     ArrayList<EdgeEnd *>* ends=in->getEdges();
     int ends_size=ends->getSize(); 
     for(int j=0; j<ends_size; j++)
     {
        ClusterVertex* v=ends->getData(j)->getAdjacentVertex();
	int index2=v->getIndex();
        cout<<"("<<i<<","<<index2<<"),";
     }
     cout<<endl;
   }
     
     
     cout<<"old credit is "<<getCredit()<<endl;*/
     for(int i=0, in=best_index; i<l; i++, in/=2)
     {
        if(in%2==1)
	{
	  moveTo((*mRandomList)[i], rev, ClusterGraph::mAlpha); 
	}
     }
     if(old_c>(mCredit+rev->getCredit()))
     {
       cout<<endl;
       for(int i=0; i<size; i++)
       {
        cout<<i;
        for(int j=0; j<size; j++)
        {
           if(hasEdge(i, j))
           {
             cout<<"\t"<<1;
           }
           else cout<<"\t"<<0;
        }
        cout<<endl;

       }
       cout<<endl;
       cout<<old_c<<" "<<mCredit<<" best_c "<<best_index<<" "<<best_c<<" split_credit"<<splitCredit(best_index)<<endl;
       
        int l=mRandomList->size();
       for(int i=0, in=best_index; i<l; i++, in/=2)
      {
        cout<<i<<":"<<(in%2)<<" "<<endl;   
       }
      
       cout<<(rev->getCredit()+mCredit)<<"credit"<<endl;
      exit(0);
     }
  }
  else 
  {
    mSplitAltered=false;
  } 
  destroyMatrix();
  delete mRandomList;
  return rev;
}

	
void
Cluster::createRandomList()
{
  mVertices->permute();/***************this changes the index order of vertices**********/
  int random_list_size=_RANDOM_LIST_THRES_;
  if(mVertices->getSize()< random_list_size)
  {
    random_list_size=mVertices->getSize();
  }
  mRandomList=new vector<ClusterVertex* >(random_list_size);
  for(int i=0; i<random_list_size; i++)
  {
     (*mRandomList)[i]=mVertices->getData(i); 
  }//for
}

double
Cluster::badCredit(int remove_list, int& num_node)
{
   int size=mRandomList->size();
   int size2=mVertices->getSize();
   double num_edges_removed=0;
   int num_node_removed=0;
   for(int i=0, remove=remove_list; i<size; i++, remove/=2)
   {
      if(remove%2==0) continue;
      for(int j=i+1, remove2=remove/2; j<size; j++, remove2/=2)
      {
	 if(remove2%2==0) continue;
	 if(hasEdge((*mRandomList)[i]->getIndex(), (*mRandomList)[j]->getIndex()))
	   num_edges_removed+=1;  	 
      }
   }
   
   for(int i=0, remove=remove_list; i<size; i++, remove/=2)
   {
      if(remove%2==0) continue;
      for(int j=0, remove2=remove_list; j<size; j++, remove2/=2)
      {
	 if(remove2%2!=0) continue;
	 if(hasEdge((*mRandomList)[i]->getIndex(), (*mRandomList)[j]->getIndex()))
	   num_edges_removed+=1;  	 
      }
   
      for(int j=size; j<size2; j++)
      {
       	 if(hasEdge((*mRandomList)[i]->getIndex(), mVertices->getData(j)->getIndex()))
	   num_edges_removed+=1;  	 
      }
      num_node_removed+=1;
   }//for
   num_node=num_node_removed;
   double delta_credit=-(ClusterGraph::mAlpha+1)*(num_edges_removed)+0.5*num_node_removed*(2*size2-1-num_node_removed);
   return delta_credit;
}
double
Cluster::splitCredit(int split_list)
{
   int size=mRandomList->size();
   int size2=mVertices->getSize();
   double num_edges_removed=0;
   int num_edges_to_the_other_cluster=0;
   double delta_v=0;

   for(int i=0, remove=split_list; i<size; i++, remove/=2)
   {
      if(remove%2==0) continue;
      delta_v+=1;
   }//for

    for(int i=0, remove=split_list; i<size; i++, remove/=2)
    {
      if(remove%2==0) continue;
      for(int j=0, remove2=split_list; j<size; j++, remove2/=2)
      {
	 if(remove2%2!=0 || i==j) continue; 
	 if(hasEdge((*mRandomList)[i]->getIndex(), (*mRandomList)[j]->getIndex()))
	   num_edges_removed+=1;
      }
      for(int j=size; j<size2; j++)
      {
       	 if(hasEdge((*mRandomList)[i]->getIndex(), mVertices->getData(j)->getIndex()))
	   num_edges_removed+=1;  	 
      }
   }//for
   double delta_credit=-(ClusterGraph::mAlpha+1)*(num_edges_removed)
	               +delta_v*(size2-delta_v);
   return delta_credit; 

}
void
Cluster::print()
{
    int size=getSize();
    vector<ClusterVertex* >* vers=mVertices->getData();
    cout<<"This cluster has vertices:"<<endl;
    for(int i=0; i<size; i++)
    {
        cout<<"    vertex_id"<<((*vers)[i])->getID()<<endl;
        vector<AdjacentElement *>* adjs=((*vers)[i])->getOutList()->getCopy();
	for(int j=0; j<adjs->size(); j++)
	{
	   (*adjs)[j]->printNeighbor();
	}
	delete adjs;
    }
   
   cout<<endl;

}


void
Cluster::depthFirstSearch(ClusterVertex * v, vector<ClusterVertex *>* visited)
{
    if(v->mLocalVisited) return;
    visited->push_back(v);

    v->mLocalVisited=true;
    AdjacentElement* ae=v->getInList();
    vector<EdgeEnd* >* ends=(ae->getEdges())->getData();
    int size=ae->getSize();       
    for(int i=0; i<size; i++)
    {
       EdgeEnd* end=(*ends)[i];
       ClusterEdge* edge=end->getEdge(); 
       if(edge->getSource()==end)
       {
          depthFirstSearch((edge->getTarget())->getVertex(), visited);       
       }    
       else 
       {
         depthFirstSearch((edge->getSource())->getVertex(), visited);
       }       
    }//for 
}


void
Cluster::partitionSingleLinkedClusters(vector<Cluster* >* new_clusters, double alpha)
{ 
    resetEdgeCycleLabel();
    
    vector<ClusterVertex *>* vertices=mVertices->getData();
    int size=mVertices->getSize();
    for(int i=0; i<size; i++)
    {
      if(!((*vertices)[i])->mLocalVisited)
      {
        depthFirstForNonCycleEdges(NULL, (*vertices)[i], 0);
      }
    } 
    resetVertexLocalLabel();  
    
    vector<ClusterVertex *>* visited_list=new vector<ClusterVertex* >(size);
   for(int i=0; i<size; i++)
   {
     (*visited_list)[i]=(*vertices)[i];
   }//for
   int count=0; 
   for(int i=0; i<size; i++)
   {
      if(!((*visited_list)[i])->mLocalVisited)
      {
	 vector<ClusterVertex *>* cluster=new vector<ClusterVertex* >();
	 int num_vertex=0;
	 int num_degree=0;
	 int num_neighbors=0;
         depthSearchForSubCluster((*visited_list)[i], num_vertex, num_degree, num_neighbors, cluster);
	 count+=num_vertex;
	 //cout<<num_vertex<<" haha "<<num_degree<<" haha "<<num_neighbors<<endl;
	 if(num_vertex<=1)
	 {
	   delete cluster; 
	 }
	 else
	 {
	    int removed_edges=num_degree/2;
	    double new_credit1=(alpha+1)*(removed_edges)-(num_vertex)*(num_vertex-1)/2;
	    int c_size=getSize();
	    double the_other_part=mCredit-(alpha+1)*(removed_edges+num_neighbors)
	                          +c_size*(c_size-1)/2-(c_size-num_vertex)*(c_size-num_vertex-1)/2;
	    //cout<<(new_credit1+the_other_part)<<" "<<mCredit<<endl;
	    if(new_credit1+the_other_part>mCredit)
	    {
	       Cluster* c=new Cluster();
	       int new_size=cluster->size();
	       for(int j=0; j<new_size; j++)
	       {
	          moveTo((*cluster)[j], c, alpha); 
	       }
	       new_clusters->push_back(c);
	    }//if
	    delete cluster; 
	 }//
      }
    }
    if(count!=getSize())
    {
       //cout<<"hahaha error"<<endl;
    }
    delete visited_list;
}


void
Cluster::resetEdgeCycleLabel()
{
   vector<ClusterVertex *>* vertices=mVertices->getData();
   int size=mVertices->getSize();
   for(int i=0; i<size; i++)
   {
      AdjacentElement*  in=((*vertices)[i])->getInList();
      ((*vertices)[i])->mLocalVisited=false;

      ArrayList<EdgeEnd*>* ends=in->getEdges();
      vector<EdgeEnd *>*     ends_v=ends->getData();
      int  size2=ends->getSize();
      for(int j=0; j<size2; j++)
      {
         ((*ends_v)[j])->getEdge()->mIfInCycle=false;
      }
   } //for
}

void
Cluster::resetVertexLocalLabel()
{
   vector<ClusterVertex *>* vertices=mVertices->getData();
   int size=mVertices->getSize();
   for(int i=0; i<size; i++)
   {
      AdjacentElement*  in=((*vertices)[i])->getInList();
      ((*vertices)[i])->mLocalVisited=false;

   } //for
}


int
Cluster::depthFirstForNonCycleEdges(ClusterVertex* p, ClusterVertex* v, int depth)
{
    if(v->mLocalVisited)
    {
      return v->mDepth;
    }
    v->mLocalVisited=true;	
    v->mDepth=depth;  

    AdjacentElement* ae=v->getInList();
    vector<EdgeEnd* >* ends=(ae->getEdges())->getData();

    int size=ae->getSize();
    int parent=-1;    
     
    for(int i=0; i<size; i++)
    {
       EdgeEnd* end=(*ends)[i]->getAdjacent();
       ClusterVertex* cv=end->getVertex();
       if(cv==p) continue; //parent
       double parent_id=depthFirstForNonCycleEdges(v, cv, depth+1);
       if(parent==-1 || parent>parent_id)
       {
	 parent=(int)parent_id;
       }
       if(parent_id>depth || parent_id==-1)
       {
          end->getEdge()->mIfInCycle=false;
       }
       else
       {
          end->getEdge()->mIfInCycle=true;
       }
    }//for
    return parent;
}
void
Cluster::depthSearchForSubCluster(ClusterVertex* v, int& num_vertex, int& num_degree, int& num_neighbors, vector<ClusterVertex* >* cluster)
{
   if(v->mLocalVisited)
   {
      return;
   } 
   num_vertex+=1;
   cluster->push_back(v);
   v->mLocalVisited=true;	

   AdjacentElement* ae=v->getInList();
   vector<EdgeEnd* >* ends=(ae->getEdges())->getData();

   int size=ae->getSize();
   for(int i=0; i<size; i++)
   {
       EdgeEnd* end=(*ends)[i]->getAdjacent();
       ClusterVertex* cv=end->getVertex();
       if(!(end->getEdge()->mIfInCycle)) 
       {
	 num_neighbors+=1; 
	 continue;
       }
       
       num_degree+=1;
       depthSearchForSubCluster(cv, num_vertex, num_degree, num_neighbors, cluster);
    }//for
    return;
}


void
Cluster::insert(ClusterVertex* v)
{
    mVertices->insert(v);  
}

void
Cluster::remove(ClusterVertex* v)
{
   mVertices->remove(v);
}

void
Cluster::insert(ClusterCounterElement* e)
{
   if(e->getCluster()==this)
   {
     vector<int>* a=new vector<int>();
     delete a;
     (*a)[0]=1;
     (*a)[10]=1;
     cout<<"insertion errors at cluster::insert"<<endl;
   }
   mAdjacentClusters->split_insert(e);  
   //mAdjacentClusters->insert(e);  
   mAdjacentHash->insert(e->getCluster(), e);
}

void
Cluster::remove(ClusterCounterElement* e)
{
   mAdjacentClusters->split_remove(e);
   //mAdjacentClusters->remove(e);
   mAdjacentHash->remove(e->getCluster());
}

void
Cluster::changeCCECounter(ClusterCounterElement* e, int s)
{
  mAdjacentClusters->split_remove(e);
  e->changeCounter(s);  
  mAdjacentClusters->split_insert(e);
}

void
Cluster::changeScore(double delta)
{
   mCredit=mCredit+delta;
}


void
Cluster::mergeTo(Cluster* c, double alpha)
{
   int size=getSize();
   vector<ClusterVertex *>* vertices_copy=mVertices->getCopy(); 
   for(int i=0; i<size; i++)
   {
      moveTo((*vertices_copy)[i], c, alpha); 
   }
   delete vertices_copy;
   //cout<<getSize()<<endl;;
}

ClusterCounterElement*
Cluster::insertClusterCounterElement(Cluster* to, AdjacentElement* list)
{
   int size=list->getSize();
   if(size<=0) return NULL;
   int adjacent_cluster_size=mAdjacentClusters->getSize();
   /*vector<ClusterCounterElement *>* cce=mAdjacentClusters->getData();
   for(int i=0; i<adjacent_cluster_size; i++)
   {
      if((*cce)[i]->getCluster()==to)
      {
         (*cce)[i]->changeCounter(adjacent_cluster_size);
	 return (*cce)[i];
      }//
   }*/
   Pair<Cluster*, ClusterCounterElement *>* p=mAdjacentHash->find(to);
   if(p!=NULL)
   {
         changeCCECounter(p->second, adjacent_cluster_size);
	 return p->second;
   }//
    
   ClusterCounterElement* cc=new ClusterCounterElement(this, adjacent_cluster_size);
   cc->setCluster(to);
   //mAdjacentClusters->insert(cc);
   mAdjacentClusters->split_insert(cc);
   mAdjacentHash->insert(to, cc);
   return cc;
}


ClusterCounterElement*
Cluster::getAdjacentCluster(Cluster* c)
{
     Pair<Cluster*,ClusterCounterElement *>* p=mAdjacentHash->find(c);
     /*int adjacent_cluster_size=mAdjacentClusters->getSize();
     vector<ClusterCounterElement *>* cce=mAdjacentClusters->getData();
     for(int i=0; i<adjacent_cluster_size; i++)
     {
       if((*cce)[i]->getCluster()==c)
       {
	  if(p->second!=(*cce)[i]) cout<<"error, please debug"<<endl;
	   return (*cce)[i];
       }
     }*/
     if(p==NULL) return NULL;
     return p->second;
}


vector<vector<ClusterVertex* > * >* 
Cluster::partitionVertexAccordingToNeighbor()
{
   int size=mAdjacentClusters->getSize();
   vector<vector<ClusterVertex* > * >* rev=new vector<vector<ClusterVertex* > * >(size);
   for(int i=0; i<size; i++)
   {
      (*rev)[i]=new vector<ClusterVertex* >(); 
   }
   vector<ClusterVertex *>* cvs=mVertices->getCopy();
   int s=mVertices->getSize();
   for(int i=0; i<s; i++)
   {
      ClusterVertex* cv=(*cvs)[i];
      ArrayList<AdjacentElement *>* ae=cv->getOutList();
      vector<AdjacentElement* >* ae_v=ae->getCopy();
      int s2=ae->getSize();
      for(int j=0; j<s2; j++)
      {
         int which=(*ae_v)[j]->getClusterCounterElement()->getIndex();
	 ((*rev)[which])->push_back(cv);
      }//for
      delete ae_v;
   }//for
   delete cvs;
   return rev;
}

vector<Cluster* >* 
Cluster::getClusterCopy()
{
   int size=mAdjacentClusters->getSize();
   vector<Cluster* >* rev=new vector<Cluster* > (size);
   for(int i=0; i<size; i++)
   {
      (*rev)[i]=mAdjacentClusters->getData(i)->getCluster(); 
   }//
   return rev;

}

void
Cluster::createMatrix(vector<ClusterVertex *>* vertices, vector<AdjacentElement* >* incoming)
{
   int size=getSize();
   int size2;
   if(vertices==NULL)
     size2=0;
   else size2=vertices->size();

   int s=(size+size2)*(size+size2-1)/2;
   mAdjMatrix=new bool[s];
   for(int i=0; i<s; i++)
   {
      mAdjMatrix[i]=false;
   }

   for(int i=0; i<size; i++)
   {
     ClusterVertex* v=mVertices->getData(i);
     AdjacentElement* in=v->getInList();
     ArrayList<EdgeEnd *>* ends=in->getEdges();
     int ends_size=ends->getSize(); 
     for(int j=0; j<ends_size; j++)
     {
        ClusterVertex* v=ends->getData(j)->getAdjacentVertex();
	int index2=v->getIndex();
	if(index2>i)
	{
           int which=index2*(index2-1)/2+i;
     	   mAdjMatrix[which]=true; 
	}
     }
   }//for
   if(size2==0) return;
   for(int i=0; i<size2; i++)
   {
      AdjacentElement* ae=(*incoming)[i];
      vector<EdgeEnd* >* ee=ae->getEdges()->getData();
      int num_ee=ae->getEdges()->getSize();
      for(int j=0; j<num_ee; j++)
      {
         ClusterVertex* v2=(*ee)[j]->getAdjacentVertex(); 
         int index=v2->getIndex();
         int which=(size+i)*(size+i-1)/2+index;
         mAdjMatrix[which]=true;	 
      }
   } 
}
void
Cluster::destroyMatrix()
{
   if(mAdjMatrix) delete [] mAdjMatrix;
   if(mDynTable) delete [] mDynTable;
   if(mTracePointers) delete [] mTracePointers;
   if(mDynamicList) delete mDynamicList;
   if(mDynOutList) delete mDynOutList;
   mAdjMatrix=NULL;
   mDynTable=NULL;
   mTracePointers=NULL;
   mDynamicList=NULL;
   mDynOutList=NULL;
}

bool
Cluster::hasEdge(int i, int j)
{
   if(i==j) return false;
   if(i<j)
    return mAdjMatrix[j*(j-1)/2+i];
   return mAdjMatrix[i*(i-1)/2+j];
}

int 
Cluster::numEdges(vector<ClusterVertex* >* cvs)
{
   int rev=0;
   int size=cvs->size();
   for(int i=0; i<size; i++)
   {
      for(int j=i+1; j<size; j++)
      {
         if(hasEdge((*cvs)[i], (*cvs)[j]))
		 rev+=1;
      }
   }//for
   return rev;
}

int 
Cluster::inDegrees(vector<ClusterVertex* >* cvs)
{
   int rev=0;
   int size=cvs->size();
   for(int i=0; i<size; i++)
   {
      rev+=(*cvs)[i]->getInList()->getSize();
   }//for
   return rev;
}
bool 
Cluster::hasEdge(ClusterVertex* v1, ClusterVertex* v2)
{
  return hasEdge(v1->getIndex(), v2->getIndex());
}


bool
Cluster::hasAdditionEdge(int index, int vector_index)
{
   int s=getSize();
   int which=(s+vector_index)*(s+vector_index-1)/2+index;
   return mAdjMatrix[which]; 
}

void
Cluster::retrieveDynamicList(Cluster *c)
{
  mDynamicList=new vector<ClusterVertex *>();
  mDynOutList =new vector<AdjacentElement *>();
  int size=getSize();
  mVertices->permute();
  for(int i=0; i<size; i++)
  {
    ClusterVertex* v=mVertices->getData(i);
    AdjacentElement* ae=v->getOutList(c);
    if(ae!=NULL)
    {
       mDynamicList->push_back(v);
       mDynOutList->push_back(ae);
    }
    if(mDynamicList->size()>=_NEIGHBOR_THRES_) break;
  }
  
  //cout<<mDynamicList->size()<<"===="<<endl;
  if(mDynamicList->size()==0)
  {
 //    cout<<"get bugs, please debug"<<endl;
  //   exit(0);
  }
}

void
Cluster::fillupDynamicTable(Cluster* prevC)
{

   if(prevC==NULL)
     	createMatrix(NULL, NULL);
   else createMatrix(prevC->getDynamicList(), prevC->getDynOutList());
	
	
   int size;
   if(mDynamicList==NULL)
	 size=0;
   else size=mDynamicList->size();
   int table_size=(1<<size);
   if(table_size>10000)
   {
      cerr<<"Table size is too large, u may consider not be use it"<<endl;
   }
   mTracePointers=new int[table_size];
   mDynTable     =new double[table_size];
   for(int i=0; i<table_size; i++)
   {
     mDynTable[i]=-100000000;
     mTracePointers[i]=0;
   }
   fillupDynamicTable(0, size, 0, 0, 0, prevC);
}

//remove_index is treated as a binary, the length of the binary equals to the length of the mDynamicList
//each bit codes if the corresponding elements is removed from the current cluster or not. 
void
Cluster::fillupDynamicTable(int depth, int size, int removed_index, int num_ver_removed, int num_edge_removed,Cluster* prevC)
{
    if(depth==size) 
  {
     fillupTablePrevIter(removed_index, num_ver_removed, num_edge_removed, prevC);  
     return;
  }

  fillupDynamicTable(depth+1, size, removed_index*2, num_ver_removed, num_edge_removed, prevC);
  int to_removed=(*mDynamicList)[depth]->getInList()->getSize();
  fillupDynamicTable(depth+1, size, removed_index*2+1, num_ver_removed-1, num_edge_removed-to_removed, prevC); 
}

void
Cluster::fillupTablePrevIter(int removed_index, int num_ver_removed, int num_edge_removed, Cluster* prev)
{
   if(prev==NULL)
   {
      computeTheCost(removed_index, 0, num_ver_removed, num_edge_removed, NULL);
      return;
   }
        
   vector<ClusterVertex *>* vertices=prev->getDynamicList();
   int size=vertices->size();
   //cout<<size<<" ha ha "<<endl;
   fillupTablePrevIter(removed_index, 0, num_ver_removed, num_edge_removed,  0, size, prev);
}

void
Cluster::fillupTablePrevIter(int removed_index, int inserted_index, 
		             int num_ver_changed, int num_edge_changed, 
			     int depth, int size, Cluster* prev)
{
    if(depth==size)
    {
        computeTheCost(removed_index, inserted_index, num_ver_changed, num_edge_changed, prev);                  
	return;
    }
    fillupTablePrevIter(removed_index, inserted_index*2,     num_ver_changed, num_edge_changed, depth+1, size, prev);
    
    fillupTablePrevIter(removed_index, inserted_index*2+1,   
		    num_ver_changed+1, num_edge_changed+prev->getDynOutListSize(depth), depth+1, size, prev); 
}


int
Cluster::edgesCodeByIndex(int index0)
{
     int rev=0;	
     if(mDynamicList==NULL) return 0;
      
     for(int list_of_node=index0, index=mDynamicList->size()-1; list_of_node>0; list_of_node/=2, index--)
     {
       if(list_of_node%2!=1) continue;
       
       for(int list_of_node2=list_of_node/2, index2=index-1; list_of_node2>0; list_of_node2/=2, index2--)
       {
          if(list_of_node2%2!=1) continue;
	   
	  if(hasEdge((*mDynamicList)[index]->getIndex(), (*mDynamicList)[index2]->getIndex()))
		     rev+=1; 
       }
     }
     return rev;
} 

int 
Cluster::edgesCodedByTwoIndex(int inserted_index, int removed_index, int size_add, int size_remove)
{
  
    int rev=0;       
    for(int index1=removed_index, which1=size_remove-1; index1!=0; index1/=2, which1--)
    {
      if(index1%2!=1) continue;
      for(int index2=inserted_index, which2=size_add-1; index2!=0; index2/=2, which2--)
      {
	   if(index2%2!=1) continue;
	   
	   if(hasAdditionEdge((*mDynamicList)[which1]->getIndex(),  which2))
	      rev+=1;
      }
    }
   return rev; 
}

double
Cluster::getCredit(int current_index, int prev_index, int delta_v, int delta_e, Cluster* prevC)
{

    int current_changed;
    int prev_changed;
    int co_changed;

    if(current_index==0)
    {
       current_changed=0;
    }
    else
    {
      current_changed=edgesCodeByIndex(current_index);
    }
    if(prev_index==0)
    {
       prev_changed=0;
    }
    else
    {
      prev_changed=prevC->edgesCodeByIndex(prev_index);
    }
    if(current_index==0 || prev_index==0)
    {
       co_changed=0;
    }
    else
    {
       co_changed=edgesCodedByTwoIndex(prev_index, current_index, prevC->getDynamicList()->size(), getDynamicList()->size());
    }
    delta_e=delta_e+current_changed+prev_changed-co_changed;
    
    int n=getSize();
    double* prev_table=NULL;
    if(prevC) prev_table=prevC->getDynTable();
    double delta_credit=(ClusterGraph::mAlpha+1)*(delta_e)-0.5*delta_v*(2*n-1+delta_v);
    return delta_credit;
}
void
Cluster::computeTheCost(int current_index, int prev_index, int delta_v, int delta_e, Cluster* prevC)
{
    double accumulated_credit=getCredit(current_index, prev_index, delta_v, delta_e, prevC);
    double* prev_table=NULL;
    if(prevC) prev_table=prevC->getDynTable();
    if(prevC!=NULL && mDynamicList!=NULL)
    {
       accumulated_credit+=prev_table[prev_index];
    
       if(accumulated_credit>mDynTable[current_index])
      {
        mDynTable[current_index]=accumulated_credit;
        mTracePointers[current_index]=prev_index;
      }
    }
    else if(mDynamicList!=NULL)
    {
       mDynTable[current_index]=accumulated_credit;
    }
    else if(prevC!=NULL)
    {
       accumulated_credit+=prev_table[prev_index];
       prev_table[prev_index]=accumulated_credit;
    }
}



int
Cluster::extractTheBestMovement(Cluster* nextC)
{
   int size=mDynamicList->size(); 
   int size_all=1<<(size);
   double max_credit=0; 
   int max_index=-1; 
   for(int i=0; i<size_all; i++)
   {
      if(mDynTable[i]>=max_credit)
      {
         max_credit=mDynTable[i];
	 max_index=i;
      }
   }

   if(max_index<0) return -1;
   int index=max_index;
   int which=size-1;
   while(index!=0)
   {
     if(index%2==1)
     {
       moveTo((*mDynamicList)[which], nextC, ClusterGraph::mAlpha);
     }   
     which--;
     index/=2;
   }  
   return mTracePointers[max_index];    
}
int 
Cluster::extractTheBestMovement(int max_index, Cluster* nextC)
{
   if(max_index<0) return -1;
   int index=max_index;
   int which=mDynamicList->size()-1;
   //cout<<index<<" "<<which<<endl;
   while(index!=0)
   {
     if(index%2==1)
     {
       moveTo((*mDynamicList)[which], nextC, ClusterGraph::mAlpha);
     }   
     which--;
     index/=2;
   }  
   return mTracePointers[max_index];    
}

void
Cluster::createCycleMat(Cluster* prevC, int first_size, bool last_to_first)
{

   if(prevC==NULL)
     createMatrix(NULL, NULL);
   else createMatrix(prevC->getDynamicList(), prevC->getDynOutList());
}

void
Cluster::fillupCycleTable(Cluster* prevC, int first_size, bool last_to_first)
{

   int size;
   if(mDynamicList==NULL)
	 size=0;
   else size=mDynamicList->size();
   int table_size=(1<<size);
   int first_=(1<<first_size);
   if(table_size*first_>100000)
   {
      cerr<<"Table size is too large, u may consider not be use it"<<endl;
   }
   mTracePointers=new int[table_size*first_];
   mDynTable     =new double[table_size*first_];
   for(int i=0; i<table_size*first_; i++)
   {
     mDynTable[i]=-1000000;
   }
  fillupDynamicTable(0, size, 0, 0, 0, prevC, first_size, last_to_first);
}

//remove_index is treated as a binary, the length of the binary equals to the length of the mDynamicList
//each bit codes if the corresponding elements is removed from the current cluster or not. 
void
Cluster::fillupDynamicTable(int depth, int size, int removed_index,
		int num_ver_removed, int num_edge_removed,Cluster* prevC, int first_size, bool last_to_first)
{
    if(depth==size) 
  {
     fillupTablePrevIter(removed_index, num_ver_removed, num_edge_removed, prevC, first_size, last_to_first);  
     return;
  }

  fillupDynamicTable(depth+1, size, removed_index*2, num_ver_removed, num_edge_removed, 
		  prevC, first_size, last_to_first);
  int to_removed=(*mDynamicList)[depth]->getInList()->getSize();
  fillupDynamicTable(depth+1, size, removed_index*2+1, num_ver_removed-1, num_edge_removed-to_removed, 
		  prevC, first_size, last_to_first); 
}

void
Cluster::fillupTablePrevIter(int removed_index, int num_ver_removed, int num_edge_removed, 
		Cluster* prev, int first_size, bool last_to_first)
{
   if(prev==NULL)
   {
      cycleCost(removed_index, 0, num_ver_removed, num_edge_removed, NULL, first_size, last_to_first);
      return;
   }
        
   vector<ClusterVertex *>* vertices=prev->getDynamicList();
   int size=vertices->size();
   //cout<<size<<" ha ha "<<endl;
   fillupTablePrevIter(removed_index, 0, num_ver_removed, num_edge_removed,  0, size, prev, first_size, last_to_first);
}

void
Cluster::fillupTablePrevIter(int removed_index, int inserted_index, 
		             int num_ver_changed, int num_edge_changed, 
			     int depth, int size, Cluster* prev, int first_size, bool last_to_first)
{
    if(depth==size)
    {
        cycleCost(removed_index, inserted_index, num_ver_changed, num_edge_changed, prev, first_size, last_to_first);                  
	return;
    }
    fillupTablePrevIter(removed_index, inserted_index*2,num_ver_changed, num_edge_changed, depth+1, size, prev, 
		    first_size, last_to_first);
    
    fillupTablePrevIter(removed_index, inserted_index*2+1,   
		    num_ver_changed+1, num_edge_changed+prev->getDynOutListSize(depth), depth+1, size, prev, first_size,
		    last_to_first); 
}


void
Cluster::cycleCost(int curr_index, int prev_index, int delta_v, int delta_e, Cluster* prevC,
   int first_index_size, bool last_to_first)
{ 
   double d=getCredit(curr_index, prev_index, delta_v, delta_e, prevC);
   int size=mDynamicList->size();
   int size_all=(1<<size)*(1<<first_index_size);
   double* prev_table=prevC->getDynTable();
   int max_first_index=(1<<first_index_size);
   int prev_prefix=(prev_index<<first_index_size);
   int curr_prefix=(curr_index<<first_index_size);
   
   if(prev_table==NULL)//at cluster 1
   {
      mDynTable[curr_prefix+prev_index]=d;
      mTracePointers[curr_prefix+prev_index]=curr_prefix+prev_index;
      return;
   }
   else if(last_to_first)
   {
      int current_size=mDynamicList->size();
      int prev_table_index=(prev_index<<current_size)+curr_index; 
      prev_table[prev_table_index]=prev_table[prev_table_index]+d;
      return; 
   }
   else
   {
     for(int i=0; i<max_first_index; i++)
     {
       if(prev_table[prev_prefix+i]+d>=mDynTable[curr_prefix+i])
       {
        mDynTable[curr_prefix+i]=d+prev_table[prev_prefix+i];
	mTracePointers[curr_prefix+i]=prev_prefix+i;
       }    
      }
   }
}


double
Cluster::getCycleCredit(Cluster* prevC)
{
     int size=mDynamicList->size();
     int first_size=prevC->getDynamicList()->size();
     int size_all=(1<<(size))*(1<<first_size);
     double max_credit=0;
     int max_index=-1;
     double* prevTable=prevC->getDynTable();
     for(int i=0; i<size_all; i++)
     {
       if(prevTable[i]>=max_credit)
       {
         max_credit=prevTable[i];
	 max_index=i;
       }
     }
    return max_credit;

}

int
Cluster::extractTheBestMovement(Cluster* nextC, Cluster* prevC)
{
   int size=mDynamicList->size(); 
   int first_size=prevC->getDynamicList()->size();

   int size_all=(1<<(size))*(1<<first_size);
   double max_credit=0; 
   int max_index=-1; 
   double* prevTable=prevC->getDynTable();

   for(int i=0; i<size_all; i++)
   {
      if(prevTable[i]>=max_credit)
      {
         max_credit=prevTable[i];
	 max_index=i;
      }
   }
   //if(max_credit>0) cout<<"The credit is increased by "<<max_credit<<endl;
  // if(max_index%(1<<size)!=0) print();
   if(max_index<0) return -1;
   int first_index=max_index%(1<<size);
   int secon_index=(max_index)>>size;
   int index=first_index;
   int which=size-1;
   while(index!=0)
   {
     if(index%2==1)
     {
       moveTo((*mDynamicList)[which], nextC, ClusterGraph::mAlpha);
     }   
     which--;
     index/=2;
   } 
   return max_index;    
}
int 
Cluster::extractTheBestMovement(int max_index, Cluster* nextC, int first_size)
{
   if(max_index<0) return -1;
   int index=(max_index)/(1<<first_size);
   //if(index!=0) print();
   int which=mDynamicList->size()-1;
   while(index!=0)
   {
     if(index%2==1)
     {
       moveTo((*mDynamicList)[which], nextC, ClusterGraph::mAlpha);
     }   
     which--;
     index/=2;
   }  
   return mTracePointers[max_index];    
}



bool
Cluster::consistencyCheck()
{
   vector<ClusterVertex* >* vertices=mVertices->getCopy();
   int size=vertices->size();
   //first check the score;
   double score=0;
   for(int i=0; i<size; i++)   
   {
     score+=((*vertices)[i])->getInList()->getSize(); 
   }
   score=score*(ClusterGraph::mAlpha+1)/2-(size*(size-1)/2);
   if(score!=mCredit)
   {
     cerr<<"WARNING: inconsistency, please check"<<endl;
   }
  //then check the counter counter elements  
   vector<ClusterCounterElement* >* cces=mAdjacentClusters->getCopy();
   for(int i =0; i<cces->size(); i++)
   {
      int counter=(*cces)[i]->getCounter();
       Cluster* c=(*cces)[i]->getCluster();
       int c2=0;

       for(int j=0; j<vertices->size(); j++)
       {
          AdjacentElement* ae=(*vertices)[j]->getOutList(c);       
          if(ae) c2+=ae->getSize(); 
       }
       if(c2!=counter)
       {
          cerr<<"WARNING: inconsistence, please check at cluster counters"<<endl;
          exit(0);
       }
  }
  
   for(int j=0; j<vertices->size(); j++)
   {
      ArrayList<AdjacentElement *>* ae=(*vertices)[j]->getOutList();       
       if(ae->getSize()!=0 && mAdjacentClusters->getSize()==0)
       {
         cerr<<"WARNING: inconsistence, please check at cluster counters"<<endl;
          exit(0);
       } 
   }
  

  delete vertices;
  delete cces;
}

vector<int>*
Cluster::getVertexNumbers()
{
  vector<int>* rev=new vector<int>();
  int s=mVertices->getSize();

  for(int i=0; i<s; i++)
  {
    rev->push_back(mVertices->getData(i)->getVertex()->getNumber());
  }


  return rev;

}


