#ifndef _CLUSTER_H_
#define _CLUSTER_H_
#include "ClusterBase.h"

class Cluster
{
   protected:
      ArrayList<ClusterVertex *>*  mVertices;
      ArrayList<ClusterCounterElement *>* mAdjacentClusters;
      SimpleHash<Cluster*, ClusterCounterElement* >* mAdjacentHash;
      vector<ClusterVertex* >* mRandomList; 
      double mCredit; 
      void removeInEdges(AdjacentElement* adj, Cluster* to);      
      void insertInEdges(ArrayList<EdgeEnd* >* ends);
      void depthFirstSearch(ClusterVertex * v, vector<ClusterVertex *>* visited);
      void adjustOutEdges(ClusterVertex* v, Cluster* c);
      int mIndex;
      void resetEdgeCycleLabel();
      void resetVertexLocalLabel();
      int depthFirstForNonCycleEdges(ClusterVertex* p, ClusterVertex* v, int depth);
      void depthSearchForSubCluster(ClusterVertex* v, 
		      int& num_vertex, int& num_degree, 
		      int& num_neighbors, 
		      vector<ClusterVertex* >* cluster);

      ClusterCounterElement* insertClusterCounterElement(Cluster* to, AdjacentElement* list);
     
      //methods 
      bool hasEdge(int i, int j);
      bool hasAdditionEdge(int index, int vector_index);
      vector<ClusterVertex *>* mDynamicList;
      vector<AdjacentElement *>* mDynOutList;
      double getCredit(int current_index, int prev_index, int delta_v, int delta_e, Cluster* prevC);
      double* mDynTable;
      bool* mAdjMatrix;
      int* mTracePointers;
     void fillupDynamicTable(int depth, int size, int removed_index, 
		    int num_ver_removed, int num_edge_removed,Cluster* prevC);
      void fillupTablePrevIter(int removed_index, int num_ver_removed, 
		      int num_edge_removed, Cluster* prev);
      
      void fillupTablePrevIter(int removed_index, int inserted_index, 
		             int num_ver_changed, int num_edge_changed, 
			     int depth, int size, Cluster* prev);
      int edgesCodeByIndex(int index);

      int edgesCodedByTwoIndex(int removed_index, int inserted_index, 
		      int size_remove, int size_add);
      void computeTheCost(int current_index, int prev_index, 
		      int delta_v, int delta_e, Cluster* prevC);
      void createRandomList();
      void exchangeWithNeighbor();
      double badCredit(int remove_list, int & num_node);
      double splitCredit(int split_list);
   public: 
    Cluster();
    Cluster(vector<ClusterVertex* >* vertices, int size);
    void mergeCluster(Cluster* c, double alpha);    
    ~Cluster();
    int getNumVertices();
    int getSize() {return mVertices->getSize(); }
    double getCredit() { return mCredit; }
    void changeScore(double score);
    void insert(ClusterVertex* c);
    void remove(ClusterVertex* c);
    void insert(ClusterCounterElement* c);
    void remove(ClusterCounterElement* c);
    void changeCCECounter(ClusterCounterElement* c, int s);
    vector<int>* getVertexNumbers();
    vector<vector<ClusterVertex* > * >* partitionVertexAccordingToNeighbor();
    vector<Cluster* >* getClusterCopy();
    void moveTo(ClusterVertex* v, Cluster* c, double alpha);
    void insertVertex(ClusterVertex* v);
    void partitionSingleLinkedClusters(vector<Cluster* >* new_clusters, double alpha);  
    void print();
    ArrayList<ClusterVertex *>* getVertices() { return mVertices; }
    vector<vector<ClusterVertex *> *>* partition();
    void setIndex(int i)  { mIndex=i; }
    int  getIndex()  { return mIndex; }
    ArrayList<ClusterCounterElement *>* getAdjacentClusters() { return mAdjacentClusters; }
    void mergeTo(Cluster* c, double alpha);
    ClusterCounterElement* getAdjacentCluster(Cluster* c); 
    bool mVisit;
    //methods for dynamic programming
    void fillupDynamicTable(Cluster* prevC);
    void retrieveDynamicList(Cluster *c);
    void createMatrix(vector<ClusterVertex* >* v, vector<AdjacentElement* >* adj);
    void destroyMatrix();
    int extractTheBestMovement(Cluster* nextC);
    int extractTheBestMovement(int max_index, Cluster* nextC);
    vector<ClusterVertex *>*   getDynamicList() { return mDynamicList; }
    vector<AdjacentElement *>* getDynOutList() { return mDynOutList; }
    int getDynOutListSize(int i) { return ((*mDynOutList)[i])->getEdges()->getSize(); }
    double* getDynTable() { return mDynTable; }
    int numEdges(vector<ClusterVertex* >* cvs);
    int inDegrees(vector<ClusterVertex* >* cvs);
    bool hasEdge(ClusterVertex* v1, ClusterVertex* v2);

    void fillupCycleTable(Cluster* prevC, int first_size, bool last_to_first);
    void createCycleMat(Cluster* prevC, int first_size, bool last_to_first);
    void fillupDynamicTable(int depth, int size, int removed_index,
		int num_ver_removed, int num_edge_removed,Cluster* prevC, int first_size, bool first_to_l);
    void fillupTablePrevIter(int removed_index, int num_ver_removed, int num_edge_removed, 
		Cluster* prev, int first_size, bool last_to_first);
    void fillupTablePrevIter(int removed_index, int inserted_index, 
		             int num_ver_changed, int num_edge_changed, 
			     int depth, int size, Cluster* prev, int first_size, bool last_to_first);
    void cycleCost(int current_index, int prev_index, int delta_v, int delta_e, Cluster* prevC, int first_size, bool first_index_size);
    int extractTheBestMovement(Cluster* nextC, Cluster* prevC);
    int extractTheBestMovement(int max_index, Cluster* nextC, int first_size);
    double getCycleCredit(Cluster* prev);
    bool consistencyCheck();
    bool mRemoveBadAltered;
    bool mSplitAltered;
    bool mSwapAltered;
    bool mAltered;
    bool mMoveSingleAltered;  
    void removeBadVertices(Cluster* c0);
    Cluster* splitCluster();
};

#endif


