#include "ClusterBase.h"
#include "ArrayList.h"
#include "Cluster.h"
#include <iostream>
using namespace std;
//----------=Edge End=------------
//
//
EdgeEnd::EdgeEnd(ClusterVertex* v, AdjacentElement* a, ClusterEdge* e)
{ 
   //mCluster=c;
   mEdge=e;
   mAdjacentElement=a;
   mVertex=v; 
}

EdgeEnd::~EdgeEnd() 
{

}

ClusterEdge* 
EdgeEnd::getEdge()
{
   return mEdge;
}

AdjacentElement*
EdgeEnd::getAdjacentElement()
{
  return mAdjacentElement;
}

Cluster*
EdgeEnd::getCluster()
{
  return mAdjacentElement->getCluster();
}
ClusterVertex*
EdgeEnd::getVertex()
{
   return mVertex;
}


 
void
EdgeEnd::setEdge(ClusterEdge* e)
{
   mEdge=e;
}


void
EdgeEnd::setAdjacentElement(AdjacentElement* a)
{
   mAdjacentElement=a;
}

/*
void
EdgeEnd::setCluster(Cluster* c)
{
   mCluster=c;
}*/

void
EdgeEnd::setVertex(ClusterVertex* v)
{
   mVertex=v;
}

EdgeEnd*
EdgeEnd::getAdjacent()
{
   ClusterEdge* e=getEdge();
   if(e->getSource()==this)
   {
     return e->getTarget();
   }
   else
   {
     return e->getSource();
   }
}
ClusterVertex*
EdgeEnd::getAdjacentVertex()
{
  return getAdjacent()->getVertex();
}

void
EdgeEnd::print()
{
    

}

//===============================
//===============================
//
//
//
ClusterEdge::ClusterEdge(Edge* e, EdgeEnd* s, EdgeEnd* t)
{
  mEdge=e;
  mSource=s;
  mTarget=t;
}
ClusterEdge::~ClusterEdge()
{
   delete mSource;
   delete mTarget; 
}

void 
ClusterEdge::setSource(EdgeEnd* e)
{
   mSource=e;
}	

void 
ClusterEdge::setTarget(EdgeEnd* e)
{
   mTarget=e;
}

EdgeEnd* 
ClusterEdge::getSource()
{
   return mSource;
}

EdgeEnd* 
ClusterEdge::getTarget()
{
   return mTarget;
}
//


Edge* 
ClusterEdge::getEdge()
{
   return mEdge;
}
void 
ClusterEdge::setEdge(Edge* e)
{
  mEdge=e;
}
//================================
//================================
//============AdjacentElement======

AdjacentElement::AdjacentElement(ClusterVertex* v)
{
  mVertex=v;
  mEdges=new ArrayList<EdgeEnd *>();
  mCounter=NULL;
}

AdjacentElement::~AdjacentElement()
{
   delete mEdges; 
}

void
AdjacentElement::insert(EdgeEnd* & end)
{
   end->setAdjacentElement(this);
   mEdges->insert(end);
}

void
AdjacentElement::remove(EdgeEnd* &end)
{
    mEdges->remove(end);
}

ClusterVertex*
AdjacentElement::getVertex()
{
  return mVertex;
}

ArrayList<EdgeEnd *>* 
AdjacentElement::getEdges()
{
  return mEdges;
}
 int 
AdjacentElement::getSize() { return mEdges->getSize(); }

void
AdjacentElement::printNeighbor()
{
   int size=mEdges->getSize();
   vector<EdgeEnd*>* ends=mEdges->getData();
   for(int i=0; i<size; i++)
   {
      ClusterEdge* e=((*ends)[i])->getEdge();
      if(e->getTarget()==(*ends)[i])
      {
         cout<<e->getSource()->getVertex()->getID()<<":"<<((*ends)[i])->getIndex()<<" ";
      }
      else
      {
         cout<<e->getTarget()->getVertex()->getID()<<":"<<((*ends)[i])->getIndex()<<" ";
      }
   }
   cout<<endl;

}
//
//
//
//=================================
ClusterVertex::ClusterVertex(Vertex* v, int k)
{
   mVertex=v;
   mInClusterList=new AdjacentElement(this);
   mOutClusterList=new ArrayList<AdjacentElement *>();  
   mOutClusterHash=new SimpleHash<Cluster*, AdjacentElement *>(k);
   mAdjacentList=new SimpleHash<ClusterVertex*, int>(k);
   mAdjacentInTriangle=new vector<ClusterVertex* >();
   mVisited=false;
   mLocalVisited=false;
}

ClusterVertex::~ClusterVertex()
{
   delete mInClusterList;
   delete mOutClusterList;
   delete mOutClusterHash;
   delete mAdjacentList; 
}


AdjacentElement* 
ClusterVertex::getInList()
{
   return mInClusterList;
}

ArrayList<AdjacentElement * >* 
ClusterVertex::getOutList()
{
   return mOutClusterList;
}

AdjacentElement * 
ClusterVertex::getOutList(Cluster* c)
{    
   Pair<Cluster*, AdjacentElement *>* p=mOutClusterHash->find(c);
   vector<AdjacentElement* >* emts=mOutClusterList->getData();
   /*int size=mOutClusterList->getSize();
   for(int i=0; i<size; i++)
   {
       if((*emts)[i]->getCluster()==c)
       {
          if((*emts)[i]!=p->second)  cout<<"error, please debug"<<endl;
          return (*emts)[i];
       }
   }
   if(p!=NULL)  cout<<"error "<<endl; 
   return NULL;*/
   if(p!=NULL) return p->second;
   return NULL;
}  


bool
ClusterVertex::hasOutListTo(Cluster* c)
{    
   Pair<Cluster*, AdjacentElement *>* p=mOutClusterHash->find(c);
   /*vector<AdjacentElement* >* emts=mOutClusterList->getData();
   int size=mOutClusterList->getSize();
   for(int i=0; i<size; i++)
   {
       if((*emts)[i]->getCluster()==c)
       {
          return true;
       }
   } 
   return false;*/
   if(p==NULL) return false;
   return true;
} 

ClusterCounterElement*
ClusterVertex::removeInVertex(EdgeEnd* end)
{
   //cout<<"Remove  from "<<getID()<<endl;
   //cout.flush();
   mInClusterList->remove(end);
   return NULL;
}

ClusterCounterElement*
ClusterVertex::removeOutVertex(EdgeEnd* end)
{
   AdjacentElement* ae=end->getAdjacentElement();
   ae->remove(end); 
   ClusterCounterElement* cce=ae->getClusterCounterElement();
   if(ae->getSize()==0)
   {
      mOutClusterList->remove(ae);
      mOutClusterHash->remove(ae->getCluster());
      delete ae;
   }//if
   //cce->changeCounter(-1);
   if(cce->getCounter()==1)
   {
     Cluster* c=end->getVertex()->getCluster();
     c->remove(cce);
     delete cce;
     return NULL; 
   }
   Cluster* c=end->getVertex()->getCluster(); 
   c->changeCCECounter(cce, -1);
   return cce;
}

ClusterCounterElement*
ClusterVertex::insertInVertex(EdgeEnd* end)
{
   mInClusterList->insert(end);
   return NULL;
}

ClusterCounterElement*
ClusterVertex::insertOutVertex(EdgeEnd* end, Cluster* c)
{
   /*vector<AdjacentElement* >* adjs=mOutClusterList->getData();
   int size=mOutClusterList->getSize();
   bool found=false;
   for(int i=0; i<size; i++)
   {
     if((*adjs)[i]->getCluster()==c)
     {
       found=true;
       AdjacentElement* e=(*adjs)[i];
       e->insert(end);
       ClusterCounterElement* cce=e->getClusterCounterElement();
       cce->changeCounter(1);

       return cce; 
       break; 
     } 
   }//for*/
   Pair<Cluster*, AdjacentElement *>* p=mOutClusterHash->find(c);
   if(p!=NULL)
   {
       AdjacentElement* e=p->second;
       e->insert(end);
       ClusterCounterElement* cce=e->getClusterCounterElement();
       mCluster->changeCCECounter(cce, 1);
       return cce;

   }
   bool found=false;
   if(!found)
   {
       AdjacentElement* e=new AdjacentElement(this);
       e->setCluster(c);
       end->setAdjacentElement(e);
       e->insert(end);
       mOutClusterList->insert(e);
       mOutClusterHash->insert(c, e);
       /*vector<ClusterCounterElement *>* ccev=mCluster->getAdjacentClusters()->getData();
       int ad_c=mCluster->getAdjacentClusters()->getSize();     
       for(int i=0; i<ad_c; i++)
       {
          ClusterCounterElement* cce=(*ccev)[i];
          if(cce->getCluster()==c)
	  {
             e->setClusterCounterElement(cce);
	     cce->changeCounter(1);
	     return cce;
	  }
       }*/
       ClusterCounterElement* cce_=mCluster->getAdjacentCluster(c);
       if(cce_)
       {
         e->setClusterCounterElement(cce_);
	 mCluster->changeCCECounter(cce_, 1);
	 return cce_; 
       }
       ClusterCounterElement* cce=new ClusterCounterElement(c, 1);
       mCluster->insert(cce);
       e->setClusterCounterElement(cce);
       return cce;
   }
}

void 
ClusterVertex::setInList(AdjacentElement*    list)
{
   mInClusterList=list;
}
void
ClusterVertex::removeOutList(AdjacentElement*    list)
{
   mOutClusterList->remove(list);
   mOutClusterHash->remove(list->getCluster());
}
void 
ClusterVertex::addToOutList(AdjacentElement* list)
{
   mOutClusterList->insert(list);
   mOutClusterHash->insert(list->getCluster(), list);
   Cluster* c=mCluster;
   /*vector<ClusterCounterElement *>* ccev=mCluster->getAdjacentClusters()->getData();   
   int s=mCluster->getAdjacentClusters()->getSize();
   for(int i=0; i<s; i++)
   {
     ClusterCounterElement* cce=(*ccev)[i];
     if(cce->getCluster()==list->getCluster())
     {
        cce->changeCounter(list->getSize());
        list->setClusterCounterElement(cce);
	return;
     }
   }*/
   ClusterCounterElement * cce_=mCluster->getAdjacentCluster(list->getCluster());
   if(cce_)
   {
      mCluster->changeCCECounter(cce_, list->getSize());
      list->setClusterCounterElement(cce_);
      return;  
   }

  ClusterCounterElement* cce=new ClusterCounterElement(list->getCluster(), list->getSize());
  mCluster->insert(cce);
  list->setClusterCounterElement(cce);
}

bool
ClusterVertex::adjacentTo(ClusterVertex* cv)
{
   Pair<ClusterVertex*, int>* pair=mAdjacentList->find(cv);
   if(pair) return true;
   return false;
}
void
ClusterVertex::addToAdjacentHash(ClusterVertex* cv, int v)
{
   if(!mAdjacentList->find(cv))
     mAdjacentList->insert(cv, v);
}

void 
ClusterVertex::setTriangle(ClusterVertex* cv)
{
    Pair<ClusterVertex*, int>* pair=mAdjacentList->find(cv);
    if(pair==NULL) return;
    if(pair->second>=2) return;
    pair->second=2;
    mAdjacentInTriangle->push_back(cv); 
}
vector<ClusterVertex* >*
ClusterVertex::getRandomAdjacentInTriangle()
{
    int size=mAdjacentInTriangle->size();
    if(size>_STAR_THRES_)
    {
      size=_STAR_THRES_;
    }
    vector<ClusterVertex* >* rev=new vector<ClusterVertex* >(size); 
    if(size<=mAdjacentInTriangle->size())
    {
       for(int i=0; i<size; i++)
       {
          (*rev)[i]=(*mAdjacentInTriangle)[i]; 
       }
       return rev;  
    }
    int s=mAdjacentInTriangle->size();


   for(int i=0; i<s; i++)
   {
      int which=random()%(s-i)+i;
      ClusterVertex* d=(*mAdjacentInTriangle)[i];
      (*mAdjacentInTriangle)[i]=(*mAdjacentInTriangle)[which];
      (*mAdjacentInTriangle)[which]=d;
   }

   for(int i=0; i<size; i++)
   {
          (*rev)[i]=(*mAdjacentInTriangle)[i]; 
   }
   return rev;  
}


vector<ClusterVertex* >*
ClusterVertex::getRAdjacentInTriangleWOSameC()
{

   vector<ClusterVertex* >* rev=new vector<ClusterVertex* >(); 
   int size=mAdjacentInTriangle->size();
   if(size>_STAR_THRES_)
   {
    for(int i=0; i<size; i++)
    {
      int which=random()%(size-i)+i;
      if(i==which) continue;
      ClusterVertex* d=(*mAdjacentInTriangle)[i];
      (*mAdjacentInTriangle)[i]=(*mAdjacentInTriangle)[which];
      (*mAdjacentInTriangle)[which]=d;
    }
   }

   for(int i=0; i<size; i++)
   {
      ClusterVertex* c=(*mAdjacentInTriangle)[i]; 
      if(c->getCluster()==getCluster()) continue;
      rev->push_back(c);
      if(rev->size()>_STAR_THRES_) break;
       
   }
   return rev;
}

bool 
ClusterVertex::inTriangle(ClusterVertex* cv)
{
    Pair<ClusterVertex*, int>* pair=mAdjacentList->find(cv);
    if(pair==NULL || pair->second<2) return false;
    return true; 
}
