#ifndef _CLUSTER_BASE_H_
#define _CLUSTER_BASE_H_
#define _NEIGHBOR_THRES_ 8
#define _NEIGHBOR_PATH_THRES_ 8
#define _RANDOM_LIST_THRES_ 10
#define _STAR_THRES_  8
#define _DYNAMIC_SIZE_THRES_  2
#include "AdjGraph.h"
#include "ArrayList.h"
#include "SimpleHash.h"
class ClusterEdge;
class ClusterVertex;
class AdjacentElement; 
class Cluster;
class ClusterCounterElement;

class EdgeEnd
{
   protected:
     ClusterVertex* mVertex;
     AdjacentElement* mAdjacentElement;
     int mEdgeIndex;
     ClusterEdge* mEdge;
     //Cluster* mCluster;
   public: 
     EdgeEnd(ClusterVertex* v, AdjacentElement* a, ClusterEdge* e);
     EdgeEnd() {; }
      ~EdgeEnd(); 
     int getIndex() { return mEdgeIndex; }
     void setIndex(int index) { mEdgeIndex=index; }
     ClusterEdge*     getEdge();
     AdjacentElement* getAdjacentElement();
     Cluster*         getCluster();
     ClusterVertex*   getVertex();


     void setEdge(ClusterEdge* e);
     void setAdjacentElement(AdjacentElement* a);
  //   void setCluster(Cluster* c);
     void setVertex(ClusterVertex* v);
     void print();
     EdgeEnd* getAdjacent(); 
     ClusterVertex* getAdjacentVertex(); 
};

class ClusterEdge
{
   protected: 
     EdgeEnd* mSource;
     EdgeEnd* mTarget;
     Edge* mEdge;
   public:
     ClusterEdge() {; }
     ClusterEdge(Edge* e, EdgeEnd* s, EdgeEnd* t);
     ~ClusterEdge();

    void setSource(EdgeEnd* e);
    void setTarget(EdgeEnd* e);
    EdgeEnd* getSource();
    EdgeEnd* getTarget();
    Edge* getEdge();
    void setEdge(Edge* e);
    bool mIfInCycle; 
};

class ClusterCounterElement
{
   protected:
     Cluster* mCluster;
     int mCounter;
     int mIndex;
   public:
     ClusterCounterElement(Cluster* cluster, int c) {  mCluster=cluster; mCounter=c; }
     ClusterCounterElement(Cluster* cluster) {  mCluster=cluster; mCounter=0; }
     ~ClusterCounterElement() {; }
     void changeCounter(int delta) { mCounter=mCounter+delta; }
     int getCounter() { return mCounter;}
     void setIndex(int i) { mIndex=i;  }
     int  getIndex() { return mIndex; }
     Cluster* getCluster()  { return mCluster; }
     void setCluster(Cluster* c) { mCluster=c; }
};


class AdjacentElement
{
   protected:
     ArrayList<EdgeEnd *>* mEdges;
     ClusterVertex* mVertex;
     Cluster* mCluster;
     int mIndex;
     ClusterCounterElement* mCounter; 
   public:
     AdjacentElement() {; }
     AdjacentElement(ClusterVertex* v);
     ~AdjacentElement();
     void setElement();
     void insert(EdgeEnd*& e);
     void remove(EdgeEnd*& e);
     void setIndex(int index) { mIndex=index; }
     int getIndex() { return mIndex; }
     int getSize();
     ClusterCounterElement* getClusterCounterElement() { return mCounter; }
     void setClusterCounterElement(ClusterCounterElement* c) { mCounter=c; }
     ClusterVertex* getVertex();
     ArrayList<EdgeEnd *>* getEdges(); 
     Cluster* getCluster() { return mCluster;}
     //Cluster* getCluster() { return mCluster;}
     Cluster* setCluster(Cluster* c) {  mCluster=c;}
     void printNeighbor();
};



class ClusterVertex
{
    protected:
      ArrayList<AdjacentElement * >* mOutClusterList;
      SimpleHash<Cluster*, AdjacentElement *> * mOutClusterHash;
      SimpleHash<ClusterVertex*, int>* mAdjacentList;
      vector<ClusterVertex* >* mAdjacentInTriangle;
      
      AdjacentElement* mInClusterList;
      Vertex* mVertex;
      int mIndex;
      Cluster* mCluster;
    public:
     ClusterVertex() {;}
     ClusterVertex(Vertex* v, int k);
     ~ClusterVertex();
     void print();
     void setTriangle(ClusterVertex* cv);
     bool inTriangle(ClusterVertex* cv);
     ClusterCounterElement* removeInVertex(EdgeEnd* end);
     ClusterCounterElement* removeOutVertex(EdgeEnd* end);
     ClusterCounterElement* insertInVertex(EdgeEnd* end);
     ClusterCounterElement* insertOutVertex(EdgeEnd* end, Cluster* c);
     void removeOutList(AdjacentElement* list);
     void setInList(AdjacentElement*    list);
     void addToOutList(AdjacentElement* list);    
     bool adjacentTo(ClusterVertex* cv); 
     void addToAdjacentHash(ClusterVertex* cv, int v); 
     int getID() { return mVertex->getNumber(); } 
     AdjacentElement* getInList();
     ArrayList<AdjacentElement * >* getOutList(); 
     AdjacentElement *  getOutList(Cluster* c); 
     bool  hasOutListTo(Cluster* c); 
     void setIndex(int i){ mIndex=i; }
     int  getIndex()  { return mIndex; }
     bool mLocalVisited;
     bool mVisited;
     int  mDepth;
     void setCluster(Cluster* c) { mCluster=c; }
     Cluster* getCluster() { return mCluster; }
     Vertex* getVertex() { return mVertex; }
     vector<ClusterVertex* >* getRandomAdjacentInTriangle();
     vector<ClusterVertex* >* getRAdjacentInTriangleWOSameC();
};

#endif
