#include "ClusterGraph.h"
#include <iostream>
using namespace std;
//=========================================
// Graph
//==========================================
double ClusterGraph::mAlpha=3;
int ClusterGraph::HASH_TABLE_SIZE=100;
int bad_counter_g=0;
double priorBadScore=0;
double max_double(double a, double b)
{
   if(a>b) return a;
   return b;
}

ClusterGraph::ClusterGraph(AdjGraph* aGraph, double alpha)
{
   mGraph=aGraph;
   mAlpha=alpha;
   mClusters=new ArrayList<Cluster * >();
   initialize();
//   print();
}

ClusterGraph::~ClusterGraph()
{

}
void
ClusterGraph::initialize()
{
   vector<Vertex *>* vertices=mGraph->getVertices();
   int num_ver=vertices->size();
   vector<ClusterVertex*>* cluster_vertices=new vector<ClusterVertex *>(num_ver);
   mVertices=cluster_vertices;
   int tot_degree=0; 
   for(int i=0; i<num_ver; i++)
   {
      Vertex* v=(*vertices)[i];    
      ClusterVertex*  cv=new ClusterVertex(v, v->getAdjacentList()->size()+1);  
      (*cluster_vertices)[i]=cv;
      tot_degree+=v->getAdjacentList()->size();
   }//for
   tot_degree/=(num_ver+1);
   HASH_TABLE_SIZE=tot_degree+3;
   mCluster0=new Cluster(cluster_vertices, tot_degree*(num_ver+1));
   mVertices=mCluster0->getVertices()->getCopy();
   for(int i=0; i<num_ver; i++)
   {
       Vertex* v=(*vertices)[i];
       ClusterVertex*  cv=(*cluster_vertices)[i];
       cv->setCluster(mCluster0);
       AdjacentElement*  ae=cv->getInList();
       vector<Vertex* >* a=v->getAdjacentList();
       int a_size=a->size();
       for(int j=0; j<a_size; j++)
       {
          Vertex* v2=(*a)[j];
	  if(v->getNumber()>v2->getNumber())
	     continue;
//	  cout<<(v->getNumber())<<" "<<(v2->getNumber())<<endl;
          Edge* e=new Edge(v, v2);
          ClusterEdge* ce=new  ClusterEdge(e, NULL, NULL);
          EdgeEnd* s=new EdgeEnd(cv, ae, ce);
          ClusterVertex* ncv=(*cluster_vertices)[v2->getNumber()];
          cv->addToAdjacentHash(ncv, 1);
          ncv->addToAdjacentHash(cv, 1);
	  AdjacentElement*  nae=ncv->getInList();
          EdgeEnd* t=new EdgeEnd(ncv, nae, ce);
          ce->setSource(s);
          ce->setTarget(t);
          cv->insertInVertex(s);
	  ncv->insertInVertex(t);
          nae->setCluster(mCluster0);
          ae->setCluster(mCluster0);
       }   
   }
   inTriangle();
   for(int i=0; i<num_ver; i++)
   {
      //cout<<"+++++++++++"<<((*mVertices)[i])->getInList()->getSize()<<endl;
   }
   
}
void
ClusterGraph::inTriangle()
{
   int size=mVertices->size();
   for(int i=0; i<size; i++)
   {
      inTriangle((*mVertices)[i]);
   } 
}


void
ClusterGraph::inTriangle(ClusterVertex* cv)
{

  AdjacentElement* in_adj=cv->getInList();
  int size=in_adj->getSize();
  vector<EdgeEnd* >* in_list=in_adj->getEdges()->getCopy();
  for(int s=0; s<size; s++)
  {
     //cout<<"S is :"<<s<<endl;
     EdgeEnd* end=(*in_list)[s];
     ClusterEdge* e=end->getEdge();

     ClusterVertex* cv2=end->getAdjacentVertex();
     if(cv2==cv) continue;
     vector<EdgeEnd* >* secon_in_list=cv2->getInList()->getEdges()->getCopy();
     int size2=cv2->getInList()->getSize();
     for(int s2=0; s2<size2; s2++)
     {
        EdgeEnd* e3=(*secon_in_list)[s2];
        ClusterVertex* cv3=e3->getAdjacentVertex();
	if(cv3==cv2 || cv3==cv) continue;
        if(cv->adjacentTo(cv3)) 
	{
	   cv->setTriangle(cv2);
	   break;
	}//if	
     }//for
     delete  secon_in_list;
  }//for
  delete in_list;

}
//==========================================================
//
//
//==========================================================
void
ClusterGraph::initializePartition()
{
   int size=mVertices->size();
   for(int i=0; i<size; i++)
   {
      (*mVertices)[i]->mVisited=false;
      (*mVertices)[i]->mLocalVisited=false;

   }
   for(int i=0; i<size; i++)
   {
   //    cout<<i<<endl;
   //    cout.flush();
     if((*mVertices)[i]->mVisited) continue;
   //    cout<<i<<endl;
  /*     int which=-1;
       int smallest_size=100000;
       for(int j=0; j<size; j++)
       {
          if((*mVertices)[j]->mVisited) continue;
	  if((*mVertices)[j]->getInList()->getSize()<smallest_size)
	  {
	    smallest_size=(*mVertices)[j]->getInList()->getSize();
	    which=j;
	  }
       }
       if(which==-1) break;*/
       ClusterVertex* v=(*mVertices)[i];      
       findBestClusterForVertex(v);  
   }
   //computeCredit();
  // print();
  // exit(0);
  int num_c=mClusters->getSize();
  for(int i=0; i<num_c; i++)
  {
    // mClusters->getData(i)->consistencyCheck(); 
  }
  //cout<<"initialized all the paritions"<<endl;
  //exit(0);
}

void
ClusterGraph::partitionSingleLinkedClusters()
{
   vector<Cluster *>* clusters=mClusters->getCopy();
   vector<Cluster* >* new_clusters=new vector<Cluster *>();

   int num_clusters=mClusters->getSize();
   for(int i=0; i<num_clusters; i++)
   {
      vector<Cluster *>* local_new_clusters=new vector<Cluster *>();
      Cluster* c=(*clusters)[i];
      c->partitionSingleLinkedClusters(local_new_clusters, mAlpha);
      
      int size=local_new_clusters->size();
      for(int j=0; j<size; j++)
      {
          new_clusters->push_back((*local_new_clusters)[j]);
      }
      delete local_new_clusters;
   }
   int size2=new_clusters->size();
   for(int i=0; i<size2; i++)
   {
      mClusters->insert((*new_clusters)[i]);
   }
   delete clusters;
   delete new_clusters;
}



double
ClusterGraph::ifInsert(AdjacentElement* e, Cluster* c)
{
  if(c->getCredit()<0) return -1000;
  double delta_credit=((mAlpha+1)*e->getSize()-c->getSize());
  return delta_credit;
}

vector<Cluster* >*
ClusterGraph::partitionNeighbors(ClusterVertex* v, int k)
{
   v->mVisited=true;
   vector<ClusterVertex *>* level_k_vertex=new vector<ClusterVertex *>();     
   //vector<ClusterVertex *>* level_1_vertex=new vector<ClusterVertex *>();     
   
   breathLevelKVertex(level_k_vertex, v, k);
   int size=level_k_vertex->size();
   for(int s=0; s<size; s++)
       (*level_k_vertex)[s]->mLocalVisited=false;
   
  // breathLevelKVertex(level_1_vertex, v, 1);
   
   Cluster* pseudoClusters=new Cluster();
   for(int s=0; s<size; s++)
   {
       mCluster0->moveTo((*level_k_vertex)[s], pseudoClusters, mAlpha);
       (*level_k_vertex)[s]->mLocalVisited=false;
   }
   //int size1=level_1_vertex->size();
   //for(int s=0; s<size1; s++)
   //    (*level_1_vertex)[s]->mLocalVisited=true;
   //delete level_1_vertex; 

   /*for(int s=0; s<size; s++)
   {
      if((*level_k_vertex)[s]->mLocalVisited!=true && 
		      (*level_k_vertex)[s]->getInList()->getSize()<=1)
      {
	  pseudoClusters->moveTo((*level_k_vertex)[s], mCluster0, mAlpha);
      }   
   }*/
   for(int s=0; s<size; s++)
   {
       (*level_k_vertex)[s]->mLocalVisited=false;
   }  

    
   vector<Cluster *>* rev=new vector<Cluster *>();
   

  vector<vector<ClusterVertex*>*>* ps=pseudoClusters->partition();  
  rev->push_back(pseudoClusters);
  if(ps->size()==1)
  {
    for(int s=0; s<size; s++)
   {
       (*level_k_vertex)[s]->mLocalVisited=false;
   }
   delete (*ps)[0];
   delete ps;
   delete level_k_vertex;  
   return rev;   
  }//if
  int num_clusters=ps->size();   
  /*vector<ClusterVertex* >* lc=NULL;   
  int largest_size=0;
  for(int i=0; i<num_clusters; i++)
  {
     if((*ps)[i]->size()>largest_size)
     {
        largest_size=(*ps)[i]->size();
	lc=(*ps)[i];
     } 
  }//for
  Cluster* c=new Cluster();
  if(lc!=NULL)
  {
    int c2=lc->size();
    for(int j=0; j<c2; j++)
    {
     pseudoClusters->moveTo((*lc)[j], c, mAlpha);
    }
    destroyCluster(pseudoClusters);
    rev->clear();
    rev->push_back(c);
  }
  else delete c;*/
  for(int i=1; i<num_clusters; i++)
  {
     Cluster* c=new Cluster();
     vector<ClusterVertex *>* cvs=(*ps)[i];
     int c2=cvs->size();
     for(int j=0; j<c2; j++)
     {  
       pseudoClusters->moveTo((*cvs)[j], c, mAlpha);
     }
     rev->push_back(c);
     delete (*ps)[i];  
  }
  if(num_clusters!=0) delete (*ps)[0]; 
  delete ps; 
  for(int s=0; s<size; s++)
   {
       (*level_k_vertex)[s]->mLocalVisited=false;
   }
  delete level_k_vertex;
  return rev;
}

void
ClusterGraph::breathLevelKVertex(vector<ClusterVertex *>* vertices, ClusterVertex* v, int k)
{
   if(k==0) return;
   AdjacentElement* ae=v->getInList(); 
   ArrayList<EdgeEnd *>* cvs=ae->getEdges();
   vector<EdgeEnd *>*    cvs_v=cvs->getCopy();
   int size=cvs->getSize();
   for(int i=0; i<size; i++)
   {
      EdgeEnd* e=(*cvs_v)[i]->getAdjacent();
      ClusterVertex* av=e->getVertex();
      if(av->mLocalVisited || av->mVisited) continue;
      if(v->inTriangle(av) || size<10) vertices->push_back(av);
      av->mLocalVisited=true;
      breathLevelKVertex(vertices, av, k-1);      
   }
   delete cvs_v;
   return;
}

vector<Cluster *>*
ClusterGraph::partitionNeighbors(ClusterVertex* v)
{
  vector<Cluster *>* rev=new vector<Cluster *>();
  Cluster* pseudoClusters=new Cluster();
  AdjacentElement* in_adj=v->getInList();
  int size=in_adj->getSize();
  vector<EdgeEnd* >* in_list=in_adj->getEdges()->getCopy();
  //in_adj->printNeighbor(); 
  //cout<<"size is "<<size<<endl;
   for(int s=0; s<size; s++)
  {
     //cout<<"S is :"<<s<<endl;
     EdgeEnd* end=(*in_list)[s];
     ClusterEdge* e=end->getEdge();
     if(e->getSource()==end)
     {
       mCluster0->moveTo(e->getTarget()->getVertex(), pseudoClusters, mAlpha);
     }
     else
     {
        mCluster0->moveTo(e->getSource()->getVertex(), pseudoClusters, mAlpha);
     }
  }
  delete in_list;
  vector<vector<ClusterVertex*>*>* ps=pseudoClusters->partition();  
  rev->push_back(pseudoClusters);
  if(ps->size()==1)
  {
   //  delete (*ps)[0];
     //delete ps;
     return rev;   
  }//if
  int num_clusters=ps->size();   
  for(int i=1; i<num_clusters; i++)
  {
     Cluster* c=new Cluster();
     vector<ClusterVertex *>* cvs=(*ps)[i];
     int c2=cvs->size();
     for(int j=0; j<c2; j++)
     {  
       pseudoClusters->moveTo((*cvs)[j], c, mAlpha);
     }
     rev->push_back(c);
    // delete (*ps)[i];  
  }
  //delete ps; 
  return rev;
}

void
ClusterGraph::destroyCluster(Cluster* c)
{ 
   //cout<<"Destroying cluster "<<endl;
   //cout<<"      "; c->print(); 
   vector<ClusterVertex *>* vertices=c->getVertices()->getCopy();
   int size=c->getSize();
   for(int i=0; i<size; i++)
   {
      c->moveTo((*vertices)[i], mCluster0, mAlpha);
      (*vertices)[i]->mVisited=false;
   }//for
   delete vertices;
   delete c;  
}
void  
ClusterGraph::clearCluster(Cluster* c)
{  
   //cout<<"Destroying cluster "<<endl;
   //cout<<"      "; c->print(); 
   vector<ClusterVertex *>* vertices=c->getVertices()->getCopy();
   int size=c->getSize();
   for(int i=0; i<size; i++)
   {
      c->moveTo((*vertices)[i], mCluster0, mAlpha);
      (*vertices)[i]->mVisited=false;
   }//for
   delete vertices;
   //delete c;  
}  
  


void
ClusterGraph::findBestClusterForVertex(ClusterVertex* v)
{
  // cout<<"----===start to process vertex "<<v->getID()<<"which has neigh size as "<<v->getInList()->getSize()<<endl;
   //cout<<"---Node 7 has  number of neighbors as "<<((*mVertices)[7])->getInList()->getSize()<<endl;
   
   //vector<Cluster* >* neighbors=partitionNeighbors(v);
   vector<Cluster* >* neighbors=partitionNeighbors(v, 1);
   ArrayList<AdjacentElement* >* adjs=v->getOutList();
   vector<AdjacentElement* >* data=adjs->getData();
   double best_score=-100000;
   bool found=false;
   double absoluteScore=0;
   AdjacentElement* candidate=NULL;
   int size=adjs->getSize();
   //cout<<"number of candidnate clusters are "<<size<<endl;
   
   for(int i=0; i<size; i++)
   {
    AdjacentElement* ae=(*data)[i];
    Cluster* c=ae->getCluster();
    double delta_credit=((mAlpha+1)*ae->getSize()-c->getSize());
    //double delta_credit=((mAlpha+1)*ae->getSize()-c->getSize()+c->getCredit())/(c->getSize()+1);

   //cout<<"credit "<<delta_credit<<" "<<c->getCredit()<<endl;
    //if(delta_credit> best_score && c->getCredit()>=0 && delta_credit>0)
    if(delta_credit> best_score && c->getCredit()/c->getSize()>=0 && delta_credit>0)
    {
       best_score=delta_credit;
       candidate=ae;
       absoluteScore=delta_credit+c->getCredit(); 
       found=true;   
    }//for    
   }//for
   //cout<<"---Node 7 has  number of neighbors as "<<((*mVertices)[7])->getInList()->getSize()<<endl;
   Cluster* candidate_cluster;
   if(!found)
   {
     candidate_cluster=new Cluster();
     mCluster0->moveTo(v, candidate_cluster, mAlpha);
     mClusters->insert(candidate_cluster);
     v->mVisited=true;   	
    
   }
   else
   {
     candidate_cluster=candidate->getCluster();
     //cout<<mCluster0->getSize()<<" c0 has size"<<endl;
     //cout<<"to be inserted in\n          "; candidate_cluster->print();
     mCluster0->moveTo(v, candidate_cluster, mAlpha);
     v->mVisited=true;
   }
   candidate_cluster->mVisit=true;
   //cout<<"---Node 7 has  number of neighbors as "<<((*mVertices)[7])->getInList()->getSize()<<endl;
   int cluster_size=neighbors->size();
   bool from_neighbor=false;

   for(int i=0; i<cluster_size; i++)
   {
     if(candidate_cluster!=(*neighbors)[i])
     {
        destroyCluster((*neighbors)[i]);      
     }
     else
     {
        from_neighbor=true;
     }    
   }
   //cout<<"---Node 7 has  number of neighbors as "<<((*mVertices)[7])->getInList()->getSize()<<endl;
   
   //cout<<"--------======= at the "end of the iteration, Cluster0 is\n       of size "<<mCluster0->getSize()<<":";
   //mCluster0->print();
   //candidate_cluster->print();
   //cout<<endl<<endl<<endl;
   if(from_neighbor==true)
   {
     mClusters->insert(candidate_cluster);
     vector<ClusterVertex* >* vertices=candidate_cluster->getVertices()->getData();
     int size=candidate_cluster->getSize();

     for(int i=0; i<size; i++)
     {
        (*vertices)[i]->mVisited=true; 
     }//for  
   }//if
  // candidate_cluster->consistencyCheck();
  // candidate_cluster->print();
   delete neighbors; 
}


void
ClusterGraph::print()
{
   int num_ver=mVertices->size();
   cout<<num_ver<<"++++++"<<" "<<mCluster0->getSize()<<endl;
   /*for(int i=0; i<num_ver; i++)
   {
      //cout<<"+++++++++++"<<((*mVertices)[i])->getInList()->getSize()<<endl;
      AdjacentElement* ads=(*mVertices)[i]->getInList();
      int size=ads->getSize();
      vector<EdgeEnd* >* ends=ads->getEdges()->getData();
      for(int j=0; j<size; j++)
      {
         EdgeEnd* end=(*ends)[j];
         ClusterEdge* e=end->getEdge();
         Edge* edge=e->getEdge();
         if(end==e->getSource())
         {
           // cout<<(edge->getSource()->getNumber())<<":"<<(edge->getTarget()->getNumber())<<endl;
         }
      }
   }*/
   cout<<"begin to print cluster information "<<endl;
   int num_cluster=mClusters->getSize();
   for(int i=0; i<num_cluster; i++)
   {
      mClusters->getData(i)->print();
   }
}
//===============================================
void
ClusterGraph::mergeClusters()
{
   vector<Cluster* >* mergedClusters=new vector<Cluster *>();
   vector<Cluster* >* existingClusters=(mClusters)->getCopy();
   int num_clusters=mClusters->getSize();
   for(int i=0; i<num_clusters; i++)
   {
      Cluster* merged=mergeNeighbors((*existingClusters)[i]); 
      if(merged!=NULL) 
      { 
         mergedClusters->push_back(merged);
         if(merged->getSize()!=0)
	 cout<<merged->getIndex()<<" "<<merged->getSize()<<" got errors "<<merged->getAdjacentClusters()->getSize()<<" "<<merged->getIndex()<<endl;	 
      }
   }
   int size=mergedClusters->size();
   for(int i=0; i<size; i++)
   {
     mClusters->remove((*mergedClusters)[i]);
     destroyCluster((*mergedClusters)[i]);
   }  
   delete existingClusters;
   delete mergedClusters;
}

Cluster*
ClusterGraph::mergeNeighbors(Cluster* c)
{
  if(c->getSize()<=0) return NULL;
  ArrayList<ClusterCounterElement *>* neigh_clusters=c->getAdjacentClusters();
  vector<ClusterCounterElement *>*    neigh_clusters_v=neigh_clusters->getCopy();
  int num_neigh=neigh_clusters->getSize();
  Cluster* best_neigh=NULL;
  double best_score=0;
  int c_size=c->getSize();
  double c_credit=c->getCredit();
  for(int i=0; i<num_neigh; i++)
  {
     ClusterCounterElement* n=(*neigh_clusters_v)[i];
     Cluster* n_c=n->getCluster();
     int n_c_size=n_c->getSize();

     double credit=(mAlpha+1)*n->getCounter()-(n_c_size*c_size);
     if(credit>best_score && n_c!=mCluster0 && n_c->getSize()>0)
     {
        best_score=credit;
	best_neigh=n_c;
     }  
  }//for
  delete neigh_clusters_v;
  if(best_neigh && best_neigh->getSize()>0)
  {
  // cout<<"merge clusters, before merge"<<endl;	
  // best_neigh->print();
  // c->print();  
   best_neigh->mergeTo(c, mAlpha);
 //  cout<<"merge clusters, after merge"<<endl;	
 //  best_neigh->print();
 //  c->print();  
 //  cout<<"end of merge "<<endl<<endl<<endl;
   return best_neigh;
  }
  return NULL;
}


//===============================================


void
ClusterGraph::extractBadVertex()
{
   int size=mVertices->size();
   for(int i=0; i<size; i++)
   {
      ClusterVertex* v=(*mVertices)[i];
      Cluster* c=v->getCluster();
      int in_l=v->getInList()->getSize();
      double credit=c->getCredit();
      double new_credit=-(mAlpha+1)*in_l+(c->getSize()-1);
      double old_average=c->getCredit()/c->getSize();
      double new_average=(c->getCredit()+new_credit)/(c->getSize()-1);
      //if(bad_counter_g) 
      if(new_credit>0 || old_average<new_average)
      {
         c->moveTo(v, mCluster0, mAlpha);
      }
   }//for
   //cout<<"before bad"<<endl;
   /*//computeCredit();
   ArrayList<ClusterVertex *>* bad_vertices=mCluster0->getVertices();
   vector<ClusterVertex *>* bad_vertices_v=bad_vertices->getData();
   int num_bad=bad_vertices->getSize();
   vector<ClusterVertex *>* bad_vertices_copy=new  vector<ClusterVertex *>(num_bad);
   for(int i=0; i<num_bad; i++)
   {
     (*bad_vertices_copy)[i]=(*bad_vertices_v)[i]; 
   }
   for(int i=0; i<num_bad; i++)
   {
     ClusterVertex* cv=(*bad_vertices_copy)[i];
     ArrayList<AdjacentElement * >* adjs=cv->getOutList();
     vector<AdjacentElement * >*    adjs_v=adjs->getData();
     int adj_size=adjs->getSize();
     double best_credit=0;
     Cluster* bestCluster=NULL;
     for(int j=0; j<adj_size; j++)
     {
       Cluster* c=(*adjs_v)[j]->getCluster();
       double credit=(mAlpha+1)*((*adjs_v)[j])->getSize()-c->getSize();
       if(credit>best_credit)
       {
         best_credit=credit;
	 bestCluster=c;
       }
     }
     if(bestCluster!=NULL)
     {
         mCluster0->moveTo(cv, bestCluster, mAlpha);  
     }
     else
     {
         Cluster* new_c=new Cluster();
	 mClusters->insert(new_c);
	 mCluster0->moveTo(cv, new_c, mAlpha);
     }
   }
   //cout<<"after bad"<<endl;
   delete bad_vertices_copy;*/
}

void
ClusterGraph::reclustering()
{
  int path_len=3;
  int num_clusters=mClusters->getSize();
  vector<Cluster *>* candidate=new vector<Cluster *>();
  vector<Cluster *>* cleared_list=new vector<Cluster *>();
  for(int i=0; i<num_clusters; i++)
  {
     Cluster* c=mClusters->getData(i);
     c->mVisit=false;
  }
  
  for(int i=0; i<num_clusters; i++)
  {  
     Cluster* c=mClusters->getData(i);
     if(c->mVisit) continue;
     c->mVisit=true;
     candidate->push_back(c);
     for(int j=0; j<path_len; j++)
     {
       ArrayList<ClusterCounterElement *>* neighbors=c->getAdjacentClusters();     
       int num_neigh=neighbors->getSize();
       vector<ClusterCounterElement *>* neighbors_v=neighbors->getData();
       Cluster* best_neigh=NULL;
       double best_connect=0;
       for(int k=0; k<num_neigh; k++)
       {
          ClusterCounterElement * cce=(*neighbors_v)[k];
	  if(cce->getCounter()>best_connect && cce->getCluster()->mVisit==false && cce->getCluster()!=mCluster0)
          {
             best_connect=(*neighbors_v)[k]->getCounter();
             best_neigh=(*neighbors_v)[k]->getCluster();  
          }
       }
       if(best_neigh==NULL) break;
       c=best_neigh;
       candidate->push_back(c);
       c->mVisit=true;   
       
     }
     if(candidate->size()>1)
     {
       reclustering(candidate);
       int c_size=candidate->size();
       for(int j=0; j<c_size; j++)
       {
         // cleared_list->push_back((*candidate)[j]);
       }
     }
     candidate->clear();   
   }
  
  int num_cleared=cleared_list->size();
  cout<<num_cleared<<" "<<num_clusters<<endl;

  for(int i=0; i<num_cleared; i++)
  {
     Cluster* c=(*cleared_list)[i];
     cout<<c->getSize()<<" "<<c->getAdjacentClusters()->getSize()<<" "<<c->getIndex()<<endl;
     mClusters->remove(c);
     delete c;
  }
  /*for(int i=0; i<mVertices->size(); i++)
  {
    (*mVertices)[i]->getInList();
  }*/
   delete cleared_list;
   delete candidate;
}
void
ClusterGraph::reclustering(vector<Cluster *>* clusters)
{
   int size=clusters->size();
   for(int i=0; i<size; i++)
   {
      clearCluster((*clusters)[i]);
   } 
   cluster();
   
}

void
ClusterGraph::cluster()
{
  ArrayList<ClusterVertex *>* vertices=mCluster0->getVertices();
  vector<ClusterVertex *>*    vertices_v=vertices->getData();
  vector<ClusterVertex *>*    vertices_copy=new  vector<ClusterVertex *>();
  
  int size=vertices->getSize(); 
  for(int i=0; i<size; i++)
  {
    vertices_copy->push_back((*vertices_v)[i]);
    (*vertices_v)[i]->mVisited=false;
  }
  for(int i=0; i<size; i++)
  {
    if((*vertices_copy)[i]->mVisited==false)
    {
      findBestClusterForVertex((*vertices_copy)[i]);
    }
  }
  //moveSingleVertex();
  //mergeClusters();
}

void
ClusterGraph::moveVertex(ClusterVertex* v)
{
   ArrayList<AdjacentElement* >* alist=v->getOutList();
   vector<AdjacentElement* >*  alist_v=alist->getCopy();
   int size=alist->getSize();
   double max_credit=0;
   Cluster* max_cluster=NULL;
   Cluster* source=v->getCluster();
   int num_lost_edges=v->getInList()->getSize();
   for(int i=0; i<size; i++)
   {
      AdjacentElement* adj=(*alist_v)[i];
      Cluster* c=adj->getCluster();
      double credit=c->getCredit();
      int introduced_size=adj->getSize();

     double delta_credit=(-((mAlpha+1)*num_lost_edges-(source->getSize()-1)))+ 
			             ((mAlpha+1)*introduced_size-c->getSize()); 
     if(delta_credit>max_credit && c!=mCluster0)
     {
        max_credit=delta_credit;
	max_cluster=c;
     }//if  
   } //for
   if(max_cluster!=NULL)
   {
     source->moveTo(v, max_cluster, mAlpha);
   }
   delete alist_v;
}

void
ClusterGraph::setMoveSingleFlags()
{
   vector<Cluster *>* clusters=mClusters->getCopy();
   int size=clusters->size();
   for(int i=0; i<size; i++)
   {
     Cluster* c=(*clusters)[i];
     if(c->mAltered)
     {
        c->mMoveSingleAltered=true; continue;
     }
     vector<ClusterCounterElement* >* cces=(*clusters)[i]->getAdjacentClusters()->getCopy();
     int size2=cces->size();
     for(int j=0; j<size2; j++)
     {
        if(((*cces)[j])->getCluster()->mAltered)
        {
          c->mMoveSingleAltered=true; break;
        }//if
     } //for
     delete cces;
   }
   for(int i=0; i<size; i++)
   {
     Cluster* c=(*clusters)[i];
     c->mAltered=false;
   } 
   delete clusters;
}
void
ClusterGraph::starSplit()
{
  int size=mVertices->size();
  for(int i=0; i<size; i++)
  {
     Cluster* new_c=starSplit((*mVertices)[i]); 
     
    if(new_c!=NULL) mClusters->insert(new_c);
  }
}

void
ClusterGraph::starPull()
{
  int size=mVertices->size();
  for(int i=0; i<size; i++)
  {
     starPull((*mVertices)[i]); 
     
  }
}


Cluster*
ClusterGraph::starSplit(ClusterVertex* cv)
{
   if(cv->getCluster()==mCluster0) return NULL;
   vector<ClusterVertex *>* cvs=cv->getRandomAdjacentInTriangle();
   int size=cvs->size();
   SimpleHash<Cluster*, int>* hash=new SimpleHash<Cluster *, int>(size);
   for(int i=0; i<size; i++)
   {
      hash->insert((*cvs)[i]->getCluster(), 0);
   } 
   
   int totSize=1<<size;
   double max_credit=0;
   int max_index=0;
   for(int i=0; i<totSize; i++)
   {
      double c=starSplitCredit(cv, i, size, cvs, hash);
      if(c>max_credit)
      {
        max_credit=c;
	max_index=i;
      } 
   }
   Cluster* rev=NULL;
   if(max_credit>0)
   {
      rev=new Cluster();
      for(int i=0, index=max_index; i<size; i++, index/=2) 
      {
         if(index%2==0) continue; 
         Cluster* c=(*cvs)[i]->getCluster();
          c->moveTo((*cvs)[i],rev, ClusterGraph::mAlpha);	 
      }//for 
      cv->getCluster()->moveTo(cv, rev, ClusterGraph::mAlpha);
   }
   delete hash; 
   delete cvs;
   return rev;
}


Cluster*
ClusterGraph::starPull(ClusterVertex* cv)
{
   if(cv->getCluster()==mCluster0) return NULL;
   vector<ClusterVertex *>* cvs=cv->getRAdjacentInTriangleWOSameC();
   int size=cvs->size();
   SimpleHash<Cluster*, int>* hash=new SimpleHash<Cluster *, int>(size);
   for(int i=0; i<size; i++)
   {
      hash->insert((*cvs)[i]->getCluster(), 0);
   } 
   
   int totSize=1<<size;
   double max_credit=0;
   int max_index=0;
   for(int i=0; i<totSize; i++)
   {
      double c=starPullCredit(cv, i, size, cvs, hash);
      if(c>max_credit)
      {
        max_credit=c;
	max_index=i;
      } 
   }
   Cluster* t=cv->getCluster();
   if(max_credit>0)
   {
      for(int i=0, index=max_index; i<size; i++, index/=2) 
      {
         if(index%2==0) continue; 
         Cluster* c=(*cvs)[i]->getCluster();
         if(c!=t) c->moveTo((*cvs)[i], t, ClusterGraph::mAlpha);
      }//for
   }
   delete hash; 
   delete cvs;
   return NULL;
}


double
ClusterGraph::starSplitCredit(ClusterVertex* cv, int nodes, int len, 
		vector<ClusterVertex* >* cvs, SimpleHash<Cluster*, int>* hash)
{
  int num_node=0;
  int edges=0;

  for(int i=0, index=nodes; i<len; i++,index/=2) 
  {
     if(index%2==0) continue;
     for(int j=i+1, index2=index/2; j<len; j++, index2/=2)
     {
       if(index2%2==0) continue; 
       bool adj=(*cvs)[i]->adjacentTo((*cvs)[j]);
       if(adj) edges+=1; 
        
       if(adj && (*cvs)[i]->getCluster()==(*cvs)[j]->getCluster())
       {
          edges+=1; 
       }   
     }
     edges-=(*cvs)[i]->getInList()->getSize();
     Pair<Cluster*, int>* p=hash->find((*cvs)[i]->getCluster());
     num_node+=1;
     p->second+=1; 
  }//for
  num_node+=1; //for cv
  edges+=num_node-cv->getInList()->getSize()-1;


  double credit=(ClusterGraph::mAlpha+1)*edges-0.5*(num_node)*(num_node-1);
  for(int i=0, index=nodes; i<len; i++, index/=2) 
  {
     if(index%2==0) continue;
     Cluster* c=(*cvs)[i]->getCluster();
     Pair<Cluster*, int>* p=hash->find(c);
     int dn=p->second;
     if(dn==0 || c==mCluster0) continue;
     credit+=0.5*(dn*(2*c->getSize()-1)-dn*dn);
     p->second=0;
  }//for

  return credit;
}

double
ClusterGraph::starPullCredit(ClusterVertex* cv, int nodes, int len, 
		vector<ClusterVertex* >* cvs, SimpleHash<Cluster*, int>* hash)
{
  int num_node=0;
  int edges=0;

  for(int i=0, index=nodes; i<len; i++, index/=2) 
  {
     if(index%2==0) continue;
     for(int j=i+1, index2=index/2; j<len; j++, index2/=2)
     {
       if(index2%2==0) continue; 
       if(cv->getCluster()==(*cvs)[i]->getCluster() 
          && cv->getCluster()==(*cvs)[j]->getCluster()) continue;
       bool adj=(*cvs)[i]->adjacentTo((*cvs)[j]);
       if(adj) edges+=1; 
        
       if(adj && (*cvs)[i]->getCluster()==(*cvs)[j]->getCluster())
       {
          edges+=1; 
       }   
     }//for
     if(cv->getCluster()!=(*cvs)[i]->getCluster())
     {
       edges-=(*cvs)[i]->getInList()->getSize();
       num_node+=1;
     }
     Pair<Cluster*, int>* p=hash->find((*cvs)[i]->getCluster());
     p->second+=1; 
  }//for
  edges+=num_node;
  double credit=(ClusterGraph::mAlpha+1)*edges;
  credit-=0.5*num_node*(2*cv->getCluster()->getSize()-1+num_node);
  for(int i=0, index=nodes; i<len; i++, index/=2) 
  {
     if(index%2==0) continue;
     Cluster* c=(*cvs)[i]->getCluster();
     Pair<Cluster*, int>* p=hash->find(c);
     int dn=p->second;
     if(dn!=0 && c!=mCluster0 && c!=cv->getCluster())
     {
       credit+=0.5*(dn*(2*c->getSize()-1)-dn*dn);
     }
     p->second=0;
  }//for
  
  return credit;
}


void
ClusterGraph::moveSingleVertex()
{
   setMoveSingleFlags();
   vector<Cluster *>* clusters=mClusters->getCopy();
   int size=clusters->size();
   for(int i=0; i<size; i++)
   {
      if(!(*clusters)[i]->mMoveSingleAltered) continue;
      vector<ClusterVertex* >* cv=(*clusters)[i]->getVertices()->getCopy(); 
      int size2=cv->size(); 
      for(int j=0; j<size2; j++)
      {
        moveVertex((*cv)[j]);
      }
      delete cv;
   }
   delete clusters;
}

void 
ClusterGraph::pullOverPartialCluster()
{
   vector<Cluster *>* clusters=mClusters->getCopy();
   int num_clu=mClusters->getSize();
   for(int i=0; i<num_clu; i++)
   {
      (*clusters)[i]->mVisit=false; 
   }

   for(int i=0; i<num_clu; i++)
   {
      pullOverPartialCluster((*clusters)[i]);
   }

   delete clusters;
}
/*
void
ClusterGraph::pullOverPartialCluster(Cluster* c)
{
   c->mVisit=true; 
   vector<vector<ClusterVertex *>* >* vertices_vectors=c->partitionVertexAccordingToNeighbor();
   vector<Cluster* >* clusters=c->getClusterCopy();
   int size=vertices_vectors->size();
   
   double best_credit=0;
   int which=-1;
   Cluster* best=NULL;
   for(int i=0; i<size; i++)
   {
      vector<ClusterVertex *>*  cv=(*vertices_vectors)[i];
      int s2=cv->size();
      Cluster* target=(*clusters)[i];
      if(target==mCluster0 || target->mVisit==true) continue;
      double old_credit=c->getCredit()+target->getCredit();

      for(int j=0; j<s2; j++)
      {
        c->moveTo((*cv)[j], target, mAlpha);
      }
      double new_credit=c->getCredit()+target->getCredit();
      double delta=new_credit-old_credit;
      if(delta>=best_credit)
      {
         delta=best_credit;
         best=target;
	 which=i; 
      }
      
      for(int j=0; j<s2; j++)
      {
        target->moveTo((*cv)[j], c, mAlpha);
      }
   }


   if(best!=NULL)
   {
      vector<ClusterVertex *>*  cv=(*vertices_vectors)[which];
      int s2=cv->size();
      for(int j=0; j<s2; j++)
      {
        c->moveTo((*cv)[j], best, mAlpha);
      }
   }

   for(int i=0; i<size; i++)
   {
     delete (*vertices_vectors)[i];
   }
   delete vertices_vectors;
   delete clusters;

}*/



void
ClusterGraph::pullOverPartialCluster(Cluster* c)
{
   
   c->mVisit=true; 
   if(c->getSize()>100) return;
   
   c->createMatrix(NULL, NULL);
   vector<vector<ClusterVertex *>* >* vertices_vectors=c->partitionVertexAccordingToNeighbor();
   vector<Cluster* >* clusters=c->getClusterCopy();
   int size=vertices_vectors->size();
    
   double best_credit=0;
   int which=-1;
   Cluster* best=NULL;
   for(int i=0; i<size; i++)
   {
      vector<ClusterVertex *>*  cv=(*vertices_vectors)[i];
      int s2=cv->size();
      Cluster* target=(*clusters)[i];
      if(target==mCluster0 || target->mVisit==true) continue;
      double delta=(mAlpha+1)*(-c->inDegrees(cv)+2*c->numEdges(cv)+c->getAdjacentCluster(target)->getCounter())  
	      -s2*((target->getSize()-c->getSize())+s2);
      if(delta>=best_credit)
      {
         delta=best_credit;
         best=target;
	 which=i; 
      }
   }


   if(best!=NULL)
   {
      vector<ClusterVertex *>*  cv=(*vertices_vectors)[which];
      int s2=cv->size();
      for(int j=0; j<s2; j++)
      {
        c->moveTo((*cv)[j], best, mAlpha);
      }
   }

   for(int i=0; i<size; i++)
   {
     delete (*vertices_vectors)[i];
   }
   c->destroyMatrix();
   delete vertices_vectors;
   delete clusters;
  
}


/*
void
ClusterGraph::moveVertex(ClusterVertex* v)
{
   ArrayList<AdjacentElement* >* alist=v->getOutList();
   vector<AdjacentElement* >*  alist_v=alist->getData();
   int size=alist->getSize();
   double max_credit=0;
   Cluster* max_cluster=NULL;
   Cluster* source=v->getCluster();
   int num_lost_edges=v->getInList()->getSize();
   for(int i=0; i<size; i++)
   {
      AdjacentElement* adj=(*alist_v)[i];
      Cluster* c=adj->getCluster();
      double credit=c->getCredit();
      int introduced_size=adj->getSize();

     double delta_credit=(-((mAlpha+1)*num_lost_edges-(source->getSize()-1)))+
                                     ((mAlpha+1)*introduced_size-c->getSize());
     if(delta_credit>max_credit)
     {
        max_credit=delta_credit;
        max_cluster=c;
     }//if  
   } //for
   if(max_cluster!=NULL)
   {
     source->moveTo(v, max_cluster, mAlpha);
   }
}*/



void
ClusterGraph::optimize()
{
   moveSingleVertex();
   //computeCredit();
   //mergeClusters();
   //computeCredit();

}
void
ClusterGraph::computeCredit()
{
   double credit=0;
   int num_cluster=mClusters->getSize();
   vector<Cluster *>* cls=mClusters->getData();
   for(int i=0; i<num_cluster; i++)
   {
     credit+=((*cls)[i])->getCredit();
     //cout<<"credit is "<<((*cls)[i])->getCredit()<<endl;
 //    ((*cls)[i])->print();
   }
   mCredit=credit;
}


bool
ClusterGraph::dynamicPathIter(Cluster* c, int depth, vector<Cluster* >* candidate)
{
   if(depth==0)
   {
      if(candidate->size()>1) return dynamicPath(candidate);
   } 
   ArrayList<ClusterCounterElement *>* neighbors=c->getAdjacentClusters();     
   vector<ClusterCounterElement *>* neighbors_v=neighbors->getData();
   int num_neigh=neighbors->getSize();
   
   num_neigh=neighbors->getSplitIndex();
   for(int i=0; i<num_neigh; i++)
   {
     if(i>=neighbors->getSize()) break;
     ClusterCounterElement * cce=neighbors->getData(i);
     if(cce->getCluster()->mVisit==true || cce->getCluster()==mCluster0 || cce->getCounter()<= _DYNAMIC_SIZE_THRES_)
	   continue;
     Cluster* n=cce->getCluster();
     if(n==NULL)
     {
         bool changed=dynamicPathIter(NULL, 0, candidate);
	 //if(changed) return true;
	 continue; 
     }
     n->mVisit=true;   
     candidate->push_back(n);
     bool changed=dynamicPathIter(n, depth-1, candidate);
     candidate->pop_back();   
     n->mVisit=false;
     //if(changed) return changed; 
   }
   return false;
}

void
ClusterGraph::dynamicPath()
{
  int path_len=10;
  int num_clusters=mClusters->getSize();
   vector<Cluster *>* candidate=new vector<Cluster *>();
   for(int i=0; i<num_clusters; i++)
   {
      Cluster* c=mClusters->getData(i);
      c->mVisit=false;
   }
  
  for(int i=0; i<num_clusters; i++)
  {  
     Cluster* c=mClusters->getData(i);
     if(c->mVisit) continue;
     c->mVisit=true;
     candidate->push_back(c);
     dynamicPathIter(c, path_len-1, candidate);
     c->mVisit=false;   
     candidate->clear();
   }
   delete candidate;
   removeEmptyClusters();
}



void
ClusterGraph::exchangeWithNeighbor()
{
   setMoveSingleFlags();

  int num_clusters=mClusters->getSize();
  vector<Cluster *>* candidate=new vector<Cluster *>();
  for(int i=0; i<num_clusters; i++)
  {
      Cluster* c=mClusters->getData(i);
      c->mVisit=false;
  }
  
  for(int i=0; i<num_clusters; i++)
  {  
     Cluster* c=mClusters->getData(i);
     if(!c->mMoveSingleAltered) continue;
    //pick up the path
     vector<ClusterCounterElement *>* cces=c->getAdjacentClusters()->getCopy();     
     int size=cces->size();
      for(int j=0; j<size; j++)
     {
	vector<ClusterCounterElement* >* cces_temp=c->getAdjacentClusters()->getData();
	if(c->getSize()==0 || (*cces)[j]!=(*cces_temp)[j]) break;
	//if((*cces)[j]->getCounter()<=1) break;
	if((*cces)[j]->getCounter()<=1) continue;;
        Cluster* c2=(*cces)[j]->getCluster();
        if(c2==mCluster0) continue;
	candidate->push_back(c);
        candidate->push_back(c2);
        dynamicPath(candidate);
        candidate->clear();
     }
     delete  cces;
   }
   delete candidate;
   removeEmptyClusters();
}
bool
ClusterGraph::dynamicPath(vector<Cluster*>* path)
{
   int len=path->size();
   for(int i=0; i<len-1; i++)
   {
      (*path)[i]->retrieveDynamicList((*path)[i+1]);
   }
   for(int i=0; i<len; i++)
   {  
      if(i==0)
      (*path)[0]->fillupDynamicTable(NULL);
      else (*path)[i]->fillupDynamicTable((*path)[i-1]);
   }//for
   int maxIndex=0;
   bool changed=false;
   for(int i=len-2; i>=0; i--)
   {
    if(len-2==i)
    {
      maxIndex=(*path)[i]->extractTheBestMovement((*path)[i+1]);
    }
    else
    {
       maxIndex=(*path)[i]->extractTheBestMovement(maxIndex, (*path)[i+1]);
    }
    if(maxIndex!=0) changed=true;
   }
   for(int i=0; i<len; i++)
   {
      (*path)[i]->destroyMatrix();
    //  (*path)[i]->mVisit=false;
   }
   return changed;
}

/*
bool
ClusterGraph::randomCycleOfClusters(Cluster* to_search, Cluster* start, int depth, vector<Cluster* >* cycle)
{
   vector<ClusterCounterElement* >* cces=to_search->getAdjacentClusters()->getSplitCopy();
   //do a random permutations of cces; 
   //int size=to_search->getAdjacentClusters()->getSize();
   int size=to_search->getAdjacentClusters()->getSplitIndex();
   for(int i=0; i<size; i++)
   {
     int which=i+random()%(size-i);
     ClusterCounterElement* the_one=(*cces)[which];
     (*cces)[which]=(*cces)[i];
     (*cces)[i]=the_one;   
   } 
   //do the depth first search
   bool found=false;
   if(depth==0)
   {
     
     for(int i=0; i<size; i++)
     {
	 if((*cces)[i]->getCluster()==start)   
	 {
	    //cycle->push_back((*cces)[i]->getCluster());
	    found=true;
	    break;
	 }
     }//for
   }
   else 
   {
     for(int i=0; i<size; i++)
     {
        Cluster* a_c=(*cces)[i]->getCluster();
        if(a_c->mVisit || a_c==mCluster0) continue;
	a_c->mVisit=true;
	found=randomCycleOfClusters(a_c, start, depth-1, cycle);
	a_c->mVisit=false;
	if(found) 
	{ 
	  cycle->push_back(a_c);
	  break;
	}
     }//for
   } //else
   delete cces;
   return found;
}*/


bool
ClusterGraph::randomCycleOfClusters(Cluster* to_search, Cluster* start, int depth, vector<Cluster* >* cycle)
{
   vector<ClusterCounterElement* >* cces=to_search->getAdjacentClusters()->getSplitCopy();
   
   int size=to_search->getAdjacentClusters()->getSplitIndex();
   if(depth==0)
   {
     
     for(int i=0; i<size; i++)
     {
         if(i>=to_search->getAdjacentClusters()->getSize()) break;
    	 if(((to_search->getAdjacentClusters()))->getData(i)->getCluster()==start)   
	 {
	    cycle->push_back(start);
            dynamicCycle(cycle);
            cycle->pop_back();	    
	    break;
	 }
     }//for
   }
   else 
   {
     for(int i=0; i<size; i++)
     {
        if(i>=to_search->getAdjacentClusters()->getSize()) break;
       ClusterCounterElement* cce=to_search->getAdjacentClusters()->getData(i);
	Cluster* a_c=cce->getCluster();
        if(a_c->mVisit || a_c==mCluster0 || cce->getCounter()<=_DYNAMIC_SIZE_THRES_) continue;
	a_c->mVisit=true;
	cycle->push_back(a_c);
	bool found=randomCycleOfClusters(a_c, start, depth-1, cycle);
	cycle->pop_back();
	a_c->mVisit=false;
     }
   } //else
   delete cces;
   return true;
}


void
ClusterGraph::dynamicCycle()
{
  int path_len=10;
  int num_clusters=mClusters->getSize();
  
  
   vector<Cluster *>* candidate=new vector<Cluster *>();
   for(int i=0; i<num_clusters; i++)
   {
      Cluster* c=mClusters->getData(i);
      c->mVisit=false;
   }
  
  for(int i=0; i<num_clusters; i++)
  {  
     Cluster* c=mClusters->getData(i);
     if(c->mVisit) continue;
     c->mVisit=true;
     bool rev=randomCycleOfClusters(c, c, path_len-1, candidate); 
     c->mVisit=false;
     candidate->clear();
   }
   delete candidate;
   removeEmptyClusters();
}


void
ClusterGraph::dynamicCycle(vector<Cluster*>* cycle)
{
   int len=cycle->size();
   bool empty=false;
   //cout<<"*********************"<<endl;
   for(int i=0; i<len; i++)
   {
      (*cycle)[i]->retrieveDynamicList((*cycle)[(i+1)%len]);
       if((*cycle)[i]->getDynamicList()->size()==0)
       {
          empty=true;
       }
   }//for
   
   if(!empty)
 {
   for(int i=0; i<len; i++)
   {
     (*cycle)[i]->createCycleMat((*cycle)[(i-1+len)%len], 0, false);
   }//for



   int size=(*cycle)[0]->getDynamicList()->size();
   for(int i=1; i<len; i++)
   {  
      (*cycle)[i]->fillupCycleTable((*cycle)[i-1], size, false);
   }//for
   (*cycle)[0]->fillupCycleTable((*cycle)[len-1], size, true);

   int maxIndex=0;

   double credit=(*cycle)[0]->getCycleCredit((*cycle)[len-1]);
 //  if(credit>0 && len==3)
//   {
 //     cout<<"***************************credit increased by"<<credit<<endl;
   //   for(int i=0; i<len; i++)  (*cycle)[i]->print();
   
 //  }//
   bool good=true;
   for(int i=len; i>0; i--)
   {
    if(i%len==0)
    {
      maxIndex=(*cycle)[0]->extractTheBestMovement((*cycle)[1], (*cycle)[len-1]);
    }
    else
    {
       if(maxIndex/(1<<size)==0)  {  good=false; }
       maxIndex=(*cycle)[i]->extractTheBestMovement(maxIndex, (*cycle)[(i+1)%len], size);
    }
   }

  // if(good && credit > 0 && len==3)  cout<<"good case"<<endl;
  }
   for(int i=0; i<len; i++)
   {
      (*cycle)[i]->destroyMatrix();
      (*cycle)[i]->mVisit=false;
   }
}

void
ClusterGraph::swapTwo()
{
  int path_len=2;
  int num_clusters=mClusters->getSize();


   vector<Cluster *>* candidate=new vector<Cluster *>();
   for(int i=0; i<num_clusters; i++)
   {
      Cluster* c=mClusters->getData(i);
      c->mVisit=false;
   }

  for(int i=0; i<num_clusters; i++)
  {
     Cluster* c=mClusters->getData(i);
     c->mSwapAltered=false; 
     vector<ClusterCounterElement* >* cces=c->getAdjacentClusters()->getCopy();
     int size=cces->size();
      for(int j=0; j<size; j++)
     {
	vector<ClusterCounterElement* >* cces_temp=c->getAdjacentClusters()->getData();
	if(c->getSize()==0 || (*cces)[j]!=(*cces_temp)[j]) break;
	//if((*cces)[j]->getCounter()<=1) break;
	if((*cces)[j]->getCounter()<=1) continue;
        Cluster* c2=(*cces)[j]->getCluster();
        if(!c->mSwapAltered && !c2->mSwapAltered) continue;
        candidate->push_back(c);
        candidate->push_back(c2);
        dynamicCycle(candidate);
        candidate->clear();
     }
     delete  cces;
   }
   delete candidate;
   removeEmptyClusters();
}

void
ClusterGraph::removeEmptyClusters()
{
  int num_clusters=mClusters->getSize();
  vector<Cluster *>* copy=mClusters->getCopy();
  for(int i=0; i<num_clusters; i++)
  {
     if((*copy)[i]->getSize()==0)
     {
        mClusters->remove((*copy)[i]);
	delete (*copy)[i];
     }
  }
  delete copy;
}
void
ClusterGraph::removeBadVertices()
{
  int num_clusters=mClusters->getSize();
  vector<Cluster *>* copy=mClusters->getCopy();
  for(int i=0; i<num_clusters; i++)
  {
     (*copy)[i]->removeBadVertices(mCluster0);
  }
  delete copy;
}

void
ClusterGraph::splitClusters()
{
  int num_clusters=mClusters->getSize();
  vector<Cluster *>* copy=mClusters->getCopy();
  for(int i=0; i<num_clusters; i++)
  {
        Cluster* rev=(*copy)[i]->splitCluster();
	if(rev) mClusters->insert(rev);
  }
  delete copy;
}

void
ClusterGraph::moveSingleVertexLoop()
{
    double c_temp;
    double initScore=mCredit;
    do 
    {
      c_temp=mCredit; 
      optimize();
      computeCredit();
    } 
    while(mCredit>c_temp);
    if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mMoveSingleScore+=mCredit-max_double(initScore, mPreviousBadScore);
    } 
}

void
ClusterGraph::mergeClusterLoop()
{
    double c_temp;
    double initScore=mCredit;
    do
    {
        c_temp=mCredit;
	mergeClusters();
	computeCredit();
    } 
    while(mCredit>c_temp);
    if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mMergeScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}
void
ClusterGraph::splitClusterLoop()
{
       double c_temp;
    double initScore=mCredit;
       do
      {             
        c_temp=mCredit;             
	partitionSingleLinkedClusters();
	splitClusters();
	computeCredit();                                  
      }                                         
      while(mCredit>c_temp);
    if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mSplitScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}

void
ClusterGraph::extractBadVertexLoop()
{
   double c_temp;
   double initScore=mCredit;
   if(priorBadScore>mCredit)
   {
     bad_counter_g++;
   }
   else
   {
     bad_counter_g=0;
     priorBadScore=mCredit;
   }
   if(bad_counter_g>2) return;

   do
   {
      c_temp=mCredit;
      //if(bad_counter_g<15) 
         extractBadVertex();
      removeBadVertices();
      cluster();
      computeCredit();

     if(mCredit>max_double(initScore, mPreviousBadScore))
     {
       mRemoveBadScore+=mCredit-max_double(initScore, mPreviousBadScore);
     }
   }
   while(mCredit>c_temp);
   mPreviousBadScore=mCredit;

}

void
ClusterGraph::dynamicPathLoop()
{
   double c_temp;
    double initScore=mCredit;
  //do
   //{
       c_temp=mCredit;
      // dynamicPath();
   //    dynamicPath();
   //    dynamicPath();
       dynamicPath();
       computeCredit();
   //}
  // while(mCredit>c_temp);
   if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mDynamicPathScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}

void
ClusterGraph::dynamicCycleLoop()
{
   double c_temp;
    double initScore=mCredit;
  // do
  // {
       c_temp=mCredit;
       dynamicCycle();
       //dynamicCycle();
       //dynamicCycle();
       //dynamicCycle();
       computeCredit();
  // }
  // while(mCredit>c_temp);
  if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mDynamicCycleScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}

void
ClusterGraph::pullOverPartialClusterLoop()
{
      double c_temp;
    double initScore=mCredit;
      do
      {
        c_temp=mCredit;
	pullOverPartialCluster();
	computeCredit();
     }
     while(mCredit>c_temp);
    if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mStarScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}

void 
ClusterGraph::swapTwoLoop()
{
   double c_temp;
    double initScore=mCredit;
   do
   {                             
         c_temp=mCredit;                                     
	 swapTwo();
	 computeCredit();
   }   
   while(mCredit>c_temp);
   if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mDynamicCycleScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}

void 
ClusterGraph::starSplitLoop()
{
   double c_temp;
    double initScore=mCredit;
   do
   {                             
         c_temp=mCredit;                                     
	 starSplit();
	 computeCredit();
   }   
   while(mCredit>c_temp);
   if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mStarScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}

void 
ClusterGraph::starPullLoop()
{
   double c_temp;
    double initScore=mCredit;
   do
   {                             
         c_temp=mCredit;                                     
	 starPull();
	 computeCredit();
   }   
   while(mCredit>c_temp);

   if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mStarScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}

	
void
ClusterGraph::exchangeWithNeighborLoop()
{
   double c_temp;
    double initScore=mCredit;
   do
   {                             
        c_temp=mCredit;                                     
	exchangeWithNeighbor();
	computeCredit();
   }   
   while(mCredit>c_temp);
   //if(mCredit<c_temp) cout<<" error 4"<<endl;
    if(mCredit>max_double(initScore, mPreviousBadScore))
    {
       mDynamicPathScore+=mCredit-max_double(initScore, mPreviousBadScore);
    }
}

void
ClusterGraph::PerformClustering()
{
    mInitialPartitionScore=0;
    mDynamicPathScore=0;
    mDynamicCycleScore=0; 
    mSplitScore=0;
    mMergeScore=0;
    mRemoveBadScore=0;
    mStarScore=0; 
    mMoveSingleScore=0;	
    mPreviousBadScore=0;

   initializePartition();
   computeCredit();
   double c=mCredit;
   mInitialPartitionScore=c;

   int counter=0;
   bad_counter_g=0;
   do
   {
      c=mCredit;
      moveSingleVertexLoop();
      mMoveSingleScore+=(mCredit-c);

      mergeClusterLoop();
      //dynamicCycleLoop();
      extractBadVertexLoop();
      splitClusterLoop();
   } while(mCredit>c);
   bad_counter_g=0;   
   do
   {
      c=mCredit;
      moveSingleVertexLoop();
      extractBadVertexLoop();
      mergeClusterLoop();
      starSplitLoop();
      starPullLoop();
      extractBadVertexLoop();
      splitClusterLoop();
      exchangeWithNeighborLoop();
      swapTwoLoop();
     pullOverPartialClusterLoop(); 
   } while(mCredit>c);

   bad_counter_g=0;
   do
   {
      c=mCredit;
      extractBadVertexLoop();  
      moveSingleVertexLoop();
      mergeClusterLoop(); 
      splitClusterLoop(); 
      starSplitLoop();
      starPullLoop();
      pullOverPartialClusterLoop(); 
      swapTwoLoop();
      exchangeWithNeighborLoop(); 
     //extractBadVertexLoop();  
      //dynamicCycleLoop();
      //dynamicPathLoop();
   } while(mCredit>c);
//   print();

     double tot_credit=mInitialPartitionScore+
    mDynamicPathScore+
    mDynamicCycleScore+ 
    mSplitScore+
    mMergeScore+
    mRemoveBadScore+
    mStarScore+
    mMoveSingleScore;
                         
}
