#ifndef _CLUSTER_GRAPH_H_
#define _CLUSTER_GRAPH_H_

#include "ClusterBase.h"
#include "AdjGraph.h"
#include "Cluster.h"
class ClusterGraph
{
    public:
      static double mAlpha; 
      static int HASH_TABLE_SIZE; 
    protected:
      ArrayList<Cluster *>* mClusters;
      Cluster* mCluster0;
      vector<ClusterVertex *>* mVertices;
      AdjGraph* mGraph;
      void initialize();
      void setMoveSingleFlags();
      void findBestClusterForVertex(ClusterVertex* v);
      void partitionVertex(ClusterVertex* v);
      void destroyCluster(Cluster* c);
      void clearCluster(Cluster* c);
      void breathLevelKVertex(vector<ClusterVertex *>* vertices, ClusterVertex* v, int k);
      vector<Cluster* >* partitionNeighbors(ClusterVertex* v, int k);
      vector<Cluster *>* partitionNeighbors(ClusterVertex* v);
      double ifInsert(AdjacentElement* e, Cluster* c);
      void moveVertex(ClusterVertex* v);
      Cluster* mergeNeighbors(Cluster* c);
      void reclustering(vector<Cluster*>* cs);
      void pullOverPartialCluster(Cluster* c);
      bool dynamicPath(vector<Cluster*>* path);
      void dynamicCycle(vector<Cluster*>* cycle);
      bool randomCycleOfClusters(Cluster* to_search, Cluster* start, int depth, vector<Cluster *>* cycle);
      void removeEmptyClusters();
      double mCredit;
      void exchangeWithNeighbor();
      void moveSingleVertexLoop();
      void mergeClusterLoop();
      void splitClusterLoop();
      void extractBadVertexLoop();
      void dynamicPathLoop();
      void dynamicCycleLoop();
      void pullOverPartialClusterLoop();
      void swapTwoLoop();
      void starSplit();
      void starSplitLoop();
      void starPull();
      void starPullLoop();
      Cluster* starPull(ClusterVertex* cv);
      Cluster* starSplit(ClusterVertex* cv);
      void inTriangle();
      void inTriangle(ClusterVertex* cv);
      void exchangeWithNeighborLoop();
      bool dynamicPathIter(Cluster* c, int depth, vector<Cluster* >* candidate);
      double starSplitCredit(ClusterVertex* cv, int nodes, int len, vector<ClusterVertex* >* cvs, 
		      SimpleHash<Cluster*, int>* hash);
      double starPullCredit(ClusterVertex* cv, int nodes, int len, vector<ClusterVertex* >* cvs, 
		      SimpleHash<Cluster*, int>* hash);

      double mInitialPartitionScore;
      double mDynamicPathScore;
      double mDynamicCycleScore; 
      double mSplitScore;
      double mMergeScore;
      double mRemoveBadScore;
      double mStarScore; 
      double mMoveSingleScore;
      double mPreviousBadScore;
   public:
      ClusterGraph(AdjGraph* g, double alpha);
      ~ClusterGraph();
      void initializePartition();
      void print();
      void computeCredit();
      void partitionSingleLinkedClusters();
      void moveSingleVertex();
      void mergeClusters(); 
      void extractBadVertex();
      void optimize();     
      void reclustering();
      void cluster();
      void pullOverPartialCluster(); 
      void dynamicPath();
      void dynamicCycle();
      void swapTwo();
      void removeBadVertices();
      void splitClusters();
      vector<Cluster *>* getClusterCopy() { return mClusters->getCopy(); }
      void PerformClustering();
};


#endif
