#ifndef IBDRW
#define IBDRW


#include <algorithm>
#include <vector>
#include <map>
#include <queue>
#include <set>
#include <iostream>




using namespace std;


//double default_test = 0.0;  // mean of chi^2
//double default_test = sqrt(2); // stddev of chi^2
//double default_test = 0.08; 
double default_test = 1.0; 


enum Gender {UNKNOWN, MALE, FEMALE};

enum IBDIndicator {UNCOLORED, ACTIVE1, ACTIVE2, INACTIVE, BOTH, ACTIVE, SHADDOW_ACTIVE, SHADDOW_INACTIVE};


class Indiv
{
public:
  Indiv(int _indiv_index,int _generation){ 
    indiv_index = _indiv_index; generation = _generation; gender = MALE;
    descendants = map<int,int>();
  };
  Indiv(const Indiv& j);
  const Indiv& operator= (const Indiv& j);

  void PushDescendant(int d){descendants[d]=1;};
  void PushDescentList(const Indiv& j);

  void InsertChildren(const Indiv& j);
  void PushChild(int c){children.push_back(c);};

  void PushParent(int p){parents.push_back(p);};
  void PushParentList(const Indiv& j);

  void GetDescentList(vector<int>& d_split);
  void GetDescentList(set<int>& d_split);
  bool HasDescendant(int d);

  bool HasChild(int c);
  void GetChildList(vector<int>& child_list);

  void GetParents(int& f, int& m); // only two parents

  void SetGender(Gender g){gender = g;};
  void setAffect(int a){affection = a;};

  void ResetColor(){color = 0;};
  void Print();
  void PrintPed(vector<string>& names, FILE* fp);

public:
  int indiv_index;
  int old_index;
  int generation;
  Gender gender;
  int affection;

  map<int,int> descendants;
  vector<int> children;
  vector<int> parents;

  unsigned int color;

};


Indiv::Indiv(const Indiv& j)
{
  color = 0;
  indiv_index = j.indiv_index;
  gender = j.gender;
  generation = j.generation;
  PushDescentList(j);
  InsertChildren(j);
  PushParentList(j);
}
const Indiv& Indiv::operator= (const Indiv& j)
{
  color = 0;
  indiv_index = j.indiv_index;
  gender = j.gender;
  generation = j.generation;
  PushDescentList(j);
  InsertChildren(j);
  PushParentList(j);
  return *this;
}
void Indiv::PushDescentList(const Indiv& j){
  map<int,int>::const_iterator mi = j.descendants.begin();
  while(mi != j.descendants.end()){
    descendants[mi->first] = 1;
    mi++;
  }
}
void Indiv::InsertChildren(const Indiv& j)
{
  for (unsigned int c = 0;  c < j.children.size();  c++){
    children.push_back(j.children[c]);
  }
}
void Indiv::PushParentList(const Indiv& j)
{
  if (j.parents.size() > 2)
    printf("\nWARNING: too many parents for person %i\n", indiv_index);
  for (unsigned int p = 0;  p < j.parents.size();  p++)
    parents.push_back(j.parents[p]);
}

void Indiv::GetDescentList(vector<int>& d_split){
  if (descendants.size()==0)
    return;
  map<int,int>::const_iterator mi = descendants.begin();
  while(mi != descendants.end()){
    d_split.push_back(mi->first);
    mi++;
  }
}
void Indiv::GetDescentList(set<int>& d_split){
  if (descendants.size()==0)
    return;
  map<int,int>::const_iterator mi = descendants.begin();
  while(mi != descendants.end()){
    d_split.insert(mi->first);
    mi++;
  }
}

bool Indiv::HasDescendant(int d){
  map<int,int>::const_iterator mi = descendants.find(d);
  if (mi != descendants.end()){
    return true;
  }
  return false;
}
bool Indiv::HasChild(int c){
  vector<int>::const_iterator vi = find(children.begin(), children.end(), c);
  if (vi != children.end()){
    return true;
  }
  return false;
}
void Indiv::GetChildList(vector<int>& child_list)
{
  vector<int>::const_iterator vi = children.begin();
  while(vi != children.end()){
    child_list.push_back(*vi);
    vi++;
  }
}

void Indiv::GetParents(int& p1, int& p2)
{
  if (parents.size() > 2)
    printf("\nWARNING: too many parents for person %i\n", indiv_index);

  p1 = parents[0];
  if (parents.size() == 2)
    p2 = parents[1];
}
void Indiv::Print()
{
  printf("  Person %i, generation %i, color %i, gender %i\n", indiv_index, generation, color, gender);
  printf("      Parents:     ");
  for (unsigned int i = 0;  i < parents.size(); i++)
    printf("%i, ", parents[i]);
  printf("\n      Children:    ");
  for (unsigned int i = 0;  i < children.size(); i++)
    printf("%i, ", children[i]);
  printf("\n      Descendants: ");
  map<int,int>::const_iterator mi = descendants.begin();
  while(mi != descendants.end()){
    printf("%i,", mi->first);
    mi++;
  }
  printf("\n");
}

void Indiv::PrintPed(vector<string>& names, FILE* fp)
{
  if (indiv_index <= names.size())
    {
      char* s = (char *) names[indiv_index-1].c_str();
      fprintf(fp, "1 %s ", s);
      
    }
  else
    {
      fprintf(fp, "%i:", indiv_index);
    }


  if (parents.size() > 0)
    {
      for (unsigned int i = 0;  i < parents.size(); i++)
	  {
	  if(i==0)
	  {
	  fprintf(fp, "%i-", parents[i]);
	  }
	  else
	  {
	fprintf(fp, "%i ", parents[i]);
	}
	}	
    }
  else
    {
      fprintf(fp, "0 0 ");
    }

  fprintf(fp, "generation:%i", generation);


  // affection status based on descendants
/*   bool isCEU = true; */
/*   bool isYRI = true; */
/*   //int last_CEU = 88; */
/*   int last_CEU = 11; */
/*   for (map<int,int>::iterator mi = descendants.begin(); */
/*        mi != descendants.end(); */
/*        mi++) */
/*     { */
/*       if (mi->first <= last_CEU) */
/* 	{ */
/* 	  isYRI = false; */
/* 	} */
/*       if (mi->first > last_CEU) */
/* 	{ */
/* 	  isCEU = false; */
/* 	} */
/*     } */

/*   int fake_aff = 0; */
/*   if (isCEU && !isYRI) */
/*     fake_aff = 1; */
/*   if (isYRI && !isCEU) */
/*     fake_aff = 2; */
/*   fprintf(fp, "%i\n", fake_aff); */

  fprintf(fp, "\n");

}






// integer indexes numbered [1...n]
//
void setParents(vector<Indiv>* pedigree, int i, int m, int f)
{
    (*pedigree)[i].PushParent(m);
    (*pedigree)[i].PushParent(f);

    (*pedigree)[m].PushChild(i);
    (*pedigree)[m].PushDescentList((*pedigree)[i]);

    (*pedigree)[f].PushChild(i);
    (*pedigree)[f].PushDescentList((*pedigree)[i]);
}

// Wontonly delete the children and descent lists of parents m & f
// Delete the parents of i
void unsetParents(vector<Indiv>* pedigree, unsigned int i, unsigned int m, unsigned int f)
{
    (*pedigree)[i].parents.clear();
    (*pedigree)[m].children.clear();
    (*pedigree)[m].descendants.clear();
    (*pedigree)[f].children.clear();
    (*pedigree)[f].descendants.clear();
}

// make a new person
unsigned int makeUntypedPerson(vector<Indiv>* pedigree, int generation)
{
  unsigned int index = pedigree->size();
  Indiv new_person(index,generation);
  pedigree->push_back(new_person);
  return index;
}

unsigned int makeTypedPerson(vector<Indiv>* pedigree, int generation)
{
  unsigned int index = pedigree->size();
  Indiv new_person(index,generation);
  new_person.PushDescendant(index);
  pedigree->push_back(new_person);
  return index;
}




void printSet(set<int> s)
{
  printf("------- begin set ------\n");
  for (set<int>::iterator si = s.begin();  si != s.end();  si++)
    printf("%i, ", *si);
  printf("\n");
  printf("------- end set ------\n");
}

void printPaths(vector<unsigned int>& paths)
{
  printf("------- begin paths ------\n");
  for (unsigned int p = 0;  p < paths.size(); p++)
    printf("%i, ", paths[p]);
  printf("\n");
  printf("------- end paths ------\n");
}


void printPedigree(vector<Indiv>& ped_d_splits)
{
  printf("------- begin pedigree ------\n");
  for (unsigned int p = 0;  p < ped_d_splits.size();  p++)
    {
      ped_d_splits[p].Print();
    }
  printf("------- end pedigree ------\n");
}



void printPedFile(vector<Indiv>& ped_d_splits, vector<string>& names, FILE* fp)
{
  for (unsigned int p = 1;  p < ped_d_splits.size();  p++)
    {
      ped_d_splits[p].PrintPed(names, fp);
    }
}



void foundersOfTyped(vector<Indiv>& pedigree, 
		     vector<set<int> >& typed_to_founder)
{
  int gen = -1;
  int prev_gen = -1;
  for (int c = pedigree.size()-1; c >= 0; c--)
    {
      // exit loop
      if (gen == -1)
	gen = pedigree[c].generation;
      prev_gen = gen;
      gen = pedigree[c].generation;
      if (gen != prev_gen)
	break;


      // Add this founder to the founder list for
      // each descendant in the d-split
      vector<int> d_split;
      pedigree[c].GetDescentList(d_split);
      for (unsigned int d = 0;  d < d_split.size();  d++)
	{
	  int element = d_split[d];
	  while((unsigned) element >= typed_to_founder.size())
	    typed_to_founder.push_back(set<int>());

	  typed_to_founder[element].insert(c);
	}
    }

  // print the founders
/*   for (unsigned int d = 0;  d < typed_to_founder.size();  d++) */
/*     { */
/*       printf("   founder sets %i: ", d); */
/*       set<int>::const_iterator si; */
/*       for (si = typed_to_founder[d].begin(); */
/* 	   si != typed_to_founder[d].end(); */
/* 	   si++) */
/* 	printf("%i ", *si); */
/*       printf("\n"); */
/*     } */
}




void resetColor(vector<Indiv>& pedigree)
{
  for (unsigned int i = 0;  i < pedigree.size();  i++){
      pedigree[i].ResetColor();
    }
}



void colorPedigree(vector<Indiv>& pedigree, int i, int j, unsigned int color)
{
  // List of people to recurse over
  queue<int> ancestors;

  // Loop over all the 
  int last_generation = pedigree[pedigree.size()-1].generation;
  for (int a = pedigree.size()-1;  a >= 0;  a--)
    {
      if (last_generation != pedigree[a].generation)
	break;

      bool hasI = pedigree[a].HasDescendant(i);
      bool hasJ = pedigree[a].HasDescendant(j);
      if (hasI && hasJ){
	ancestors.push(a);
	pedigree[a].color = color;
      }
    }

  while (!ancestors.empty())
    {
      int index = ancestors.front();
      ancestors.pop();

      for (unsigned int c = 0;  c < pedigree[index].children.size();  c++)
	{
	  int child = pedigree[index].children[c];
	  bool hasI = pedigree[child].HasDescendant(i);
	  bool hasJ = pedigree[child].HasDescendant(j);

	  if (hasI || hasJ){
	    ancestors.push(child);
	    pedigree[child].color = color;
	  }
	}
    }
}






// do a dfs search to find min # of meioses separating i and j
unsigned int minMeioses(vector<Indiv>& pedigree, 
			int i, int j, 
			unsigned int color, 
			int depth, 
			int& min_depth)
{
  //printf("minMeioses(ped, %i,%i, %i, %i, %i)\n", i,j,color, depth, min_depth);

  if (i == j)
    {
      if (depth < min_depth)
	min_depth = depth;

      return min_depth;
    }

  for(unsigned int p = 0 ;  p < pedigree[i].parents.size(); p++)
    {
      int p_idx = pedigree[i].parents[p];
      int m;
      if (pedigree[p_idx].color == color){
	pedigree[p_idx].color = 0; // to avoid coming back this way
	m = minMeioses(pedigree, p_idx, j, color, depth+1, min_depth);
	if (m < min_depth)
	  min_depth = m;
      }
    }
  for (unsigned int c = 0;  c < pedigree[i].children.size(); c++)
    {
      int c_idx = pedigree[i].children[c];
      int m;
      if (pedigree[c_idx].color == color){
	pedigree[c_idx].color = 0; // to avoid coming back this way
	m = minMeioses(pedigree, c_idx, j, color, depth+1, min_depth);
	if (m < min_depth)
	  min_depth = m;
      }
    }


  return min_depth;
}




// Assumes that the generations are regular,
// i.e. that individuals mate only with other individuals in their generation
//
unsigned int  recurseIBD(vector<unsigned int>& all_paths,
			 vector<Indiv>* pedigree,
			 unsigned int color, 
			 bool hasIBD,
			 vector<int>& partial_path, 
			 int* status,
			 int size,
			 int generation)
{
  //printf("recurseIBD, hasIBD %i, gen %i\n", hasIBD, generation);

  // list of people in this generation
  // having colored meioses
  vector<int> gen_indices;
  int count_colored = 0;
  for (unsigned int i = 1; i < pedigree->size();  i++)
    {
	if ((*pedigree)[i].color == color)
	  {
	    for (unsigned int p = 0;  p < (*pedigree)[i].parents.size(); p++)
	      {
		unsigned int p_idx = (*pedigree)[i].parents[p];
		if( (*pedigree)[p_idx].color == color)
		  count_colored++;
	      }

	    if ((*pedigree)[i].generation == generation)
	      {

		gen_indices.push_back(i);
		if ((*pedigree)[i].parents.size() > 0)
		  {
		    partial_path[2*i] = 0;
		    partial_path[2*i+1] = 0;
		  }
	      }
	  }
/*       if (generation == 1 && (*pedigree)[i].generation == 1) */
/*       { */
/* 	printf("%i] p(%i,%i),s(%i,%i), ", i, partial_path[2*i], partial_path[2*i+1], status[2*i], status[2*i+1]); */
/*       } */
    }
  //printf("\n");
  //printf("---- end top\n");



  // set the status to that induced by children
  bool newHasIBD = hasIBD;
  if (generation != 1)
    {
      vector<int> g_status(6,0);
      bool hasActive = false;

      for (unsigned int j = 0; j < gen_indices.size();  j++)
	{
	  unsigned int i = gen_indices[j];
	  int index = 2*i;
	  int indiv = i;

	  if ((*pedigree)[indiv].color != color)
	    continue;
	  

	  //printf("parent %i, ", i);
	  // parent allele (parent is indiv)
	  // for each child allele C_allele's choice of a parent
	  for (unsigned int allele = 0;  allele < 2; allele++)
	    {
	      //printf("allele %i, ", allele);
	      status[index+allele] = UNCOLORED;
	      if ((*pedigree)[indiv].color == color)
		status[index+allele] = INACTIVE;
	      
	      // could have more than one status, so track all of them
	      vector<int> c_status(6, 0);
	      for (unsigned int j = 0;  j < (*pedigree)[indiv].children.size(); j++)
		{
		  unsigned int c = (*pedigree)[indiv].children[j];
		  //printf("child %i, ", c);
		  for (unsigned int c_allele = 0;  c_allele < (*pedigree)[c].parents.size(); c_allele++)
		    {
		      //printf("#### children alleles (p %i) #######\n", (*pedigree)[c].parents[c_allele]);
		      //printf("c_a %i, ", c_allele);
		      
		      if ((unsigned) partial_path[2*c+c_allele] == allele)
		      {
			  if (indiv == (*pedigree)[c].parents[c_allele])
			    {
			      //printf("#### parent allele #######\n");
			      int s = status[2*c+c_allele];
			      c_status[s]++;
			      g_status[s]++;
			      //printf(" match s %i, ", s);
			    }
			}
		    }
		  
		  if (c_status[ACTIVE1] > 0 && c_status[ACTIVE2] > 0 ){
		    status[index+allele] = BOTH;
		  } else if (c_status[ACTIVE1] > 0) {
		    status[index+allele] = ACTIVE1;
		  } else if (c_status[ACTIVE2] > 0 ) {
		    status[index+allele] = ACTIVE2;
		  }
		  
		  if (status[index+allele] == BOTH)
		    newHasIBD = true;
		}

/* 	      printf("   (hIBD %i) person 2*%i+%i, status: ", newHasIBD, indiv, allele); */
/* 	      for (unsigned int c = 0; c < c_status.size(); c++) */
/* 		if(c_status[c] > 0) */
/* 		  printf("%i] %i, ", c, c_status[c]); */
/* 	      printf("\n"); */
	      

	    }
	  //printf("\n");
		     


	  
	  //if ((*pedigree)[i].generation == generation)
	  //{
	  //  printf("%i] s(%i,%i), ", i, status[2*i], status[2*i+1]);
	  //}
	  //printf("\n");
	}
      //printf("\n");

      if (g_status[BOTH] > 0 || (g_status[ACTIVE1] > 0 && g_status[ACTIVE2] > 0 )){
	hasActive = true;
      } else {
	//printf("RETURN due to no active in generation %i\n", generation);
	return 0;
      }



    }





  // Base Case
  int last = pedigree->size()-1;
  if (last >= 0 && generation == (*pedigree)[last].generation)
    {
      // print out the IBD inheritance vector

      //if (newHasIBD)
      //printf("BaseCase: \n");
      unsigned int path_index = 0;
      unsigned int base = 1;
      unsigned int num_meioses = 0;
      for (unsigned int i = 0;  i < partial_path.size();  i++)
	{
	  //if (partial_path[i] < 0)
	  //continue;

	  int index = i;
	  int indiv = (int) index / 2;
	  int allele = index % 2;

	  if ((*pedigree)[indiv].color != color)
	    continue;
	  if ((*pedigree)[indiv].parents.size() == 0)
	    continue;
	  int p = (*pedigree)[indiv].parents[allele];
	  if ((*pedigree)[p].color != color)
	    continue;

	  //if (newHasIBD)
	  //printf("+ %i(%i) ", partial_path[i]*base, indiv);
	    //printf("i%i,a%i=%i,s%i ", indiv, allele, partial_path[i], status[2*indiv+allele]);

	  num_meioses++;
	  path_index += base * partial_path[i];
	  base *= 2;
	}


      if (newHasIBD){
	//printf(" => %i\n", path_index);
	all_paths.push_back(path_index);
      } else {
	//printf("\n");
      }

      unsigned int max = (unsigned int) pow(2,count_colored);
      if (path_index > max)
	{
	  printf("FATAL ERROR: out of bounds path index, colored %i, max %i < %i actual\n", count_colored, max, path_index);
	  exit(-432);
	}



      return num_meioses;
    }


  // loop over all possible partial path assignments for this generation
  unsigned int num_bits = 2*gen_indices.size();
  //printf("num bits: %i\n", num_bits);
  int num_assignments = (int) pow(2, num_bits);
  int m = 0;
  for (int a = 0;  a < num_assignments;  a++)
    {
      // convert to binary
      vector<int> bits(num_bits, 0);
      int base = 1;
      for (unsigned int i = 0;  i < num_bits;  i++)
	{
	  int mask = a & base;
	  if (mask > 0)
	    bits[i] = 1;
	  else
	    bits[i] = 0;
	  
	  base *= 2;
	}

      // copy into partial path
      for (unsigned int j = 0; j < gen_indices.size();  j++)
	{
	  unsigned int i = gen_indices[j];
	  if (bits[j] > 1){
	    printf("WARNING: invalid bit %i\n", bits[j]);
	    exit(-34);
	  }
	  partial_path[2*i] = bits[2*j];
	  partial_path[2*i+1] = bits[2*j+1];
	}

      // recurse
      m = recurseIBD(all_paths,
		     pedigree, color, newHasIBD, partial_path, 
		     status, size, generation+1);
    }



  return m;
}






// alleles represented as 2*index, and 2*index+1 (first parent, second parent)
unsigned int getInheritance(vector<unsigned int>& all_paths,
			    vector<Indiv>& pedigree, 
			    int i, int j, int color)
{

  // put all the alleles of i,j into the active list
  int size = 2*pedigree.size();
  int* status = new int[size];
  for (int a = 0;  a < size;  a++)
    status[a] = UNCOLORED;

  if (i == j){
    status[2*i] = ACTIVE1;
    status[2*i+1] = ACTIVE2;
  } else {
    status[2*i] = ACTIVE1;
    status[2*i+1] = ACTIVE1;
    status[2*j] = ACTIVE2;
    status[2*j+1] = ACTIVE2;
  }


  //printf("Status: uncol %i, a1 %i, a2 %i, ina %i, both %i\n", UNCOLORED, ACTIVE1, ACTIVE2, INACTIVE, BOTH);


  //printf(" ########### getInheritance() ########### \n");
  vector<int> path(2*pedigree.size(), 0);
  for (unsigned int i = 0; i < pedigree.size();  i++)
    if (pedigree[i].children.size() > 0 && pedigree[i].parents.size() != 0)
      {
	if (pedigree[i].color != (unsigned) color)
	  {
	    //printf("%i %i ", 2*i, 2*i+1);
	    path[2*i] = -1;
	    path[2*i+1] = -1;
	  }
      }
  path[0] = -1;
  path[1] = -1;
  //printf("\n");

  int m = recurseIBD(all_paths, &pedigree, color, 0,
		    path, status, size, 1);

  delete[] status;

  return m;
}









// Assumes that the generations are regular,
// i.e. that individuals mate only with other individuals in their generation
unsigned int  recurseIBD2(vector<unsigned int>& all_paths,
			 vector<Indiv>* pedigree,
			 int color, bool hasIBD,
			 map<int,int>& partial_path, 
			 map<int,int> active, 
			 map<int,int> inactive,
			 map<int,int> shaddow_active,
			 map<int,int> shaddow_inactive)
{

  unsigned int m = 0;

  //printf("recurseIBD(%i)\n", color);


  // update inactive and shaddow_inactive lists
  // loop over both the active and shaddow active lists
  bool isBaseCase = 1;
  map<int,int>::iterator li;
  for (li = active.begin();  li != active.end();  li++)
    {
      // Inactive: formerly active alleles that have no colored parents
      int indiv = (int) li->first / 2;
      unsigned int allele = (unsigned) li->first % 2;

      //printf("  index %i, indiv %i, allele %i\n", li->first, indiv, allele);
      //printf("  ped[%i].parents.size = %i\n",indiv, (*pedigree)[indiv].parents.size());

      int meiosis_color = -1;
      if (allele < (*pedigree)[indiv].parents.size()){
	int parent = (*pedigree)[indiv].parents[allele];
	meiosis_color = (*pedigree)[parent].color;
      }
	  
      if (meiosis_color != color)
	{
	  //printf("Inactive: %i\n", li->first);
	  inactive[li->first] = li->first;
	}
      else
	{
	  isBaseCase = 0;
	}
    }

  for (li = shaddow_active.begin();  li != shaddow_active.end();  li++)
    {
      // Shaddow Inactive: formerly active alleles that have no colored parents
      int indiv = (int) li->first / 2;
      unsigned int allele = (unsigned) li->first % 2;

      int meiosis_color = -1;
      if (allele < (*pedigree)[indiv].parents.size()){
	int parent = (*pedigree)[indiv].parents[allele];
	meiosis_color = (*pedigree)[parent].color;
      }
	  
      if (meiosis_color != color)
	{
	  //printf("Shaddow Inactive: %i\n", li->first);
	  shaddow_inactive[li->first] = li->first;
	}
      else
	{
	  isBaseCase = 0;
	}
    }




  if (active.empty() && shaddow_active.empty())
    isBaseCase = 1;

  // Base Case: when no active or shaddow_active alleles
  if (isBaseCase)
    {
      // print out the IBD inheritance vector
      //printf("BaseCase: \n");
      unsigned int path_index = 0;
      unsigned int base = 1;
      map<int,int>::iterator li;
      for (li = partial_path.begin();  li != partial_path.end();  li++)
	{
	  //int index = li->first;
	  //int indiv = (int) index / 2;
	  //int allele = index % 2;
	  //printf("i%i,a%i=%i, ", indiv, allele, li->second);

	  path_index += base * li->second;
	  base *= 2;
	}


      if (hasIBD){
	//printf(" => %i\n", path_index);
	all_paths.push_back(path_index);
      } else {
	//printf("\n");
      }


      return partial_path.size();
    }




  // loop over all binary assignments to the variables in the active 
  // and shaddow_active lists
  bool notFinished = 1;
  while (notFinished)
    {
      // looping is accomplished by adding 1 and carrying
      bool carry = 1;
      bool wasZero = 1;
      bool newHasIBD = hasIBD;


      // make new active and shaddow active lists
      map<int,int> new_active;
      map<int,int> new_shaddow_active;

      map<int, int>::iterator found;


      // Active alleles
      map<int,int>::iterator li;
      for (li = active.begin();  li != active.end();  li++)
	{
	  int index = li->first;
	  int indiv = (int) index / 2;
	  unsigned int allele = (unsigned) index % 2;

	  //printf("  ped[%i].parents.size = %i\n",indiv, (*pedigree)[indiv].parents.size());
	  
	  int meiosis_color = -1;
	  int parent = -1;
	  if (allele < (*pedigree)[indiv].parents.size()){
	    parent = (*pedigree)[indiv].parents[allele];
	    meiosis_color = (*pedigree)[parent].color;
	  }

	  //printf("  carry = %i\n", carry);
	  //printf("Active: index %i, indiv %i, allele %i, m color %i\n", index, indiv, allele, meiosis_color);


	  if (meiosis_color == color)
	    {
	      // next setting of this meiosis
	      partial_path[index] += carry;
	      carry = partial_path[index] / 2;
	      partial_path[index] = partial_path[index] % 2;

	      //printf("Active: partial_path[%i] = %i\n", index, partial_path[index]);

	      if (partial_path[index] != 0)
		wasZero = 0;

	      int this_allele = 2*parent + partial_path[index];
	      found  = new_active.find(this_allele);
	      if (found != new_active.end())
		newHasIBD = 1;
	      new_active[this_allele] = this_allele;
	      // if this allele is in shaddow list, remove it
	      map<int,int>::iterator found = new_shaddow_active.find(this_allele);
	      if (found != new_shaddow_active.end())
		new_shaddow_active.erase(found);
	      
	      // if other_allele not in active list, add to shaddow list
	      int other_allele = 2*parent + 1 - partial_path[index];
	      found = new_active.find(other_allele);
	      if (found == new_active.end())
		new_shaddow_active[other_allele] = other_allele;
	    }
	}



      // New Shaddow active allele:
      //   if not chosen (currently) from the alternatives for the active list
      //   i.e. for each parent allele not chosen, add it to the inactive list if
      //        it does not appear in the active list, and for each parent allele
      //        chosen, make sure that it is NOT in the inactive list
      //   if it's parent was in the shaddow list, then add it to the new_shaddow_active
      //        but only if it isn't in the new_active


      // Shaddow Alleles
      for (li = shaddow_active.begin();  li != shaddow_active.end();  li++)
	{
	  int index = li->first;
	  int indiv = (int) index / 2;
	  unsigned int allele = (unsigned) index % 2;
	  
	  int meiosis_color = -1;
	  int parent = -1;
	  if (allele < (*pedigree)[indiv].parents.size()){
	    parent = (*pedigree)[indiv].parents[allele];
	    meiosis_color = (*pedigree)[parent].color;
	  }

	  //printf("  carry = %i\n", carry);
	  if (meiosis_color == color)
	    {
	      // next setting of this meiosis
	      partial_path[index] += carry;
	      carry = partial_path[index] / 2;
	      partial_path[index] = partial_path[index] % 2;

	      //printf("Shaddow Active: partial_path[%i] = %i\n", index, partial_path[index]);


	      if (partial_path[index] != 0)
		wasZero = 0;

	      int this_allele = 2*parent + partial_path[index];
	      // if this allele is NOT in the active list, 
	      // add it to the new shaddow list
	      map<int,int>::iterator found = new_active.find(this_allele);
	      if (found == new_active.end())
		new_shaddow_active[this_allele] = this_allele;
	      
	      // if other_allele not in active list, add to shaddow list
	      int other_allele = 2*parent + 1 - partial_path[index];
	      found = new_active.find(other_allele);
	      if (found == new_active.end())
		new_shaddow_active[other_allele] = other_allele;
	    }
	}




      // recurse
      m = recurseIBD2(all_paths,
		     pedigree, color, newHasIBD, partial_path, 
		     new_active, inactive,
		     new_shaddow_active, shaddow_inactive);



      // decide if we are finished already, because we did zero
      if (wasZero)
	notFinished = 0;
      if (active.empty() && shaddow_active.empty())
	notFinished = 0;

    }


  return m;
}




// alleles represented as 2*index, and 2*index+1 (first parent, second parent)
unsigned int getInheritance2(vector<unsigned int>& all_paths,
			    vector<Indiv>& pedigree, 
			    int i, int j, int color)
{
  // put all the alleles of i,j into the active list

  map<int,int> active;
  active[2*i] = 2*i;
  active[2*i+1] = 2*i+1;
  active[2*j] = 2*j;
  active[2*j+1] = 2*j+1;

  map<int,int> empty;

  map<int,int> path;

  return recurseIBD2(all_paths, &pedigree, color, 0,
		    path, active, empty, empty, empty);
}













unsigned int hammingDistance(unsigned int basis, unsigned int x, unsigned int y)
{
  unsigned int xor_value = x ^ y;

  //printf("    xor %i + %i = %i\n", x, y, xor_value);

  int dist = 0;
  int base = 1;
  //printf("basis sum: ");
  for (unsigned int i = 0;  i < basis;  i++)
    {
      int mask = xor_value & base;
      if (mask > 0){
	//printf(" + 1 (base %i, i %i)  ", base, i);
	dist += 1;
      }
      base *= 2;
    }
  //printf("\n");

  //printf("hamming %i to %i = %i\n", x, y, dist);

  return dist;
}



void getTransitionRates(vector<vector<int> >& rates,
			vector<int>& sum,
			vector<unsigned int>& all_paths, 
			unsigned int m) // max distance, i.e. # meioses
{
  // compute upper triangular matrix of transition rates
  // between all_paths and all_paths


  // print column lables
  //for (int j = 0;  j < all_paths.size();  j++)
  //  printf("%i ", all_paths[j]);
  //printf("\n-----------------------------------------------------------\n");


  for (unsigned int i = 0;  i < all_paths.size();  i++)
    {
      // row labels
      //printf("%i| ", all_paths[i]);
      for (unsigned int j = i;  j < all_paths.size();  j++)
	{
	  unsigned int dist = hammingDistance(m, all_paths[i], all_paths[j]);

	  // if we make a Piosson-style transition matrix
	  // i.e. just mark which IBD vectors are 1-step away from each other
	  rates[i][j] = 0;
	  if (0 < dist && dist <= 1)
	    {
	      rates[i][j] = 1;
	      sum[i] += 1;
	      if (j != i)
		sum[j] += 1;
	      //printf("%i,%i dist %i\n", i,j,dist);
	    }
	}
      //printf("\n");
    }

  int size_ibd = all_paths.size();
  for (unsigned int i = 0;  i < all_paths.size();  i++)
    {
      //printf("%i] %i (sum: %i)\n", all_paths[i], m-sum[i], sum[i]);
      
      // Poisson-style transition matrix
      rates[i][size_ibd] = m-sum[i];

      // HMM-style transition matrix
      //rates[i][size_ibd] = 1-sum[i];
    }
}









void samplePoissonRandomWalk(vector<double>& length,
			     //map<unsigned int, double>& prob,
			     vector<unsigned int>& paths, // inheritance paths
			     vector<vector<int> >& neighbors,
			     double lambda,
			     unsigned int m,  // number of meioses
			     unsigned int num_samples)
{

  int size = paths.size();

  if (size == 0)
    return;
  

  for (unsigned int i=0;  i < num_samples;  i++)
    {
      //printf("r.w. %i] ", i);

      // draw start state
      double r = drand48();
      unsigned int start_index = ((int) (r * 10000.0)) % size;
      unsigned int steps = 0;

      for (int l=0;  l < 2;  l++)
	{
	  unsigned int index = start_index;

	  // as long as we are still in an IBD state, keep taking
	  // steps whose length in the genome is exponentially distributed
	  // (where each step moves from a state to a neighboring state)
	  bool notAbsorbed = 1;
	  while(notAbsorbed)
	    {
	      //printf("%i, ", paths[index]);

	      // select the neighbor by picking a meiosis to flip
	      unsigned int n = ((int) (drand48() * 10000.0)) % m;

	      // check for absorption
	      if (n <= (unsigned int) neighbors[index][size])
		{
		  //printf("a.");
		  notAbsorbed = 0;
		}
	      else
		{
		  int p = neighbors[index][size];
		  int t;
		  for (int j = size-1; j >= 0;  j--)
		    {
		      n -= p;
		      t = neighbors[index][j];
		      if (index > (unsigned) j)
			t = neighbors[j][index];
		      if (n <= (unsigned int) t)
			{
			  index = j;
			  break;
			}
		      p = t;
		    } // end pick state
		}
	      steps++;
	    } // while random walk not absorbed
	  //printf("\n");

	} // end for both directions from center


      //prob[steps] += 1.0;
      double len = 0.0;
      for (unsigned int s = 0; s < steps; s++){
	double r = drand48();
	len -= log(r);
	//printf("- log(%f) ",r);
      }
      len = len / (m*lambda);
      //printf(" / (%i*%f)", m, lambda);

      length.push_back(len);
      //printf("  = %f\n", len);

    } // end sample multiple random walk


  //printf("Length of paths:\n");
  //for (int i = 0;  i < length.size();  i++)
  //  {
  //    printf("  %i %f\n", i, length[i]);
  //  }

/*   printf("Number of Exponential-Length Steps:\n"); */
/*   map<unsigned int,double>::iterator li; */
/*   for (li = prob.begin();  li != prob.end();  li++) */
/*     { */
/*       printf("%i %f\n", li->first, li->second/num_samples); */
/*     } */



}



// Compute the coefficient for the test
double testCoefficient(map<unsigned int, double>& prob, 
		       double observed_sharing)

{
  // i,j is fixed for the values of the parameters
  // (t_{i,j} - T_{i,j})^2 / var(t_{ij} - T_{i,j})
  // T_{i,j} = E_{prob}[x]
  // var(t_{i,j} - T_{i,j}) = E_{prob}[(x-T_{i,j})^2]

  if (prob.size() == 0)
    return 0.0;

  // compute expected length
  double T = 0.0;
  for (unsigned int x = 0; x < prob.size();  x++)
    {
      T += x*prob[x];
    }
  T /= prob.size();

  // compute variance
  double var = 0.0;
  for (unsigned int x = 0; x < prob.size();  x++)
    {
      var += pow((x-T),2) * prob[x];
    }
  var /= prob.size();
  

  // compute the test statistic
  double test = pow((observed_sharing - T),2);
  test /= var;
  //printf("   test (%f-%f^2)/%f = %f\n", observed_sharing, T, var, test);


  return test;
}




// Compute the coefficient for the test
double testCoefficient(vector<double>& length, 
		       double observed_sharing)

{
  // i,j is fixed for the values of the parameters
  // (t_{i,j} - T_{i,j})^2 / var(t_{ij} - T_{i,j})
  // T_{i,j} = E_{prob}[x]
  // var(t_{i,j} - T_{i,j}) = E_{prob}[(x-T_{i,j})^2]

  if (length.size() == 0){
    //printf("  WARNING: no paths\n");
    return default_test;
  }

  // compute expected length
  //printf("   E(len) = ");
  double T = 0.0;
  for (unsigned int x = 0; x < length.size();  x++)
    {
      T += length[x];
      //printf(" %f + ", length[x]);
    }
  T /= (double) length.size();
  //printf(" = %f\n", T);

  // compute variance
  //printf("   var(len) = ");
  double var = 0.0;
  for (unsigned int x = 0; x < length.size();  x++)
    {
      double val = pow((length[x]-T),2);
      var += val;
    }
  var /= (double) length.size();
  //printf(" = %f\n", var);
  
  // compute the test statistic
  double test = pow((observed_sharing - T),2);
  test /= var;

  //printf("   test (%f-%f^2)/%f = %f\n", observed_sharing, T, var, test);


  return test;
}




// compute the kinship (expected sharing) on
// all pairs of individuals in the pedigree
void kinship(vector<Indiv>& pedigree, vector<vector<double> >& kinship)
{
  // allocate
  for (unsigned int i = 0;  i < pedigree.size();  i++) {
    vector<double> row(pedigree.size(), 0.0);
    kinship.push_back(row);
  }


  // initialize the founder sharing values
  int i = pedigree.size()-1;
  if (i <= 0)
    printf("WARNING: out of bounds index because ped size %i\n", (int) pedigree.size());
  fflush(stdout);
  int g = pedigree[i].generation;
  int founder_gen = g;
  while (g == founder_gen)
    {
      //printf("founder %i = 0.5\n", i);
      kinship[i][i] = 0.5;

      int j = pedigree.size()-1;
      while (j >= 0)
	{
	  if (!pedigree[i].HasDescendant(j))
	    kinship[i][j] = 0;

	  j--;
	}

      i--;
      founder_gen = g;
      g = pedigree[i].generation;
    }
  int first_nonfounder = i;

  // in top-down order...
  for (int i = first_nonfounder;  i > 0;  i--)
    {
      int g = pedigree[i].generation;
      if (pedigree[i].parents.size() == 0)
	{
	  printf("WARNING: attempted out of bound access on parents\n");
	  printf("  pedigree[%i].parents.size() = %d\n", i, (int)pedigree[i].parents.size());
	  printf("  founder gen %i, ped gen %i\n",  founder_gen, pedigree[i].generation);
	  exit(-34534);
	}
      int mi = pedigree[i].parents[0];
      int fi = pedigree[i].parents[1];
      if (g < founder_gen)
	{
	  kinship[i][i] = 0.5 + 0.5*kinship[mi][fi];
	  kinship[i][mi] = 0.25;
	  kinship[mi][i] = 0.25;
	  kinship[i][fi] = 0.25;
	  kinship[fi][i] = 0.25;
	  for (int j = first_nonfounder;  j > i;  j--)
	    {
	      if (!pedigree[i].HasDescendant(j))
		{
		  kinship[i][j] = 0.5*kinship[mi][j] + 0.5*kinship[fi][j];
		  //printf("kinship %i,%i:  %f\n", i,j, kinship[i][j]);
		  kinship[j][i] = kinship[j][i];
		}
	    }
	}
    }
}








#endif
