#ifndef MAT_RGRAPH
#define MAT_RGRAPH

#include <vector>

using namespace std;

class MatGraph
{ 
  public:
    MatGraph(int size)   
    ~MatGraph();
    WeightType getWeight(int i, int j);
    void setWeight(int i, int j, WeightType weight);
  protected:
    int mSize;
    WeightType** mMatrix; 
};


MatGraph::MatGraph(int size)
{
  mSize=size;
  mMatrix=new  WeightType[mSize];
  for(int i=0; i<size; i++)
  {
    mMatrix=new WeightType[][]
  }
}
const MatGraph& MatGraph::operator= (const MatGraph& g)
{
  num_nodes = g.num_nodes;
  CopyGraph(g);
  return *this;
}

void MatGraph::CopyGraph(const MatGraph& g)
{
  matrix.resize(g.num_nodes, vector<double>(num_nodes, 0.0));
  for (unsigned int i = 0; i < g.matrix.size();  i++)
    {
      //matrix[i].resize(g.matrix[i].size(), 0.0);
      for (unsigned int j = i;  j < g.matrix[i].size();  j++)
	{
	  matrix[i][j] = g.matrix[i][j];
	}
    }
}

double MatGraph::GetEdge(unsigned int i, unsigned int j)
{
  if (i > j){
    unsigned int tmp = j;
    j = i;
    i = tmp;
  }
  if (j > num_nodes){
    printf("FATAL ERROR: out of bounds edge (%i,%i) when %i nodes\n", i,j,num_nodes);
    exit(-345);
  }

  return matrix[i][j];
}

void MatGraph::SetEdge(unsigned int i, unsigned int j, double w)
{
  if (i > j){
    unsigned int tmp = j;
    j = i;
    i = tmp;
  }
  if (j > num_nodes){
    printf("FATAL ERROR: out of bounds edge (%i,%i) when %i nodes\n", i,j,num_nodes);
    exit(-345);
  }

  if (matrix.size() <= i || matrix[i].size() < j)
  {
      printf("FATAL ERROR: out of bounds edge (%i,%i) when %i nodes\n", i,j,num_nodes);
  }

  matrix[i][j] = w;
}



void MatGraph::DeleteEdgeSet(unsigned int i)
{
  if (i >= num_nodes){
    printf("FATAL ERROR: out of bounds node (%i) when %i nodes\n", i,num_nodes);
    exit(-345);
  }

  for (unsigned int j = 0;  j <= i;  j++)
    matrix[j][i] = 0.0;
  for (unsigned int j = i+1; j < num_nodes;  j++)
    matrix[i][j] = 0.0;

  // if person is in the adj_list, then remove them
  if (i > 0 || i < adj_list.size())
    {
      // remove i from each adj_list that it is in
      for (unsigned int j = 0;  j < adj_list.size();  j++)
	{
	  list<unsigned int>::iterator erasor = find(adj_list[j].begin(), adj_list[j].end(), i);
	  if (erasor != adj_list[j].end())
	    adj_list[j].erase(erasor);
	}
      adj_list[i].clear();
    }
}




void MatGraph::AdjList(unsigned int lower_bound, unsigned int upper_bound)
{
  adj_list.clear();

  // construct adjacency lists
  for (unsigned int u = 0;  u < lower_bound;  u++)
    adj_list.push_back(list<unsigned int>());

  for (unsigned int u = lower_bound;  u <= upper_bound; u++)
    {
      list<unsigned int> neighbors;
      //printf(" vertex %i: ", u);
      neighbors.push_back(u);
      for (unsigned int v = u+1; v <= upper_bound; v++)
	if (matrix[u][v] > 0.0)
	  {
	    //printf("%i, ", v);
	    neighbors.push_back(v);
	  }
      //printf("\n");
      adj_list.push_back(neighbors); // for vertex u
    }
}

bool MatGraph::IsEmptyAdjList()
{
  return adj_list.empty();
}







void MatGraph::SequentialColor(unsigned int lower_bound, 
				      unsigned int upper_bound)
{
  if (upper_bound + 1 > num_nodes)
    printf("WARNING: upper_bound, %i, is too high (<= %i)\n", upper_bound, num_nodes);


  unsigned int num_vertices = upper_bound - lower_bound + 1;
  printf("SequentialColor(%i,%i) numvert %i\n", lower_bound, upper_bound, num_vertices);

  // order the vertices by degree
  vector<int> degree(num_vertices, 0);
  for (unsigned int i = 0;  i < num_vertices;  i++)
    {
      degree[i] = adj_list[lower_bound + i].size();
    }

  // sort the vertices by increasing order
  vector<int> order_map(num_vertices, -1);

  for (unsigned int i = 0;  i < num_vertices;  i++)
    {
      int larger_than = 0;
      int least_deg = 99999;
      int least_v = -1;
      for (unsigned int j = 0; j < num_vertices;  j++)
	{
	  if (degree[j] < least_deg && degree[j] >= larger_than)
	    {
	      if (order_map[j] == -1)
		{
		  least_deg = degree[j];
		  least_v = j;
		}
	    }
	}
      if (least_v != -1)
	{
	  order_map[least_v] = i;
	  larger_than = least_v;
	}
      else
	printf("WARNING: invalid sort of vertices\n");
    }
  vector<int> order(num_vertices,-1);
  for (unsigned int i = 0;  i < num_vertices;  i++)
    {
      if (order_map[i] == -1)
	printf("WARNING: invalid sort of vertices; missing order\n");
      else
	order[order_map[i]] = i;
    }
  
  // print out
/*   printf("Ordering Map: "); */
/*   for (unsigned int i = 0;  i < num_vertices;  i++) */
/*     { */
/*       printf("[%i] d=%i, o=%i ", lower_bound + i, degree[i], order_map[i]); */
/*     } */
/*   printf("\n"); */

  // print out
/*   printf("Ordering: "); */
/*   for (unsigned int i = 0;  i < num_vertices;  i++) */
/*     { */
/*       printf("[%i] v=%i, d=%i ", i, order[i]+lower_bound, degree[order[i]]); */
/*     } */
/*   printf("\n"); */



  // pick a coloring, i.e. smallest available color
  int num_colors = 0;
  vector<int> l_coloring(num_vertices, -1);
  l_coloring[order[order.size()-1]] = 0;
  //printf("colored %i with %i\n", order[order.size()-1]+lower_bound, 0);
  for (int i = order.size()-2;  i > -1;  i--)
    {
      // find smallest color that respects colored adjacent neighbors;
      unsigned int u = lower_bound + order[i];
      vector<int> used_colors(num_colors+1, -1);
      for (unsigned int v = lower_bound;  v <= upper_bound; v++)
	{
	  int j = v - lower_bound;
	  double weight =  matrix[u][v];
	  if (u > v)
	    weight = matrix[v][u];
	  if (weight > 0 && l_coloring[j] != -1){
	      used_colors[l_coloring[j]] = 1;
	  }
	}
      int my_color = -1;
      for (unsigned int j = 0;  j < used_colors.size();  j++)
	{
	  if (used_colors[j] == -1){
	    my_color = j;
	    break;
	  }
	}
      if (my_color == -1)
	{
	  num_colors++;
	  my_color = num_colors;
	}
      l_coloring[order[i]] = my_color;
      //printf("colored %i with %i\n", order[i]+lower_bound, my_color);
    }

  // print out
  //printf("Coloring: ");
  //for (unsigned int i = 0;  i < num_vertices;  i++)
  //  {
  //    printf("[%i] c=%i ", lower_bound + i, l_coloring[i]);
  //  }
  //printf("\n");



  vector<int> used_colors(num_colors+1, 0);
  for (unsigned int i = 0;  i < l_coloring.size();  i++)
    used_colors[l_coloring[i]]++;

  
  int c_color = 0;
  int max_count = -1;
  for (unsigned int i = 0; i < used_colors.size(); i++)
    {
      if (used_colors[i] > max_count)
	{
	  c_color = i;
	  max_count = used_colors[i];
	}
    }
  chosen_color = c_color;

  //printf("Chose color: %i, poisson %f\n", chosen_color, max_count);

  coloring = l_coloring;
}




/* void MatGraph::SequentialColor(unsigned int lower_bound,  */
/* 				      unsigned int upper_bound, */
/* 				      double poisson_mean) */
/* { */
/*   double best; */
/*   vector<int> best_coloring; */
/*   int best_chosen_color; */
/*   for (int i = 0; i < 100;  i++) */
/*     { */
/*       double b = SequentialColorRand(lower_bound, upper_bound, poisson_mean); */
/*       if (b < best) */
/* 	{ */
/* 	  best = b; */
/* 	  best_coloring = coloring; */
/* 	  best_chosen_color = chosen_color; */
/* 	} */
/*     } */

/*   chosen_color = best_chosen_color; */
/*   coloring = best_coloring; */
/* } */


double MatGraph::SequentialColor(unsigned int lower_bound, 
					  unsigned int upper_bound,
					  double poisson_mean)
{
  if (upper_bound + 1 > num_nodes)
    printf("WARNING: upper_bound, %i, is too high (<= %i)\n", upper_bound, num_nodes);


  unsigned int num_vertices = upper_bound - lower_bound + 1;
  printf("SequentialColor(%i,%i) numvert %i\n", lower_bound, upper_bound, num_vertices);

  // order the vertices by degree
  vector<int> degree(num_vertices, 0);
  for (unsigned int i = 0;  i < num_vertices;  i++)
    {
      degree[i] = adj_list[lower_bound + i].size();
    }

  // sort the vertices by increasing order
  vector<int> order_map(num_vertices, -1);

  for (unsigned int i = 0;  i < num_vertices;  i++)
    {
      int larger_than = 0;
      int least_deg = 99999;
      int least_v = -1;
      for (unsigned int j = 0; j < num_vertices;  j++)
	{
	  if (degree[j] < least_deg && degree[j] >= larger_than)
	    {
	      if (order_map[j] == -1)
		{
		  least_deg = degree[j];
		  least_v = j;
		}
	    }
	}
      if (least_v != -1)
	{
	  order_map[least_v] = i;
	  larger_than = least_v;
	}
      else
	printf("WARNING: invalid sort of vertices\n");
    }
  vector<int> order(num_vertices,-1);
  for (unsigned int i = 0;  i < num_vertices;  i++)
    {
      if (order_map[i] == -1)
	printf("WARNING: invalid sort of vertices; missing order\n");
      else
	order[order_map[i]] = i;
    }
  
  // print out
/*   printf("Ordering Map: "); */
/*   for (unsigned int i = 0;  i < num_vertices;  i++) */
/*     { */
/*       printf("[%i] d=%i, o=%i ", lower_bound + i, degree[i], order_map[i]); */
/*     } */
/*   printf("\n"); */

  // print out
/*   printf("Ordering: "); */
/*   for (unsigned int i = 0;  i < num_vertices;  i++) */
/*     { */
/*       printf("[%i] v=%i, d=%i ", i, order[i]+lower_bound, degree[order[i]]); */
/*     } */
/*   printf("\n"); */



  // pick a coloring, i.e. smallest available color
  int num_colors = 0;
  vector<int> l_coloring(num_vertices, -1);
  l_coloring[order[order.size()-1]] = 0;
  //printf("colored %i with %i\n", order[order.size()-1]+lower_bound, 0);
  for (int i = order.size()-2;  i > -1;  i--)
    {
      // find smallest color that respects colored adjacent neighbors;
      unsigned int u = lower_bound + order[i];
      vector<int> used_colors(num_colors+1, -1);
      for (unsigned int v = lower_bound;  v <= upper_bound; v++)
	{
	  int j = v - lower_bound;
	  double weight =  matrix[u][v];
	  if (u > v)
	    weight = matrix[v][u];
	  if (weight > 0 && l_coloring[j] != -1){
	      used_colors[l_coloring[j]] = 1;
	  }
	}
      int my_color = -1;
      for (unsigned int j = 0;  j < used_colors.size();  j++)
	{
	  if (used_colors[j] == -1){
	    my_color = j;
	    break;
	  }
	}
      if (my_color == -1)
	{
	  num_colors++;
	  my_color = num_colors;
	}
      l_coloring[order[i]] = my_color;
      //printf("colored %i with %i\n", order[i]+lower_bound, my_color);
    }

  // print out
  //printf("Coloring: ");
  //for (unsigned int i = 0;  i < num_vertices;  i++)
  //  {
  //    printf("[%i] c=%i ", lower_bound + i, l_coloring[i]);
  //  }
  //printf("\n");



  vector<int> used_colors(num_colors+1, 0);
  for (unsigned int i = 0;  i < l_coloring.size();  i++)
    used_colors[l_coloring[i]]++;


  coloring = l_coloring;



  int c_color = 0;
  double max_count = -1;
  for (unsigned int i = 0; i < used_colors.size(); i++)
    {
      double tmp = WeightPartitionSet(lower_bound,
				      upper_bound,
				      i,
				      poisson_mean);

      if (tmp > max_count)
	{
	  c_color = i;
	  max_count = tmp;
	}
    }
  chosen_color = c_color;
  
  printf("Chose color: %i, poisson %f\n", chosen_color, max_count);


  //printf("Chose color: %i\n", chosen_color);

  coloring = l_coloring;

  return max_count;
}





double  MatGraph::WeightPartitionSet(unsigned int lower_bound,
					    unsigned int upper_bound,
					    int _chosen_color,
					    double poisson_mean)
{
  vector<int> tmp_coloring(coloring.size(), 0);

  for (unsigned int i = 0; i < coloring.size(); i++)
    {
      tmp_coloring[i] = coloring[i];
    }

  vector<int> sizes;
  bool coloredRemain = true;
  while (coloredRemain)
    {
      int size = 0;

      bool hasSingletons = false;
      int singleton = -1;
      for (unsigned int u = lower_bound; u <= upper_bound; u++)
	{
	  if (tmp_coloring[u-lower_bound] == _chosen_color)
	    {
	      vector<int> color_set;
	      color_set.push_back(u);
	      
	      for (unsigned int v = lower_bound;  v <= upper_bound; v++)
		{
		  double weight =  matrix[u][v];
		  if (u > v)
		    weight = matrix[v][u];
		  if (weight > 0)
		    {
		      if (tmp_coloring[v-lower_bound] != -1)
		      { // if it hasn't been chosen before
			  size++;
			  tmp_coloring[v-lower_bound] = -1;
		       }
		    }
		}
	      tmp_coloring[u-lower_bound] = -1;
	    }
	  else
	    {
	      if (tmp_coloring[u-lower_bound] != -1){
		size = 1;
		hasSingletons = true;
		singleton = u;
	      }
	    }
	}
      

      if (hasSingletons)
	{
	  tmp_coloring[singleton-lower_bound] = -1;
	}
      

      coloredRemain = false;
      for (unsigned int i = 0; i < tmp_coloring.size(); i++)
	{
	  if (tmp_coloring[i] != -1)
	    coloredRemain = true;
	}

      sizes.push_back(size);
    }


  // for partitions of size sizes
  // compute the poisson probability
  double weight = 0;
  for (unsigned int p = 0;  p < sizes.size();  p++)
    {
      if (sizes[p] > FACTSIZE)
	printf("WARNING: size of cluster exceeds size of factorial table\n");
      fflush(stdout);
      weight += -poisson_mean + sizes[p] * log(poisson_mean) - log(factorial_table[sizes[p]]);
    }


  return exp(weight);
}





vector<int> MatGraph::NextPartitionSet(
			     unsigned int lower_bound, 
			     unsigned int upper_bound)
{
  // find one set of vertices adjacent to a vertex of the chosen color
  // and return it

  // if none of the chosen color, but there are singletons, return them

  bool hasSingletons = false;
  int singleton = -1;
  for (unsigned int u = lower_bound; u <= upper_bound; u++)
    {
      if (coloring[u-lower_bound] == chosen_color)
	{
	  vector<int> color_set;
	  color_set.push_back(u);

	  for (unsigned int v = lower_bound;  v <= upper_bound; v++)
	    {
	      double weight =  matrix[u][v];
	      if (u > v)
		weight = matrix[v][u];
	      if (weight > 0)
		{
		  if (coloring[v-lower_bound] != -1)
		  { // if it hasn't been chosen before
		      color_set.push_back(v);
		      coloring[v-lower_bound] = -1;
		  }
		}
	    }
	  coloring[u-lower_bound] = -1;
	  return color_set;
	}
      else
	{
	  if (coloring[u-lower_bound] != -1){
	    hasSingletons = true;
	    singleton = u;
	  }
	}
    }


  if (hasSingletons)
    {
      coloring[singleton-lower_bound] = -1;
      return vector<int>(1,singleton);
    }
  

  return vector<int>(0,0);
}





bool isAllZero(int* array, int size)
{
  for (int i = 0;  i < size;  i++)
    if (array[i] == 1)
      return false;

  return true;
}

unsigned int getCliqueSize(int* array, int size)
{
  int cSize = 0;
  for (int i = 0;  i < size;  i++)
    if (array[i] > 0)
    cSize++;

  return cSize;
}

int popFront(int* array, int size)
{
  for (int i = 0;  i < size;  i++)
    if (array[i] == 1)
      {
	array[i] = 0;
	return i;
      }
  return -1;
}





// Invariants:
// 1) candidate_vertices contains putative clique nodes
//    and all elements of partial_clique have edges to candidate_vertices
// 2) elements of vector are in the clique under investigation
//    and they all have edges to each other
// 3) if a clique contains edge (u,v) then vertex min{u,v} will be visited first
//

int* MatGraph::BuildClique(int* _partial_clique,
					 int* _candidate_vertices, 
					 unsigned int lower_bound, 
					 unsigned int upper_bound,
					 unsigned int size)
{

  unsigned int num_vertices = upper_bound - lower_bound + 1;


  // Vertices indicated with 1 are in clique, 0 otherwise
  int* partial_clique;
  partial_clique = new int[num_vertices]; 
  
  // vertices indicated with 1 are candidates, 0 otherwise
  int* candidate_vertices;
  candidate_vertices = new int[num_vertices];

  for (unsigned int i = 0;  i < num_vertices; i++){
    partial_clique[i] = _partial_clique[i];
    candidate_vertices[i] = _candidate_vertices[i];
  }



/*   printf("BuildClique (size %i)\n", size); */
/*   printf("  partial_clique: "); */
/*   for (unsigned int i = 0;  i < num_vertices;  i++) */
/*     if (partial_clique[i] == 1) */
/*       printf("%i, ", i+lower_bound); */
/*   printf("\n"); */
/*   printf("  candidate_vertices: "); */
/*   for (unsigned int i = 0;  i < num_vertices;  i++) */
/*     if (candidate_vertices[i] == 1) */
/*       printf("%i, ", i+lower_bound); */
/*   printf("\n"); */



  bool isEmpty = isAllZero(candidate_vertices, num_vertices);
  if (isEmpty)
    {
      if (getCliqueSize(partial_clique, num_vertices) >= size){
	delete[] candidate_vertices;
	return partial_clique; 
      } else {
	delete[] partial_clique;
	delete[] candidate_vertices;
	return NULL;
      }
    }

  // put u in partial_clique
  int u_idx = popFront(candidate_vertices, num_vertices);
  partial_clique[u_idx] = 1;
  int u = u_idx + lower_bound;


  // remove candidate vertices that are not adjacent to u
  for (unsigned int v = lower_bound;  v <= upper_bound;  v++)
    {
      if (candidate_vertices[v-lower_bound] == 1)
	{
	  if (matrix[u][v] <= 0.0)
	    {
	      candidate_vertices[v-lower_bound] = 0;
	    }
	}
    }

  // return if we have a large enough clique, already
  isEmpty = isAllZero(candidate_vertices, num_vertices);
  unsigned int cSize = getCliqueSize(partial_clique, num_vertices);
  if  (isEmpty){
    if (cSize >= size){
      delete[] candidate_vertices;
      //printf("######## FOUND CLIQUE #######\n");
      return partial_clique;
    }
  }


  // check all subsets of the common neighbor list
  while (!isEmpty)
    {
      cSize = getCliqueSize(partial_clique, num_vertices);
      unsigned int vSize = getCliqueSize(candidate_vertices, num_vertices);
      if (cSize + vSize < size)
	{
	  delete[] partial_clique;
	  delete[] candidate_vertices;
	  return NULL; // empty set
	}

      int* clique = BuildClique(partial_clique, 
				candidate_vertices, 
				lower_bound,
				upper_bound,
				size);

      bool cIsEmpty = false;
      if (clique == NULL || isAllZero(clique, num_vertices))
	cIsEmpty = true;
      if (cIsEmpty) // no clique of size size involves this node
	popFront(candidate_vertices, num_vertices);
      else
	{
	  delete[] partial_clique;
	  delete[] candidate_vertices;
	  return clique;
	}

      delete[] clique;
      isEmpty = isAllZero(candidate_vertices, num_vertices);
    }

  delete[] partial_clique;
  delete[] candidate_vertices;
  return NULL;
}










// u in [lower_bound, upper_bound]
int* MatGraph::MaxClique(unsigned int lower_bound, 
				unsigned int upper_bound)
{

  //printf("MaxClique check: ");
  //check.printClique();



  if (upper_bound + 1 > num_nodes)
    printf("WARNING: upper_bound, %i, is too high (<= %i)\n", upper_bound, num_nodes);


  unsigned int num_vertices = upper_bound - lower_bound + 1;


  // Vertices indicated with 1 are in clique, 0 otherwise
  int* partial_clique;
  partial_clique = new int[num_vertices]; 
  
  // vertices indicated with 1 are candidates, 0 otherwise
  int* candidate_vertices;
  candidate_vertices = new int[num_vertices];



  //printf("num_vertices: %i btw [%i,%i]\n", num_vertices, lower_bound, upper_bound);

  for (int cliqueSize = num_vertices; cliqueSize > 0;  cliqueSize--)
    {
      //printf("Clique size: %i\n", cliqueSize);

      // search all the vertices for one having degree num_vertices
      // and all neighbors making a complete graph
      for (unsigned int u = lower_bound;  u <= upper_bound; u++)
	{
	  if (adj_list[u].size() >= (unsigned) cliqueSize)
	    {
	      //printf("  putative clique %i\n", u);


	      // find a subset of the partial adj_list 
	      // that has a clique of this size


	      for (unsigned int i = 0;  i < num_vertices; i++){
		partial_clique[i] = 0;
		candidate_vertices[i] = 0;
	      }
	      partial_clique[u-lower_bound] = 1;
	      for (list<unsigned int>::iterator li = adj_list[u].begin();
		   li != adj_list[u].end();
		   li++)
		candidate_vertices[*li-lower_bound] = 1;

	      candidate_vertices[u-lower_bound] = 0;


	      bool isEmpty = isAllZero(candidate_vertices, num_vertices);
	      if (isEmpty){
		delete[] candidate_vertices;
		return partial_clique;
	      }

	      while (!isEmpty)
		{
		  int* clique = BuildClique(partial_clique,
					    candidate_vertices, 
					    lower_bound,
					    upper_bound,
					    cliqueSize);

		  bool cIsEmpty = false;
		  if (clique == NULL || isAllZero(clique, num_vertices)) 
		    cIsEmpty = true;
		  if (cIsEmpty) // no clique of size size involves this node
		    popFront(candidate_vertices, num_vertices);
		  else
		    {
		      delete[] candidate_vertices;
		      delete[] partial_clique;
		      return clique;
		    }
		  delete[] clique;
		  isEmpty = isAllZero(candidate_vertices, num_vertices);
		}

	    } // end check clique
	} // end for each vertex
    } // end for each cliqueSize
    
  
  delete[] candidate_vertices;
  delete[] partial_clique;
  return NULL; // empty set
}







// Invariants:
// 1) candidate_vertices contains putative clique nodes
//    and all elements of partial_clique have edges to candidate_vertices
// 2) elements of vector are in the clique under investigation
//    and they all have edges to each other
// 3) if a clique contains edge (u,v) then vertex min{u,v} will be visited first
//

/* vector<int> MatGraph::BuildClique2(vector<int> partial_clique, */
/* 					 list<unsigned int> candidate_vertices, */
/* 					 unsigned int size) */
/* { */
/* /\*   printf("BuildClique\n"); *\/ */
/* /\*   printf("  partial_clique: "); *\/ */
/* /\*   for (unsigned int i = 0;  i < partial_clique.size();  i++) *\/ */
/* /\*     printf("%i, ", partial_clique[i]); *\/ */
/* /\*   printf("\n"); *\/ */
/* /\*   printf("  candidate_vertices: "); *\/ */
/* /\*   for (list<unsigned int>::iterator li = candidate_vertices.begin(); *\/ */
/* /\*        li != candidate_vertices.end(); *\/ */
/* /\*        li++) *\/ */
/* /\*     printf("%i, ", *li); *\/ */
/* /\*   printf("\n"); *\/ */


/*   if (candidate_vertices.empty()) */
/*     { */
/*       if (partial_clique.size() >= size){ */
/* 	//check->printClique(); */
/* 	//rtn_clique.printClique(); */
/* 	return partial_clique; */
/*       } else { */
/* 	return vector<int>();  // empty set */
/*       } */
/*     } */

/*   // put u in partial_clique */
/*   int u = candidate_vertices.front(); */
/*   candidate_vertices.pop_front(); */
/*   partial_clique.push_back(u); */


/*   // remove candidate vertices that are not adjacent to u */
/*   list<unsigned int>::iterator erasor; */
/*   for (list<unsigned int>::iterator li = candidate_vertices.begin(); */
/*        li != candidate_vertices.end(); */
/*        ) */
/*     { */
/*       int v = *li; */
/*       if (matrix[u][v] <= 0.0) */
/* 	{ */
/* 	  erasor = li; */
/* 	  li++; */
/* 	  candidate_vertices.erase(erasor); */
/* 	} */
/*       else */
/* 	li++; */
/*     } */

/*   // return if we have a large enough clique, already */
/*   if  (candidate_vertices.empty() && partial_clique.size() >= size){ */
/*     //new_check.printClique(); */
/*     //rtn_clique.printClique(); */
/*     return partial_clique; */
/*   } */


/*   // check all subsets of the common neighbor list */
/*   while (!candidate_vertices.empty()) */
/*     { */
/*       if (partial_clique.size() + candidate_vertices.size() < size) */
/*         return vector<int>(0,0); // empty set */

/*       vector<int> clique = BuildClique2(partial_clique, */
/* 				       candidate_vertices, */
/* 				       size); */
/*       if (clique.empty()) // no clique of size size involves this node */
/* 	candidate_vertices.pop_front(); */
/*       else */
/* 	return clique; */
/*     } */

/*   return vector<int>(0,0); // empty set */
/* } */




/* vector<int> MatGraph::MaxClique2(unsigned int lower_bound, */
/* 				       unsigned int upper_bound) */
/* { */

/*   //printf("MaxClique check: "); */
/*   //check.printClique(); */


/*   if (upper_bound + 1 > num_nodes) */
/*     printf("WARNING: upper_bound, %i, is too high (<= %i)\n", upper_bound, num_nodes); */

/*   unsigned int num_vertices = upper_bound - lower_bound + 1; */

/*   //printf("num_vertices: %i btw [%i,%i]\n", num_vertices, lower_bound, upper_bound); */

/*   for (int cliqueSize = num_vertices; cliqueSize > 0;  cliqueSize--) */
/*     { */
/*       //printf("Clique size: %i\n", cliqueSize); */

/*       // search all the vertices for one having degree num_vertices */
/*       // and all neighbors making a complete graph */
/*       for (unsigned int u = lower_bound;  u <= upper_bound; u++) */
/* 	{ */
/* 	  if (adj_list[u].size() >= (unsigned) cliqueSize) */
/* 	    { */
/* 	      //printf("  putative clique %i\n", u); */


/* 	      // find a subset of the partial adj_list */
/* 	      // that has a clique of this size */
/* 	      vector<int> partial_clique = vector<int>(1,u); */
/* 	      list<unsigned int> candidate_vertices = adj_list[u]; */
/* 	      candidate_vertices.pop_front();  // remove self-edge */
/* 	      if (candidate_vertices.empty()){ */
/* 		return partial_clique; */
/* 	      } */
/* 	      while (!candidate_vertices.empty()) */
/* 		{ */
/* 		  vector<int> clique = BuildClique2(partial_clique, */
/* 						   candidate_vertices, */
/* 						   cliqueSize); */

/* 		  if (clique.empty()) // no clique of size size involves this node */
/* 		    candidate_vertices.pop_front(); */
/* 		  else */
/* 		    return clique; */
/* 		} */

/* 	    } // end check clique */
/* 	} // end for each vertex */
/*     } // end for each cliqueSize */


/*   return vector<int>(0,0); // empty set */
/* } */




#endif
