#include <vector>
#include <map>
#include <queue>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include "Option.h"

#include "test_alg4.h"
#include "graph.h"
#include "ibd_rw.h"



using namespace std;



vector<string> names;

vector<vector<int> > ibd_sharing;






int          Option::number_of_regions=3000;
int          Option::number_of_iterations=100;
int          Option::number_of_ancestors=10; 
int          Option::number_of_generations=3;
int          Option::number_of_children=3;
double       Option::threshold_edge_test=0.75;
double       Option::region_length=1.0;
double       Option::a_value=0.01; 
int          Option::usegraph=1;
string       Option::outfile="outfile";
int          Option::cop=1;

string       Option::in_file="";
string       Option::out_file="outfile";


#define USAGE "reconstruct \n \
              [-r num_of_regions       (default 3000)]\n \
              [-l region_length in mb  (default 1.0)]\n \
              [-g num_of_generations   (default 3)]\n \
              [-e threshold_edge_test  (default 0.75)]\n \
              [-a a_value              (default 0.01)]\n \
              [-m methold  (CIP/COP)   (default COP)]\n \
              [-o out_file             (default outfile)]\n \
              <-i in_file>   -- IBD predictions as input\n"



void
initialize_options(int argc, char** argv)
{

   char c=0;
   string algo="";
   while((c=getopt(argc, argv, "g:e:a:m:o:l:r:i:"))!=EOF)
   {
      switch(c)
      {
         case 'r': if(optarg==NULL) 
                   {
                      printf(USAGE);exit(0);
                   }  
                   Option::number_of_regions=atoi(optarg); break;
         case 'l':if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }   
                   Option::region_length=atof(optarg); break;
         case 'g': 
                   Option::number_of_generations=atoi(optarg); break;
         case 'e': if(optarg==NULL)
                   Option::threshold_edge_test=atof(optarg); break;
         case 'a': if(optarg==NULL)
                   Option::a_value=atoi(optarg); break;
         case 'm':
             algo=optarg;
           break;
         case 'o': 
                 Option::out_file=optarg; break;
         case 'i':
                 Option::in_file=optarg;  break;
         default:
             printf(USAGE);
             exit(0);
      }

   }
   if(algo!="" && (algo!=string("COP") && algo!=string("CIP")))
   {
      cerr<<"Methold can only be CIP or COP \n"<<endl;
             exit(0); 
   }
    else if(algo==string("CIP"))
   { 
             Option::cop= 0;
   }
   else Option::cop= 1; 




   if(Option::in_file=="")
   {
      cerr<<"Input file must be specified "<<endl;
      printf(USAGE);
      exit(0);

   }
}






//////////////////
//
// Estimate average length of IBD regions from the 
// input ibd_sharing which are pair-wise ibd predictions
// (i.e. 0,1,2 alleles shared IBD)
//
// output: sharing, graph containing weighted edges where 
//                  the weight is the expected length of 
//                  shared regions for that pair of individuals
// 

void observed_sharing_from_ibd(UndirectedGraph& sharing, 
			       MatGraph* m_sharing,
			       int num_indiv)
{
 
  // Data in global variable: ibd_sharing
  // for every pair of people and every position, the number of alleles ibd
  // indexes for pairs are counted from 



  double avg_avg_length = 0.0;
  int avg_avg_number = 0;
  double avg_sq_diff = 0.0;
  int pair_index = 0;
  for (int i = 0; i < num_indiv; i++)
    {

      for (int j=i+1;  j < num_indiv; j++, pair_index++)
	{
	  //printf("  %i,%i) ", i+1,j+1);

	  // compute sharing for this pair of people
	  vector<int> max_share;

	  int length1 = 0;
	  int length2 = 0;
	  for (int pos = 0;  pos < g_actualnumofregions; pos++)
	    {
	      // indicates orientation of sharing (consider both pairings of haps)
	      //printf("ibd_sharing[%i][%i]\n", pair_index, pos);
	      //fflush(stdout);
	      int numShared = ibd_sharing[pair_index][pos];

	      //if (i==60&&j==79){
	      //printf("%i   ");
	      //}

	      if (numShared >= 1) {
		length1++;
		//if (i==60&&j==79){
		// printf(" [1]");
		//}
	      } else if (numShared == 0){
		if (length1 > 0)
		  max_share.push_back(length1);
		length1 = 0;
		//if (i==60&&j==79){
		//printf(" -1-");
		//}
	      }
	      if (numShared == 2) {
		length2++;
		//if (i==60&&j==79){
		// printf(" [2]");
		//}
	      } else {
		if (length2 > 0)
		  max_share.push_back(length2);
		length2 = 0;
		//if (i==60&&j==79){
		//  printf(" -2-");
		//}
	      }

	      //if (i==60&&j==79){
	      //printf("\n");
	      //}

	    } // end for pos
	  if (length1 > 0)
	    max_share.push_back(length1);
	  if (length2 > 0)
	    max_share.push_back(length2);
	  //printf("\n");

	  // average length of max-length sharing regions
	  double avg_length = 0.0;
	  //printf("Avg Share: ");
	  for (unsigned int k = 0;  k < max_share.size();  k++)
	    {
	      //printf(" %i +", max_share[k]);
	      avg_length += max_share[k];
	      avg_avg_length += max_share[k];
	      avg_avg_number++;
	      avg_sq_diff += pow(max_share[k]-2.690608, 2);
	    }
	  if (max_share.size() > 0)
	    avg_length /= max_share.size();
	  //printf(" = %f\n", avg_length);

	  //printf("sharing.SetEdge(%i,%i) = %f\n", i+1, j+1, avg_length);
	  //fflush(stdout);
	  sharing.SetEdge(i+1,j+1, avg_length);
	  m_sharing->setWeight(i+1, j+1, avg_length);
	  
	}
    }


  // set the diagonal elements
  for (int i = 0; i < num_indiv; i++)
    {
      sharing.SetEdge(i+1,i+1, g_actualnumofregions);
      m_sharing->setWeight(i+1, i+1, g_actualnumofregions);
    }





  //printf("avg of all lengths %f = %f/%i\n", (avg_avg_length / (double) avg_avg_number), avg_avg_length, avg_avg_number);
  //printf("var of all lengths %f\n", (avg_sq_diff / (double) avg_avg_number));
  //printf("\n");
}







int readNumIndiv(char* infile)
{
  char s[16384];
  char *s2;

  int c = 0;

  FILE *fp = fopen (infile, "r");
  if (fp == NULL) {
    printf ("cannot open file %s\n", infile);
    exit(-1);
  }


  // Get names of people -- first line
  int have_endline = 0;
  if (fgets(s,16384,fp) != NULL)
    {
      int c = 0;
      s2 = strtok (s, " ");
      while (s2 != NULL)
	{
	  if (s2[0] != '\n')
	    {
	      // place into vector
	      //printf("[%i] %s\n", c, s2);
	      names.push_back(s2);	  
	      c++;
	    }
	  else
	    have_endline = 1;
	  s2 = strtok (NULL, " ");
	}
    }
  else
    {
      printf("FATAL ERROR: No first line with people names.\n");
      exit(-324);
    }
  if (have_endline == 0)
    {
      printf("\nFATAL ERROR: total length of names in first line exceeded buffer size.\n");
      printf("Shorten the names.\n\n");
      exit(-324);
    }


  printf("Number of people %i\n",names.size());
  

  fclose(fp);

  return names.size();
}







int readData(char* infile)
{
  int size = (names.size() * (names.size()-1))+5;
  char* s = new char[size];
  char *s2;


  int pos = 0;
  int line = 1;

  FILE *fp = fopen (infile, "r");
  if (fp == NULL) {
    printf ("cannot open file %s\n", infile);
    exit(-1);
  }


  // ignore first line... already read the names
  if (fgets(s,size,fp) == NULL)
    {
      printf("No first line with people names\n");
      exit(-324);
    }
  line++;


  int entries = (names.size() * (names.size()-1))/2;
  ibd_sharing.resize(entries, vector<int>(0, 0));

  while (fgets(s,size,fp) != NULL)
    {
      line++;
      s2 = strtok (s, " ");

      int index = 0;
      for (int i = 0;  i < names.size(); i++)
	{
	  for (int j = i+1;  j < names.size(); j++, index++)
	    {
	      if (s2 != NULL)
		{
		  if (s2[0] != '\n')
		    {
		      ibd_sharing[index].push_back(atoi(s2));
		    }
		  s2 = strtok (NULL, " ");
		}
	      else
		{
		  int entries = (names.size() * (names.size()-1))/2;
		  printf("\nFATAL ERROR: line %i\n", line);
		  printf("Every line must contain %i entries, since there are %i people\n", entries, names.size());
		  printf("That is the n(n-1)/2 upper-triangular off-diagonal entries in the\n");
		  printf("%i x %i matrix of pairs of people.\n\n", names.size(), names.size());
		  printf("(note that this error may if the bufffer size was exceeded)\n");
		  
		  exit(-324);
		}
	    }
	}
      pos++;
    }


  fclose(fp);

  return pos;
}





int main (int argc, char **argv)
{

  FILE *fp;
  double regionlen;
  int i,j,k;
  long int seed;
  char outfile[100];
  char infile[100];
  int numofiterations = 1;
  int usegraph = Option::usegraph;
  int iter;


  initialize_options(argc, argv);

   if (Option::cop)
     {
       printf("COP Algorithm\n-------------\n\n"); 
     }
   else
     {
       printf("CIP Algorithm\n-------------\n\n"); 
     }


  strcpy(infile, Option::in_file.c_str());
  strcpy(outfile, Option::out_file.c_str());


  g_numofregions = Option::number_of_regions;
  numofiterations = Option::number_of_iterations;
  g_numofancestors = Option::number_of_ancestors;
  g_numofgenerations = Option::number_of_generations;
  g_avgnumofchildren = Option::number_of_children;
  g_threshedgetest=   Option::threshold_edge_test;
  g_outbred = 0;
  regionlen = Option::region_length;




  // each region has length in units of 1 kb
  // num_regions * regionlen * 1kb = 3 mb == 30 recomb.
  g_regionlen = 1.0;
  g_recombrate = RECRATE*g_regionlen*1000000.0;

  
  tmp_ancestors = g_numofancestors;


  //printf("  recrate = %f*%f*%f = %f\n", RECRATE,regionlen,1000000.0, g_recombrate);

  
  
  seed = time(NULL);
  //seed = 1;
  srand48(seed);

  g_numofancestors = readNumIndiv(infile);
  g_avgnumofchildren = Option::number_of_children;
  g_actualnumindividuals = g_numofancestors;


  printf("outfile =  ");
  for (i = 0; i < (int) strlen(outfile); i++) {
    printf ("%c", outfile[i]);
  }
  printf("\n");
  
  printf ("num of regions = %d\n", g_numofregions);
  printf ("num of ancestors = %d\n", g_numofancestors);
  printf ("recombination rate = %lf\n", g_recombrate);
  printf ("num of generations = %d\n", g_numofgenerations);
  printf ("average num of children = %lf\n", g_avgnumofchildren);
  printf ("thresh edge_test = %f\n",g_threshedgetest);
  printf ("outfile name = %s\n", outfile);
  printf ("number of iterations = %d\n", numofiterations);
  

  if (g_recombrate > 0.5)
    {
      printf("g_recombrate, set from region, is too large\n");
      printf("  recrate = %f*%f*%f = %f\n", RECRATE,regionlen,1000000.0, g_recombrate);
      exit(-345);
    }

  if (g_numofgenerations * g_numofancestors * g_numofregions > 100000000) {
    printf ("Not enough memory\n");
    exit(-1);
  }
  g_genotype = (int ***) malloc (sizeof (int **) *  g_numofgenerations);
  if (g_genotype == NULL) {
    printf ("Cannot allocate memory\n");
    exit(-1);
  }
  for (i = 0; i < g_numofgenerations; i++) {
    g_genotype[i] = (int **) malloc (sizeof (int *) * 2 *g_numofancestors * SAFENET);
    if (g_genotype[i] == NULL) {
      printf ("Cannot allocate memory\n");
      exit(-1);
    }
    for (j = 0; j < 2*g_numofancestors; j++) {
      g_genotype[i][j] = (int *) malloc (sizeof (int) * g_numofregions);
      if (g_genotype[i][j] == NULL) {
	printf ("Cannot allocate memory\n");
	exit(-1);
      }
      for (k = 0; k < g_numofregions; k++)
	g_genotype[i][j][k] = -1;
    }
  }
  
  g_used = (int **) malloc (sizeof (int *) * g_numofgenerations);
  if (g_used == NULL) {
    printf ("Cannot allocate memory\n");
    exit(-1);
  }
  for (i = 0; i < g_numofgenerations; i++) {
    g_used[i] = (int *) malloc (sizeof (int) * g_numofancestors * SAFENET);
    if (g_used[i] == NULL) {
      printf ("Cannot allocate memory\n");
      exit(-1);
    }
  }

  g_parents = (int ***) malloc (sizeof (int **) * 2);
  if (g_parents == NULL) {
    printf ("Cannot allocate memory\n");
    exit(-1);
  }
  for (i = 0; i < 2; i++) {
    g_parents[i] = (int **) malloc (sizeof (int *) * g_numofgenerations);
    if (g_parents[i] == NULL) {
      printf ("Cannot allocate memory\n");
      exit(-1);
    }
    for (j = 0; j < g_numofgenerations; j++) {
      g_parents[i][j] = (int *) malloc (sizeof (int) * g_numofancestors * SAFENET);
      if (g_parents[i][j] == NULL) {
	printf ("Cannot allocate memory\n");
	exit(-1);
      }
      for (k = 0; k < g_numofancestors; k++) {
	g_parents[i][j][k] = -1;
      }
    }
  }
  
  fp = fopen (outfile, "w");
  if (fp == NULL) {
    printf ("Cannot open %s\n", outfile);
    exit(-1);
  }



  double sum_mistakes = 0.0;
  double max_mistakes = 0.0;
  double sum_nosibship_acc = 0.0;

  double sum_edit_similarity = 0.0;
  double sum_edit_similarity_1 = 0.0;
  double sum_edit_similarity_rand = 0.0;
  double min_edit_similarity = 9999999;

  double sum_kinship_difference = 0.0;
  double sum_haplo_diff = 0.0;
  double sum_perturb_kin_diff_1 = 0.0;
  //double sum_perturb_kin_diff_5 = 0.0;
  double sum_perturb_kin_diff_rand = 0.0;




  // fill the haplotype vectors with data from file
  g_actualnumofregions = readData(infile);


  initFactorial();

  vector<vector<double> > observed_expected_sharing;
  UndirectedGraph u_sharing(g_actualnumindividuals+1);
  MatGraph* m_sharing = new MatGraph(g_actualnumindividuals+1);
  observed_sharing_from_ibd(u_sharing, m_sharing, names.size());



  // reconstruct pedigree
  printf("\nReconstruction\n");
  for (int s = test_thresholds.size(); s < g_numofgenerations; s++)
    test_thresholds.push_back(g_threshedgetest);



  if(Option::cop)
    {
      reconstruct_pedigree(g_numofgenerations-1, g_actualnumindividuals, test_thresholds,
			   observed_expected_sharing, m_sharing);
    }
  else
    {
      reconstruct_pedigreeCIP(g_numofgenerations-1, g_actualnumindividuals, test_thresholds,
			      observed_expected_sharing, u_sharing);
    }


  //printf("Estimated Pedigree:\n");
  //printPedigree(pedigree);


  // print *.ped file
  printPedFile(pedigree,names, fp);


  fclose(fp);



  exit(0);

}
