#include <vector>
#include <map>
#include <queue>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>



#include "test_alg.h"
#include "test_alg4.h"
#include "graph.h"
#include "ibd_rw.h"
#include "Option.h"


double       Option::rate_of_half_sibling=0.5;
double 	     Option::rate_internal_haplo=0;
int          Option::allow_hs_mate=1;
int          Option::allow_sb_mate=1;
int          Option::number_of_regions=3000;
int          Option::number_of_iterations=10; 
int          Option::number_of_ancestors=10; 
int          Option::number_of_generations=3;
int          Option::number_of_children=3;
int          Option::threshold_distance=0; 
int          Option::threshold_similarity=0; 
double       Option::threshold_edge_test=0.70; 
double       Option::region_length=1;
double       Option::a_value=0.01; 
int          Option::usegraph=1;
string       Option::outfile="outfile"; 
int          Option::outbred_sim=0;
int          Option::cop=1;

string       Option::in_file="";
string       Option::out_file="";


#define USAGE "simulate \n \
	[-r num_of_regions       (default 3000)]\n \
[-h rate_of_half_sibling (default 0.5)]\n \
[-z allow_hs_mate    (default 1(yes))]\n \
[-y allow_sb_mate    (default 1(yes))]\n \
[-i rate_internal_haplo   (default 0)]\n \
[-t num_of_iterations    (default 10)]\n \
[-p num_of_ancestors     (default 10)]\n \
[-g num_of_generations   (default 3)]\n \
[-c num_of_children      (default 3)]\n \
[-s outbred simulation   (default 0)]\n \
[-e threshold_edge_test  (default 0.7)]\n \
[-l region_length in mb  (default 1.0)]\n \
[-a a_value              (default 0.01)]\n \
[-m methold  (CIP/COP)   (default COP)]\n \
[-o out_file             (default outfile)]\n"


	void
initialize_options(int argc, char** argv)
{

	char c=0;
	while((c=getopt(argc, argv, "r:t:h:z:y:i:p:g:c:s:e:l:a:m:o:"))!=EOF)
	{
		switch(c)
		{
			case 'r': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }
				  Option::number_of_regions=atoi(optarg); break;

			case 't': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  } 
				  Option::number_of_iterations=atoi(optarg); break; 
			case 'h': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }	
				  Option::rate_of_half_sibling=atof(optarg); break;
			case 'z': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }
				  Option::allow_hs_mate=atoi(optarg); break;
        case 'y': if(optarg==NULL)
                                  {
                                          printf(USAGE);exit(0);
                                  }
                                  Option::allow_sb_mate=atoi(optarg); break;

			case 'i': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }
				  Option::rate_internal_haplo=atof(optarg); break;

			case 'p': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }  
				  Option::number_of_ancestors=atoi(optarg); break;
			case 'g': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }  
				  Option::number_of_generations=atoi(optarg); break;
			case 'c': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }  
				  Option::number_of_children=atoi(optarg); break;
			case 's': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }  
				  Option::outbred_sim=atoi(optarg); 
				  if (Option::outbred_sim != 0 && Option::outbred_sim != 1)
				  {
					  printf("Outbred value is boolean\n");
					  printf(USAGE);
					  exit(0);
				  }
				  break;
			case 'e': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }
				  Option::threshold_edge_test=atof(optarg); break;

			case 'l':if(optarg==NULL)
				 {
					 printf(USAGE);exit(0);
				 }   
				 Option::region_length=atof(optarg); break;

			case 'a':
				 Option::a_value=atoi(optarg); break;
			case 'm':
				 if(optarg==NULL || (strcmp(optarg, "CIP")!=0 && strcmp(optarg, "COP")!=0))
				 {
					 cerr<<"Methold can only be CIP or COP \n"<<endl;
					 exit(0);
				 }
				 else if(strcmp(optarg, "CIP")==0)
				 {
					 Option::cop= 0;
				 }
				 break;


			case 'o': if(optarg==NULL)
				  {
					  printf(USAGE);exit(0);
				  }  
				  Option::outfile=optarg; break;
			default:
				  printf(USAGE);
				  exit(0);
		} 

	}

}

void compute_variance(vector<double>& var,
		vector<vector<double> > values,
		vector<double> means)
{
	for (unsigned int v=0;  v < means.size();  v++)
	{
		double variance = 0;
		for (unsigned int s=0;  s < values[v].size();  s++)
		{
			variance += pow((values[v][s] - means[v]), 2);
		}
		variance /= (values[v].size()-1);
		var[v] = variance;
	}
}


int main (int argc, char **argv)
{
	double regionlen;
	int i,j,k;
	long int seed;
	char s[1000], *s2;
	char outfile[100];
	int numofiterations = 1;
	int usegraph = 0;
	int iter;

	initialize_options(argc, argv);



	seed = time(NULL);
	srand48(seed);



	g_numofregions = Option::number_of_regions;
	numofiterations = Option::number_of_iterations;
	g_rate_of_half_sibling=Option::rate_of_half_sibling;
	allow_hs_mate=Option::allow_hs_mate;
	 allow_sb_mate=Option::allow_sb_mate;
	rate_internal_haplo=Option::rate_internal_haplo;
	g_numofancestors = Option::number_of_ancestors;
	g_numofgenerations = Option::number_of_generations;
	g_avgnumofchildren = Option::number_of_children;
	g_threshdist       = Option::threshold_distance;
	g_threshsimilarity= Option::threshold_similarity;
	g_threshedgetest=   Option::threshold_edge_test;


	g_outbred = Option::outbred_sim;
	regionlen = Option::region_length;
	g_regionlen = Option::region_length;
	strcpy(outfile, Option::outfile.c_str());

	// each region has length in units of 1 kb
	// num_regions * regionlen * 1kb = 3 mb == 30 recomb.
	g_recombrate = RECRATE*regionlen*1000000.0;
	//printf("  recrate = %f*%f*%f = %f\n", RECRATE,regionlen,1000000.0, g_recombrate);




	printf("outfile =  ");

	for (i = 0; i < (int) strlen(outfile); i++) {
		printf ("%c", outfile[i]);
	}
	printf("\n");

	printf ("outbred sim = %d\n", g_outbred);
	printf ("number of iterations = %d\n", numofiterations);
	printf ("num of regions = %d\n", g_numofregions);
	printf ("num of ancestors = %d\n", g_numofancestors);
	printf ("recombination rate = %lf\n", g_recombrate);
	printf ("num of generations = %d\n", g_numofgenerations);
	printf ("average num of children = %lf\n", g_avgnumofchildren);
	printf ("outfile name = %s\n", outfile);




	if (g_recombrate > 0.5)
	{
		printf("g_recombrate, set from region, is too large\n");
		printf("  recrate = %f*%f*%f = %f\n", RECRATE,regionlen,1000000.0, g_recombrate);
		exit(-345);
	}

	if (g_outbred)
	{
		tmp_ancestors = g_numofancestors;
		g_numofancestors = tmp_ancestors*100;
	}
	else
	{
		tmp_ancestors = g_numofancestors;
	}
	g_actualnumofregions = g_numofregions;



	if (g_numofgenerations * g_numofancestors * g_numofregions > 100000000) {
		printf ("Not enough memory\n");
		exit(-1);
	}
	g_genotype = (int ***) malloc (sizeof (int **) *  g_numofgenerations);
	if (g_genotype == NULL) {
		printf ("Cannot allocate memory\n");
		exit(-1);
	}
	for (i = 0; i < g_numofgenerations; i++) {
		g_genotype[i] = (int **) malloc (sizeof (int *) * 2 *g_numofancestors * SAFENET);
		if (g_genotype[i] == NULL) {
			printf ("Cannot allocate memory\n");
			exit(-1);
		}
		for (j = 0; j < 2*g_numofancestors; j++) {
			g_genotype[i][j] = (int *) malloc (sizeof (int) * g_numofregions);
			if (g_genotype[i][j] == NULL) {
				printf ("Cannot allocate memory\n");
				exit(-1);
			}
			for (k = 0; k < g_numofregions; k++)
				g_genotype[i][j][k] = -1;
		}
	}

	g_used = (int **) malloc (sizeof (int *) * g_numofgenerations);
	if (g_used == NULL) {
		printf ("Cannot allocate memory\n");
		exit(-1);
	}
	for (i = 0; i < g_numofgenerations; i++) {
		g_used[i] = (int *) malloc (sizeof (int) * g_numofancestors * SAFENET);
		if (g_used[i] == NULL) {
			printf ("Cannot allocate memory\n");
			exit(-1);
		}
	}

	g_parents = (int ***) malloc (sizeof (int **) * 2);
	if (g_parents == NULL) {
		printf ("Cannot allocate memory\n");
		exit(-1);
	}
	for (i = 0; i < 2; i++) {
		g_parents[i] = (int **) malloc (sizeof (int *) * g_numofgenerations);
		if (g_parents[i] == NULL) {
			printf ("Cannot allocate memory\n");
			exit(-1);
		}
		for (j = 0; j < g_numofgenerations; j++) {
			g_parents[i][j] = (int *) malloc (sizeof (int) * g_numofancestors * SAFENET);
			if (g_parents[i][j] == NULL) {
				printf ("Cannot allocate memory\n");
				exit(-1);
			}
			for (k = 0; k < g_numofancestors; k++) {
				g_parents[i][j][k] = -1;
			}
		}
	}
	FILE *fp;
	FILE *fp_ped;
	FILE *fp_ibd;
	FILE *fp_gen;
	char *filename_gen;
	char *filename_ibd;
	char *filename_ped;
	char *tmp_char;
	fp = fopen (outfile, "w");
	if (fp == NULL) {
		printf ("Cannot open %s\n", outfile);
		exit(-1);
	}

	filename_gen=new char[200];
	filename_ibd=new char[200];
	filename_ped=new char[200];
	tmp_char=new char[200];

	double sum_mistakes = 0.0;
	double max_mistakes = 0.0;
	double sum_nosibship_acc = 0.0;

	double sum_edit_similarity = 0.0;
	double sum_edit_similarity_1 = 0.0;
	double sum_edit_similarity_rand = 0.0;
	double min_edit_similarity = 9999999;

	double sum_kinship_difference = 0.0;
	double sum_haplo_diff = 0.0;
	double sum_perturb_kin_diff_1 = 0.0;
	//double sum_perturb_kin_diff_5 = 0.0;
	double sum_perturb_kin_diff_rand = 0.0;


	initFactorial();

	// keep all the iteration values for computing the variance
	vector<vector<double> > values(6, vector<double>(numofiterations, 0.0));

	for (iter = 0; iter < numofiterations; iter++) {
		sprintf(tmp_char,"%d",iter);
		strcpy(filename_ped,outfile);
		filename_ped=strcat(filename_ped,"_ite_");
		filename_ped=strcat(filename_ped,tmp_char);
		filename_ped=strcat(filename_ped,".ped");

		strcpy(filename_ibd,outfile);
		filename_ibd=strcat(filename_ibd,"_ite_");
		filename_ibd=strcat(filename_ibd,tmp_char);
		filename_ibd=strcat(filename_ibd,".ibd");


		strcpy(filename_gen,outfile);
		filename_gen=strcat(filename_gen,"_ite_");
		filename_gen=strcat(filename_gen,tmp_char);
		filename_gen=strcat(filename_gen,".gen");



		fp_ibd=fopen(filename_ibd,"w");
		fp_gen=fopen(filename_gen,"w");
		fp_ped=fopen(filename_ped,"w");
		cout<<filename_ped<<endl;
		cout<<filename_ibd<<endl;
		cout<<filename_gen<<endl;
		printf ("\nIteration %d\n", iter);

		vector<int>selectedInternal;
		vector<Indiv> rand_pedigree;
		vector<int>selectedParents;
		vector<string> names;
		g_actualnumindividuals = 0;
		vector<int> extant_ptrs;
		while (g_actualnumindividuals == 0)
		{
			rand_pedigree.clear();
			extant_ptrs=drawRandPed(rand_pedigree);
		}
		cout<<"printing to pedigree file"<<endl;
		printPedFile(rand_pedigree,names,fp_ped);
		int cc;
		double rand_num;
		selectedInternal.clear();
		selectedParents.clear();	
		for (int i = 0; i < g_actualnumindividuals  ; i++) {
			cout<<"gen: "<<0<<"  "<<i<<endl;
			fflush(stdout);
			if(g_outbred){

				cc=extant_ptrs[i];
			}
			else{
				cc=i;
			}
			if(find(selectedParents.begin(),selectedParents.end(),g_parents[0][g_numofgenerations-1][cc])==selectedParents.end()){
				selectedParents.push_back(g_parents[0][g_numofgenerations-1][cc]);
			}
			if(find(selectedParents.begin(),selectedParents.end(),g_parents[1][g_numofgenerations-1][cc])==selectedParents.end()){
				selectedParents.push_back(g_parents[1][g_numofgenerations-1][cc]);
			}
			for (int j = 0; j < g_numofregions; j++) {
				fprintf(fp_ibd,"%d ",g_genotype[g_numofgenerations-1][2*i][j]);
			}
			fprintf(fp_ibd,"\n");
			for (int j = 0; j < g_numofregions; j++) {
				fprintf(fp_ibd,"%d ",g_genotype[g_numofgenerations-1][2*i+1][j]);

			}
			fprintf(fp_ibd,"\n");
			fprintf(fp_gen,"1\n1\n");
		}
		int index;
		vector<int> tmp;
		for(int i=0;i<selectedParents.size();i++)
		{	
			cc=i;
			rand_num=drand48();
			if(find(selectedInternal.begin(),selectedInternal.end(),g_parents[0][g_numofgenerations-1][cc])==selectedInternal.end())
			{
				if( rand_num<rate_internal_haplo){
					selectedInternal.push_back(g_parents[0][g_numofgenerations-1][cc]);
				}
			}
			rand_num=drand48();
			if(find(selectedInternal.begin(),selectedInternal.end(),g_parents[1][g_numofgenerations-1][cc])==selectedInternal.end())
			{

				if( rand_num<rate_internal_haplo){
					selectedInternal.push_back(g_parents[1][g_numofgenerations-1][cc]);
				}
			}
		}


		for(int k=1;k<g_numofgenerations-1;k++)
		{
			cout<<"gen: "<<k<<" ";
			fflush(stdout);	
			for(int x=0;x<selectedInternal.size();x++){
				index=selectedInternal[x];
				for (int j = 0; j < g_numofregions; j++) {
					fprintf(fp_ibd,"%d ",g_genotype[g_numofgenerations-1-k][2*index][j]);
				}
				fprintf(fp_ibd,"\n");
				for (int j = 0; j < g_numofregions; j++) {
					fprintf(fp_ibd,"%d ",g_genotype[g_numofgenerations-1-k][2*index+1][j]);

				}
				fprintf(fp_ibd,"\n");
				fprintf(fp_gen,"%d\n%d\n",k+1,k+1);

			}
			selectedInternal.clear();
			tmp.clear();	
			for(int i=0;i<selectedParents.size();i++)
			{
				cout<<"   "<<i<<endl;
				rand_num=drand48();
				index=selectedParents[i];
				if(index>-1)
				{
					if(find(tmp.begin(),tmp.end(),g_parents[0][g_numofgenerations-1-k][index])==tmp.end())
					{
						tmp.push_back(g_parents[0][g_numofgenerations-1-k][index]);
					}
					if(find(tmp.begin(),tmp.end(),g_parents[1][g_numofgenerations-1-k][index])==tmp.end())
					{

						tmp.push_back(g_parents[1][g_numofgenerations-1-k][index]);
					}
				}
			}
			cout<<"+++++ "<<tmp.size()<<endl;
			for(int i=0;i<tmp.size();i++)
			{

				index=tmp[i];
				cout<<"find: "<<i<<"   "<<index<<endl;
				fflush(stdout);
				if(index>-1){	
					rand_num=drand48();
					if(find(selectedInternal.begin(),selectedInternal.end(),index)==selectedInternal.end())
					{

						if( rand_num<rate_internal_haplo){
							selectedInternal.push_back(index);
						}

					}
				}
			}
			cout<<"+++++ "<<selectedInternal.size()<<endl;
			//	selectedParents.clear();
			selectedParents=tmp;

		}
		cout<<"printing last generation: "<<selectedInternal.size()<<endl;
		for(int x=0;x<selectedInternal.size();x++){
			index=selectedInternal[x];
			cout<<index<<endl;
			for (int j = 0; j < g_numofregions; j++) {
				fprintf(fp_ibd,"%d ",g_genotype[0][2*index][j]);
			}
			fprintf(fp_ibd,"\n");
			for (int j = 0; j < g_numofregions; j++) {
				fprintf(fp_ibd,"%d ",g_genotype[0][2*index+1][j]);

			}
			fprintf(fp_ibd,"\n");
			fprintf(fp_gen,"%d\n%d\n",g_numofgenerations,g_numofgenerations);

		}


		printf("Finished current iteration\n");	
		fclose(fp_ibd);
		fclose(fp_ped);
		fclose(fp_gen);
		vector<unsigned int> rand_generations;
		makeRandGenerations(rand_generations, rand_pedigree);
		int rand_numindiv = g_actualnumindividuals;

		vector<Indiv> sim_pedigree;
		g_actualnumindividuals = 0;
		while (g_actualnumindividuals != rand_numindiv)
		{
			sim_pedigree.clear();
			drawRandPed(sim_pedigree);
		}

		printf("\nReconstruction\n");
		for (int s = test_thresholds.size(); s < g_numofgenerations; s++)
			test_thresholds.push_back(g_threshedgetest);




		vector<vector<double> > observed_expected_sharing;
		if(Option::cop)
		{
			MatGraph*  sharing=new MatGraph( g_actualnumindividuals+1);
			observed_sharing(sharing, observed_expected_sharing,  g_actualnumindividuals);

			reconstruct_pedigree(g_numofgenerations-1, g_actualnumindividuals, test_thresholds,
					observed_expected_sharing, sharing);
		}
		else
		{
			UndirectedGraph sharing(g_actualnumindividuals+1);
			observed_sharing(sharing, observed_expected_sharing, g_actualnumindividuals);
			reconstruct_pedigreeCIP(g_numofgenerations-1, g_actualnumindividuals, test_thresholds,
					observed_expected_sharing, sharing);
		}



		vector<Indiv> perturbed_ped_1;
		vector<unsigned int> per1_newgen;
		perturbPedigree(pedigree, perturbed_ped_1, generations, per1_newgen, 0.005);
		vector<Indiv> perturbed_ped_5;
		vector<unsigned int> per5_newgen;
		perturbPedigree(pedigree, perturbed_ped_5, generations, per5_newgen, 0.05);
		printf("\n");

		double nosibship_acc = 0.0;
		double mistakes = comparePedigrees(sim_pedigree,
				pedigree, generations,
				g_actualnumindividuals,
				nosibship_acc);
		sum_nosibship_acc += nosibship_acc;
		sum_mistakes+=mistakes/2.0;
		if (mistakes/2.0 > max_mistakes)
			max_mistakes = mistakes/2.0;

		double kin_difference = kinshipDistance(sim_pedigree,
				pedigree, generations,
				g_actualnumindividuals);
		sum_kinship_difference += kin_difference;
		double haplo_difference = kinshipEmpiricalDistance(sim_pedigree,
				observed_expected_sharing,
				generations,
				g_actualnumindividuals);
		sum_haplo_diff += haplo_difference;

		double perturb_kin_difference_1 = kinshipDistance(sim_pedigree,
				perturbed_ped_1, per1_newgen,
				g_actualnumindividuals);
		sum_perturb_kin_diff_1 += perturb_kin_difference_1;


		if (rand_pedigree.size() == 0)
			printf("WARNING: rand_pedigree size 0\n");
		if (sim_pedigree.size() == 0)
			printf("WARNING: sim_pedigree size 0\n");

		double perturb_kin_difference_rand = kinshipDistance(sim_pedigree,
				rand_pedigree, rand_generations,
				g_actualnumindividuals);
		sum_perturb_kin_diff_rand += perturb_kin_difference_rand;

		values[3][iter] = kin_difference;
		values[4][iter] = perturb_kin_difference_1;
		values[5][iter] = perturb_kin_difference_rand;


		printf("Kinship difference %f\n", kin_difference);
		printf("Perturbed Kin difference %f\n", perturb_kin_difference_1);


		double edit_sameness = editDistance(sim_pedigree,
				pedigree, generations,
				g_actualnumindividuals);
		double edit_sameness_1 = editDistance(sim_pedigree,
				perturbed_ped_1, per1_newgen,
				g_actualnumindividuals);
		double edit_sameness_rand = editDistance(sim_pedigree,
				rand_pedigree, rand_generations,
				g_actualnumindividuals);


		double num_meioses = numMeioses(pedigree) + numMeioses(sim_pedigree);
		double num_meioses_1 = numMeioses(perturbed_ped_1) + numMeioses(sim_pedigree);
		double num_meioses_rand = numMeioses(rand_pedigree) + numMeioses(sim_pedigree);
		edit_sameness = 2*edit_sameness / num_meioses;
		edit_sameness_1 = 2*edit_sameness_1 / num_meioses_1;
		edit_sameness_rand = 2*edit_sameness_rand / num_meioses_rand;
		sum_edit_similarity += edit_sameness;
		sum_edit_similarity_1 += edit_sameness_1;
		sum_edit_similarity_rand += edit_sameness_rand;
		if (edit_sameness < min_edit_similarity)
			min_edit_similarity = edit_sameness;
		printf("Heuristic Edit similarity: %f\n", edit_sameness);
		printf("------------------------------\n");


		values[0][iter] = edit_sameness;
		values[1][iter] = edit_sameness_1;
		values[2][iter] = edit_sameness_rand;


		pedigree.clear();
		generations.clear();
		sim_pedigree.clear();




		double avg_edit_sim = sum_edit_similarity / (double) numofiterations;
		printf("PARTIAL Avg edit similarity %f\n", avg_edit_sim);
		//printf("PARTIAL Min edit similarity %f\n", min_edit_similarity);
		fprintf(fp, "%f\n", avg_edit_sim);
		fflush(fp);
	}
	printf("\n");
	double avg_diff = sum_mistakes/(double)numofiterations;
	printf("Avg of differences: %f\n", avg_diff);
	printf("Max num of differences: %f\n", max_mistakes);
	double max_diff = g_actualnumindividuals*(g_actualnumindividuals-1)/2.0;
	printf("Max possible num of differences: %f\n", max_diff);
	printf("\nAccuracy: %f\n\n", 1-(avg_diff/max_diff));

	printf("\n");
	double avg_edit_sim = sum_edit_similarity / (double) numofiterations;
	double avg_edit_sim_1 = sum_edit_similarity_1 / (double) numofiterations;
	double avg_edit_sim_rand = sum_edit_similarity_rand / (double) numofiterations;
	printf("Avg edit similarity %f\n", avg_edit_sim);
	printf("Min edit similarity %f\n", min_edit_similarity);



	double avg_kin_diff = sum_kinship_difference / (double) numofiterations;
	double avg_perturb_kin_diff_1 = sum_perturb_kin_diff_1 / (double) numofiterations;
	double avg_perturb_kin_diff_rand = sum_perturb_kin_diff_rand / (double) numofiterations;
	vector<double> means(6,0.0);
	means[0] = avg_edit_sim;
	means[1] = avg_edit_sim_1;
	means[2] = avg_edit_sim_rand;
	means[3] = avg_kin_diff;
	means[4] = avg_perturb_kin_diff_1;
	means[5] = avg_perturb_kin_diff_rand;

	vector<double> var(6,0.0);
	compute_variance(var, values, means);

	fprintf(fp, "%f %f %f %f %f %f\n",
			avg_edit_sim,
			avg_edit_sim_1,
			avg_edit_sim_rand,
			avg_kin_diff,
			avg_perturb_kin_diff_1,
			avg_perturb_kin_diff_rand);
	for (unsigned int i = 0;  i < var.size();  i++)
	{
		fprintf(fp, "%f", var[i]);
		if (i != var.size()-1)
			fprintf(fp, " ");
	}
	fprintf(fp, "\n");


	fclose(fp);



	// end iteration for each pedigree replicate
	exit(0);


}
