#include <vector>
#include <map>
#include <queue>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>


#include "test_alg4.h"
#include "graph.h"
#include "ibd_rw.h"



using namespace std;







int          Option::number_of_regions=3000;
int          Option::number_of_iterations=100; 
int          Option::number_of_ancestors=10; 
int          Option::number_of_generations=3;
int          Option::number_of_children=3;
int          Option::threshold_distance=0; 
int          Option::threshold_similarity=0; 
double       Option::threshold_edge_test=0.70; 
double       Option::region_length=1.0;
double       Option::a_value=0.01; 
int          Option::usegraph=1;
string       Option::outfile="outfile"; 
int          Option::outbred_sim=0;
int          Option::cop=1;
 
string       Option::in_file="";
string       Option::out_file="";


#define USAGE "simulate_gbirp \n \
              [-r num_of_regions       (default 3000)]\n \
              [-t num_of_iterations    (default 100)]\n \
              [-p num_of_ancestors     (default 10)]\n \
              [-g num_of_generations   (default 3)]\n \
              [-c num_of_children      (default 3)]\n \
              [-s outbred simulation   (default 0)]\n \
              [-e threshold_edge_test  (default 0.7)]\n \
              [-l region_length in mb  (default 1.0)]\n \
              [-a a_value              (default 0.01)]\n \
              [-m methold  (CIP/COP)   (default COP)]\n \
              [-o out_file             (default outfile)]\n"


void
initialize_options(int argc, char** argv)
{

   char c=0;
   while((c=getopt(argc, argv, "r:t:p:g:c:s:e:l:a:m:o:"))!=EOF)
   {
      switch(c)
      {
         case 'r': if(optarg==NULL) 
                   {
                      printf(USAGE);exit(0);
                   }  
                   Option::number_of_regions=atoi(optarg); break;
         case 't': if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }  
                   Option::number_of_iterations=atoi(optarg); break;
         case 'p': if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }  
                   Option::number_of_ancestors=atoi(optarg); break;
         case 'g': if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }  
                   Option::number_of_generations=atoi(optarg); break;
         case 'c': if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }  
                   Option::number_of_children=atoi(optarg); break;
         case 's': if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }  
                   Option::outbred_sim=atoi(optarg); 
		   if (Option::outbred_sim != 0 && Option::outbred_sim != 1)
		     {
		       printf("Outbred value is boolean\n");
		       printf(USAGE);
		       exit(0);
		     }
		   break;

         case 'e': if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }  
                   Option::threshold_edge_test=atof(optarg); break;
         case 'l':if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }   
                   Option::region_length=atof(optarg); break;
         case 'a': 
                   Option::a_value=atoi(optarg); break;
         case 'm':
           if(optarg==NULL || (strcmp(optarg, "CIP")!=0 && strcmp(optarg, "COP")!=0))
           {
             cerr<<"Methold can only be CIP or COP \n"<<endl;
             exit(0);
           }
           else if(strcmp(optarg, "CIP")==0)
           { 
             Option::cop= 0;
           } 
           break;
         case 'o': if(optarg==NULL)
                   {
                      printf(USAGE);exit(0);
                   }  
                 Option::outfile=optarg; break;
         default:
             printf(USAGE);
             exit(0);
      } 

   }

}







// ASSUME genotypes are PHASED
//
// Genotypes of extant generation are in
//   g_genotypes[g_numofgenerations-1][chr][pos]
//      where chr = 2 * indiv + j, with j in {0,1} 
//      and pos in [0,...,g_numofregions]
void printPedFile(char* _file, 
		  vector<Indiv>& sim_pedigree,
		  int lb_pos,
		  int ub_pos)
{


  FILE *fp = fopen (_file, "w");
  if (fp == NULL) {
    printf ("Cannot open %s\n", _file);
    exit(-1);
  } else {
    printf ("opened %s\n", _file);
  }


  for (unsigned int j=1;  j < sim_pedigree.size(); j++)
    {
      int p1 = 0;
      int p2 = 0;
      if (sim_pedigree[j].parents.size() == 2)
	{
	  p1 =sim_pedigree[j].parents[0];
	  p2 =sim_pedigree[j].parents[1];
	}
      
      fprintf(fp, "1 %i %i %i %i %i  ", j, 
	      p1, p2,
	      sim_pedigree[j].gender,
	      0
	      );


      // if typed
      int i = j-1;
      for (int pos = lb_pos;  pos < ub_pos; pos++)
	{
	  int a_i0 = 0;
	  int a_i1 = 0;
	  if (sim_pedigree[j].generation == 1)
	    {
	      // indicates orientation of sharing (consider both pairings of haps)
	      a_i0 = g_genotype[g_numofgenerations-1][2*i][pos];
	      a_i1 = g_genotype[g_numofgenerations-1][2*i+1][pos];

	      a_i0 = (int) floor((a_i0-pos)/(double) g_numofregions) +1;
	      a_i1 = (int) floor((a_i1-pos)/(double) g_numofregions) +1;
	    }
	  fprintf(fp, "%i %i ", a_i0, a_i1);
	} // end for each position
      fprintf(fp, "\n");

    } // end for each individual

  fclose(fp);
}




void printDatFile(char* _file,
		  int lb_pos,
		  int ub_pos)
{

  int num_pos = ub_pos - lb_pos;


  FILE* fp = fopen (_file, "w");
  if (fp == NULL) {
    printf ("Cannot open %s\n", _file);
    exit(-1);
  } else {
    printf ("opened %s\n", _file);
  }

  fprintf(fp, "%i 0 0 5 << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM\n", num_pos+1);
  fprintf(fp, "0 0.0 0.0 0 << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)\n");
  for (int pos = 0;  pos < num_pos+1; pos++)
    {
      fprintf(fp, "%i ", pos+1);
    }
  fprintf(fp, "\n");
  fprintf(fp, "1 2 << AFFECTION, NO. OF ALLELES\n");
  fprintf(fp, "0.88 0.12 << GENE FREQUENCIES (change if required)\n");
  fprintf(fp, "1 << NO. OF LIABILITY CLASSES (change if required)\n");
  fprintf(fp, "0.003 0.900 0.900 << PENETRANCES (change if required)\n");

  // 3  12 << D16S521 Marker 1 ALLELE NUMBERS, NO. OF ALLELES
  // 0.424 0.0193 0.00148 0.00148 0.00148 0.0593 0.223 0.166 0.0668 0.0237 0.0119 0.00148

  double f =  1.0 / (double) (2*g_numofancestors);
  for (int pos = lb_pos;  pos < ub_pos; pos++)
    {
      fprintf(fp, "3 %i << MARKER%i\n", 2*g_numofancestors, pos);
      for (int i=0; i < 2*g_numofancestors; i++)
	{
	  fprintf(fp, "%f ", f);
	}
      fprintf(fp, "\n");
    } // end for each position


  fprintf(fp, "0 0 << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)\n");
  fprintf(fp, "0.0 "); // disease locus recomb rate
  for (int pos = lb_pos;  pos < ub_pos-1; pos++)
    {
      fprintf(fp, "%f ", g_recombrate);
    }
  fprintf(fp,"\n");
  fprintf(fp, "1 0.1 0.45 << REC VARIED, INCREMENT, FINISHING VALUE\n");


  fclose(fp);
}




void printHaploFile(char* _file, 
		    vector<Indiv>& sim_pedigree,
		    int lb_pos,
		    int ub_pos)
{


  FILE *fp = fopen (_file, "w");
  if (fp == NULL) {
    printf ("Cannot open %s\n", _file);
    exit(-1);
  } else {
    printf ("opened %s\n", _file);

  }


  fprintf(fp, "FAMILY 1 [Uninformative]\n");


  for (unsigned int j=sim_pedigree.size()-1;  j >= 1; j--)
    {
      int p1 = 0;
      int p2 = 0;
      if (sim_pedigree[j].parents.size() == 2)
	{
	  p1 =sim_pedigree[j].parents[0];
	  p2 =sim_pedigree[j].parents[1];
	  fprintf(fp, "%i (%i,%i)\n", j, p1, p2);
	}
      else
	{
	  fprintf(fp, "%i (F)\n", j);
	}



      // if typed
      int i = j-1;
      for (int pos = lb_pos;  pos < ub_pos; pos++)
	{
	  int a_i0 = 0;
	  int a_i1 = 0;
	  if (sim_pedigree[j].generation == 1)
	    {
	      // indicates orientation of sharing (consider both pairings of haps)
	      a_i0 = g_genotype[g_numofgenerations-1][2*i][pos];
	      a_i1 = g_genotype[g_numofgenerations-1][2*i+1][pos];

	      a_i0 = (int) floor((a_i0-pos)/(double) g_numofregions) +1;
	      a_i1 = (int) floor((a_i1-pos)/(double) g_numofregions) +1;
	      fprintf(fp, "%i  |  %i \n", a_i0, a_i1);
	    }
	  else
	    {
	      fprintf(fp, "?  :  ? \n");
	    }

	} // end for each position
      fprintf(fp, "\n");

    } // end for each individual


  // for each individual
  for (unsigned int j=sim_pedigree.size()-1;  j >= 1; j--)
    {
      for (int g = 0;  g < 2;  g++)
	{
	  if (sim_pedigree[j].generation == 1)
	    {
	      fprintf(fp, "    %i (FOUNDER)  ", j);
	    }
	  else if (g == 1)
	    {
	      fprintf(fp, "    %i (MATERNAL)  ", j);
	    }
	  else 
	    {
	      fprintf(fp, "    %i (PATERNAL)  ", j);
	    }


	  int i = j-1;
	  for (int pos = lb_pos;  pos < ub_pos; pos++)
	    {
	      int a_i = 0;
	      if (sim_pedigree[j].generation == 1)
		{
		  // indicates orientation of sharing (consider both pairings of haps)
		  a_i = g_genotype[g_numofgenerations-1][2*i+g][pos];
		  a_i = (int) floor((a_i-pos)/(double) g_numofregions) +1;
		  
		  fprintf(fp, " %i  ", a_i);
		}
	      else
		{
		  fprintf(fp, "?  ");
		}
	    } // end for each position
	  fprintf(fp, "\n");
	} // end for each gamete


    } // end for each individual



  fclose(fp);
}



void printIDFile(char* _file, 
		 vector<Indiv>& sim_pedigree)
{


  FILE *fp = fopen (_file, "w");
  if (fp == NULL) {
    printf ("Cannot open %s\n", _file);
    exit(-1);
  } else {
    printf ("opened %s\n", _file);
  }

  for (unsigned int j=1;  j < sim_pedigree.size(); j++)
    {
      if (sim_pedigree[j].generation == 1)
	{
	  fprintf(fp, "%i ", j);
	}
      else
	break;
    } // end for each individual
  fprintf(fp, "\n");

  fclose(fp);
}




void makeOutput(char* base, 
		vector<Indiv>& sim_pedigree)
{
  char fname[100];
  sprintf(fname, "%s.idlist", base);
  printIDFile(fname, sim_pedigree);


  // make 22 chromosomes
  double frac_loci = (double) g_numofregions / (double) 22;
  int loci_all = (int) floor(frac_loci);
  int num_extra = g_numofregions - loci_all*22;
  printf("num regions %i, frac_loci %f, loci_all %i, num_extra %i\n",
	 g_numofregions, frac_loci, loci_all, num_extra);

  int pos = 0;
  int lb_pos;
  int ub_pos;
  for (int c = 1; c <= 22; c++)
    {
      // positions:
      if (c <= num_extra){
	lb_pos = pos;
	ub_pos = pos+loci_all+1;
      } else{
	lb_pos = pos;
	ub_pos = pos+loci_all;
      }
      // [lb_pos, ub_pos)
      
      char* s = new char[100];
      sprintf(s, "%s.c%i.pre", base, c);
      printPedFile(s, sim_pedigree, lb_pos, ub_pos);

      sprintf(s, "%s.c%i.dat", base, c);
      printDatFile(s, lb_pos, ub_pos);

      sprintf(s, "%s.c%i.merlin.haplos", base, c);
      printHaploFile(s, sim_pedigree, lb_pos, ub_pos);
      delete[] s;
      
      pos=ub_pos;
    }
};



void relatedness(char* _file, 
		 vector<Indiv>& sim_pedigree)
{

  FILE *fp = fopen (_file, "w");
  if (fp == NULL) {
    printf ("Cannot open %s\n", _file);
    exit(-1);
  } else {
    printf ("opened %s\n", _file);
  }



  for (unsigned int j=1;  j < sim_pedigree.size(); j++)
    {
      if (sim_pedigree[j].generation == 1)
	{
	  for (unsigned int i=j;  i < sim_pedigree.size(); i++)
	    {
	      if (sim_pedigree[i].generation == 1)
		{
		  unsigned int color = 9;
		  resetColor(sim_pedigree);
		  colorPedigree(sim_pedigree, j,i, color);

		  //printPedigree(sim_pedigree);

		  int min_meioses = 99999;
		  int m = minMeioses(sim_pedigree, j,i, color, 0,min_meioses);
		  //printf( "%i %i %i\n", j, i, m);
		  fprintf(fp, "%i %i %i\n", j, i, m);
		}
	      else
		break;
	    }
	}
      else
	break;
    } // end for each individual





  fclose(fp);
}















int main (int argc, char **argv)
{


  FILE *fp;
  double regionlen;
  int i,j,k;
  long int seed;
  char outfile[100];
  int numofiterations = 1;
  int usegraph = 0;
  int iter;


  initialize_options(argc, argv);


   if (Option::cop)
     {
       printf("Simulator + COP Algorithm\n-------------------------\n\n"); 
     }
   else
     {
       printf("Simulator + CIP Algorithm\n-------------------------\n\n"); 
     }






  g_numofregions = Option::number_of_regions;
  g_actualnumofregions = g_numofregions;
  numofiterations = Option::number_of_iterations;
  g_numofancestors = Option::number_of_ancestors;
  g_numofgenerations = Option::number_of_generations;
  g_avgnumofchildren = Option::number_of_children;
  g_threshdist       = Option::threshold_distance;
  g_threshsimilarity= Option::threshold_similarity;
  g_threshedgetest=   Option::threshold_edge_test;
  g_outbred = Option::outbred_sim;
  regionlen = Option::region_length;
  g_regionlen = Option::region_length;
  strcpy(outfile, Option::outfile.c_str());

  // each region has length in units of 1 kb
  // num_regions * regionlen * 1kb = 3 mb == 30 recomb.
  g_recombrate = RECRATE*g_regionlen*1000000.0;

  
  if (g_outbred)
    {
      tmp_ancestors = g_numofancestors;
      g_numofancestors = tmp_ancestors*100;
    }
  else
    {
      tmp_ancestors = g_numofancestors;
    }


  //printf("  recrate = %f*%f*%f = %f\n", RECRATE,regionlen,1000000.0, g_recombrate);

  
  
  

  seed = time(NULL);
  srand48(seed);
  


  printf("outfile =  ");
  for (i = 0; i < (int) strlen(outfile); i++) {
    printf ("%c", outfile[i]);
  }
  printf("\n");

  printf ("outbred sim = %d\n", g_outbred);
  printf ("number of iterations = %d\n", numofiterations);  
  printf ("num of regions = %d\n", g_numofregions);
  printf ("num of ancestors = %d\n", g_numofancestors);
  printf ("recombination rate = %lf\n", g_recombrate);
  printf ("num of generations = %d\n", g_numofgenerations);
  printf ("average num of children = %lf\n", g_avgnumofchildren);
  printf ("thresh edge_test = %f\n",g_threshedgetest);
  printf ("clustering a = %f\n", Option::a_value);
  printf ("outfile name = %s\n", outfile);

  

  if (g_recombrate > 0.5)
    {
      printf("g_recombrate, set from region, is too large\n");
      printf("  recrate = %f*%f*%f = %f\n", RECRATE,regionlen,1000000.0, g_recombrate);
      exit(-345);
    }

  if (g_numofgenerations * g_numofancestors * g_numofregions > 100000000) {
    printf ("Not enough memory\n");
    exit(-1);
  }


  g_genotype = (int ***) malloc (sizeof (int **) *  g_numofgenerations);
  if (g_genotype == NULL) {
    printf ("Cannot allocate memory\n");
    exit(-1);
  }
  for (i = 0; i < g_numofgenerations; i++) {
    g_genotype[i] = (int **) malloc (sizeof (int *) * 2 *g_numofancestors * SAFENET);
    if (g_genotype[i] == NULL) {
      printf ("Cannot allocate memory\n");
      exit(-1);
    }
    for (j = 0; j < 2*g_numofancestors; j++) {
      g_genotype[i][j] = (int *) malloc (sizeof (int) * g_numofregions);
      if (g_genotype[i][j] == NULL) {
	printf ("Cannot allocate memory\n");
	exit(-1);
      }
      for (k = 0; k < g_numofregions; k++)
	g_genotype[i][j][k] = -1;
    }
  }
  
  g_used = (int **) malloc (sizeof (int *) * g_numofgenerations);
  if (g_used == NULL) {
    printf ("Cannot allocate memory\n");
    exit(-1);
  }
  for (i = 0; i < g_numofgenerations; i++) {
    g_used[i] = (int *) malloc (sizeof (int) * g_numofancestors * SAFENET);
    if (g_used[i] == NULL) {
      printf ("Cannot allocate memory\n");
      exit(-1);
    }
  }

  g_parents = (int ***) malloc (sizeof (int **) * 2);
  if (g_parents == NULL) {
    printf ("Cannot allocate memory\n");
    exit(-1);
  }
  for (i = 0; i < 2; i++) {
    g_parents[i] = (int **) malloc (sizeof (int *) * g_numofgenerations);
    if (g_parents[i] == NULL) {
      printf ("Cannot allocate memory\n");
      exit(-1);
    }
    for (j = 0; j < g_numofgenerations; j++) {
      g_parents[i][j] = (int *) malloc (sizeof (int) * g_numofancestors * SAFENET);
      if (g_parents[i][j] == NULL) {
	printf ("Cannot allocate memory\n");
	exit(-1);
      }
      for (k = 0; k < g_numofancestors; k++) {
	g_parents[i][j][k] = -1;
      }
    }
  }
  
  fp = fopen (outfile, "w");
  if (fp == NULL) {
    printf ("Cannot open %s\n", outfile);
    exit(-1);
  } else {
    printf ("opened %s\n", outfile);

  }


  double sum_mistakes = 0.0;
  double max_mistakes = 0.0;
  double sum_nosibship_acc = 0.0;

  double sum_edit_similarity = 0.0;
  double sum_edit_similarity_1 = 0.0;
  double sum_edit_similarity_rand = 0.0;
  double min_edit_similarity = 9999999;

  double sum_kinship_difference = 0.0;
  double sum_haplo_diff = 0.0;
  double sum_perturb_kin_diff_1 = 0.0;
  //double sum_perturb_kin_diff_5 = 0.0;
  double sum_perturb_kin_diff_rand = 0.0;


  for (iter = 0; iter < numofiterations; iter++) {

    printf ("\nIteration %d\n", iter);

    vector<Indiv> rand_pedigree;

    g_actualnumindividuals = 0;
    while (g_actualnumindividuals == 0)
      {
	rand_pedigree.clear();
	drawRandPed(rand_pedigree);
	printf("actual people: %i\n", g_actualnumindividuals);
      }
    vector<unsigned int> rand_generations;
    makeRandGenerations(rand_generations, rand_pedigree);
    int rand_numindiv = g_actualnumindividuals;

    vector<Indiv> sim_pedigree;
    g_actualnumindividuals = 0;
    while (g_actualnumindividuals != rand_numindiv)
      {
	sim_pedigree.clear();
	drawRandPed(sim_pedigree);
	printf("actual people: %i\n", g_actualnumindividuals);
      }



    //vector<unsigned int> sim_generations;
    //makeRandGenerations(sim_generations, sim_pedigree);
    //printf("Sim ped gens, for sim with %i people with param %i,%i:\n", sim_pedigree.size(), 
    //   g_actualnumindividuals, g_numofancestors);
    //for (unsigned int g = 0;  g < sim_generations.size();  g++)
    //  printf("generations[%i] = %i\n", g, sim_generations[g]);


    ////////////////////
    //  Output files 
    //   for merlin
    //
    // use g_genotype
    makeOutput("merlin", sim_pedigree);

    relatedness("relatedness.sim", sim_pedigree);

    initFactorial();



    // reconstruct pedigree
    printf("\nReconstruction\n");
    for (int s = test_thresholds.size(); s < g_numofgenerations; s++)
      test_thresholds.push_back(g_threshedgetest);



    //reconstruct_pedigree(g_numofgenerations-1, 
    //                     g_actualnumindividuals, 
    //                     test_thresholds,
    //                     observed_expected_sharing);




    vector<vector<double> > observed_expected_sharing;
    if(Option::cop)
    {
       MatGraph*  sharing=new MatGraph( g_actualnumindividuals+1);
      observed_sharing(sharing, observed_expected_sharing,  g_actualnumindividuals);

        reconstruct_pedigree(g_numofgenerations-1, g_actualnumindividuals, test_thresholds,
                         observed_expected_sharing, sharing);
    }
    else
    {
       UndirectedGraph sharing(g_actualnumindividuals+1);
       observed_sharing(sharing, observed_expected_sharing, g_actualnumindividuals);
       reconstruct_pedigreeCIP(g_numofgenerations-1, g_actualnumindividuals, test_thresholds,
                         observed_expected_sharing, sharing);
    }



 
    //printPedigree(pedigree);

    relatedness("relatedness.est", pedigree);

    // Two perturbed pedigrees
    vector<Indiv> perturbed_ped_1;
    vector<unsigned int> per1_newgen;
    perturbPedigree(pedigree, perturbed_ped_1, generations, per1_newgen, 0.005);
    vector<Indiv> perturbed_ped_5;
    vector<unsigned int> per5_newgen;
    perturbPedigree(pedigree, perturbed_ped_5, generations, per5_newgen, 0.05);

    // compare reconstruction with actual
    printf("\n");


    //printf("Est ped gens:\n");
    //for (unsigned int g = 0;  g < generations.size();  g++)
    //  printf("generations[%i] = %i\n", g, generations[g]);


    double nosibship_acc = 0.0;
    double mistakes = comparePedigrees(sim_pedigree, 
				       pedigree, generations, 
				       g_actualnumindividuals,
				       nosibship_acc);
    sum_nosibship_acc += nosibship_acc;
    sum_mistakes+=mistakes/2.0;
    if (mistakes/2.0 > max_mistakes)
      max_mistakes = mistakes/2.0;
    printf("Number of differences: %f\n", mistakes/2.0);

    double kin_difference = kinshipDistance(sim_pedigree, 
					    pedigree, generations, 
					    g_actualnumindividuals);
    sum_kinship_difference += kin_difference;
    double haplo_difference = kinshipEmpiricalDistance(sim_pedigree, 
						       observed_expected_sharing,
						       generations, 
						       g_actualnumindividuals);
    sum_haplo_diff += haplo_difference;

    double perturb_kin_difference_1 = kinshipDistance(sim_pedigree, 
						    perturbed_ped_1, per1_newgen, 
						    g_actualnumindividuals);
    sum_perturb_kin_diff_1 += perturb_kin_difference_1;


    if (rand_pedigree.size() == 0)
      printf("WARNING: rand_pedigree size 0\n");
    if (sim_pedigree.size() == 0)
      printf("WARNING: sim_pedigree size 0\n");

    double perturb_kin_difference_rand = kinshipDistance(sim_pedigree, 
							 rand_pedigree, rand_generations, 
							 g_actualnumindividuals);
    sum_perturb_kin_diff_rand += perturb_kin_difference_rand;





    printf("Kinship difference %f\n", kin_difference);
    printf("Haplo Kinship difference %f\n", haplo_difference);
    printf("Perturbed Kin difference %f\n", perturb_kin_difference_1);


    double edit_sameness = editDistance(sim_pedigree, 
					pedigree, generations, 
					g_actualnumindividuals);
    double edit_sameness_1 = editDistance(sim_pedigree, 
    				  perturbed_ped_1, per1_newgen, 
    				  g_actualnumindividuals);
    double edit_sameness_rand = editDistance(sim_pedigree, 
    				  rand_pedigree, rand_generations, 
    				  g_actualnumindividuals);
    //printf("edit sameness 0: %f, 1: %f, 5: %f\n", edit_sameness, edit_sameness_1, edit_sameness_5);

    double num_meioses = numMeioses(pedigree) + numMeioses(sim_pedigree);
    double num_meioses_1 = numMeioses(perturbed_ped_1) + numMeioses(sim_pedigree);
    double num_meioses_rand = numMeioses(rand_pedigree) + numMeioses(sim_pedigree);
    edit_sameness = 2*edit_sameness / num_meioses;
    edit_sameness_1 = 2*edit_sameness_1 / num_meioses_1;
    edit_sameness_rand = 2*edit_sameness_rand / num_meioses_rand;
    sum_edit_similarity += edit_sameness;
    sum_edit_similarity_1 += edit_sameness_1;
    sum_edit_similarity_rand += edit_sameness_rand;
    if (edit_sameness < min_edit_similarity)
      min_edit_similarity = edit_sameness;
    printf("Edit similarity: %f of %f (perturbed %f)\n", edit_sameness, num_meioses, edit_sameness_1);
    printf("------------------------------\n");

    pedigree.clear();
    generations.clear();
    sim_pedigree.clear();




    double avg_edit_sim = sum_edit_similarity / (double) numofiterations;
    printf("PARTIAL Avg edit similarity %f\n", avg_edit_sim);
    printf("PARTIAL Min edit similarity %f\n", min_edit_similarity);
    fprintf(fp, "%f\n", avg_edit_sim);

//     double avg_diff = sum_mistakes/(double) (iter+1);
//     printf("PARTIAL Avg of differences: %f\n", avg_diff);
//     printf("PARTIAL Max num of differences: %f\n", max_mistakes);
//     double max_diff = g_actualnumindividuals*(g_actualnumindividuals-1)/2.0;
//     printf("Max possible num of differences: %f\n", max_diff);
//     printf("\nPARTIAL Accuracy: %f\n", 1-(avg_diff/max_diff));
//     printf("------------------------------\n");

//     double rep_acc = 1- ((mistakes/2.0)/ max_diff);
//     fprintf(fp, "%f  %f\n", rep_acc, nosibship_acc);
    


  } // end iteration for each pedigree replicate

  printf("\n");
  double avg_diff = sum_mistakes/(double)numofiterations;
  printf("Avg of differences: %f\n", avg_diff);
  printf("Max num of differences: %f\n", max_mistakes);
  double max_diff = g_actualnumindividuals*(g_actualnumindividuals-1)/2.0;
  printf("Max possible num of differences: %f\n", max_diff);
  printf("\nAccuracy: %f\n\n", 1-(avg_diff/max_diff));
  
  printf("\n");
  double avg_edit_sim = sum_edit_similarity / (double) numofiterations;
  double avg_edit_sim_1 = sum_edit_similarity_1 / (double) numofiterations;
  double avg_edit_sim_rand = sum_edit_similarity_rand / (double) numofiterations;
  printf("Avg edit similarity %f\n", avg_edit_sim);
  printf("Min edit similarity %f\n", min_edit_similarity);


  
  double avg_kin_diff = sum_kinship_difference / (double) numofiterations;
  double avg_perturb_kin_diff_1 = sum_perturb_kin_diff_1 / (double) numofiterations;
  double avg_perturb_kin_diff_rand = sum_perturb_kin_diff_rand / (double) numofiterations;
  //double avg_haplo_diff = sum_haplo_diff / (double) numofiterations;
  

  // Avg Accuracy
  //fprintf(fp, "%f  %f\n", 1-(avg_diff/max_diff), sum_nosibship_acc/(double)numofiterations);
  //fprintf(fp, "%f\n", avg_edit_sim);
  fprintf(fp, "%f %f %f %f %f %f\n", 
	  avg_edit_sim, 
	  avg_edit_sim_1,
	  avg_edit_sim_rand,
	  avg_kin_diff, 
	  avg_perturb_kin_diff_1, 
	  avg_perturb_kin_diff_rand);

  fclose(fp);



  exit(0);

}
