#ifndef TESTALG
#define TESTALG

#include <vector>
#include <map>
#include <queue>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>



using namespace std;
#define FACTSIZE 200
vector<int> factorial_table(FACTSIZE, 1);


#include "ibd_rw.h"
#include "graph.h"

//#define RECRATE 0.1
#define RECRATE 0.00000001
#define SAFENET 1



// for debugging, uncomment
//#define MODELS 



// Pedigree Data Structure

// first index is generation 
// (in reconstruction order... reverse of generation order)
// second index is individual
vector<Indiv> pedigree;
// person's index is their integer in children, parent, and descent lists

// list of indices that begin each generation
vector<unsigned int> generations;



vector<double> test_thresholds;

// fraction of paths to keep in vector
double fraction = 0.6;

vector<vector<vector<int> *> *>* mExtantChildren;



int tmp_ancestors;
bool g_outbred;
double g_rate_of_half_sibling;
int allow_hs_mate;
int allow_sb_mate;
double rate_internal_haplo;

int g_actualnumofregions;

int g_numofregions; 
int g_numofancestors;
int g_numofgenerations;
double g_avgnumofchildren;

int g_threshdist;
int g_threshsimilarity;
double g_threshedgetest;
double g_recombrate;
double g_regionlen;

int ***g_genotype;
int **g_used;
int ***g_parents;

int g_actualnumindividuals;



void initFactorial()
{
	factorial_table[0] = 1;
	for (int i = 1 ; i < FACTSIZE; i++)
	{
		factorial_table[i] = factorial_table[i-1] * i;
	}
}

void initializegeneration0() {

	int i,j;

	for (i = 0; i < 2*g_numofancestors; i++) {
		for (j = 0; j < g_numofregions; j++) {
			g_genotype[0][i][j] = i*g_numofregions + j;
		}
	}

	for (i = 0; i < g_numofancestors; i++) {
		g_used[0][i] = 1;
	}

	return;
}


long int factorial(int n) {

	int i;
	long int sum;

	sum = 1;

	for (i = 1; i <= n; i++) {
		sum = sum*i;
	}
	return(sum);

}

void constructindividual (int generation, int mother, int father, int ind) {

	int pos, chr, parentchr, flag;
	double tmp;

	chr = 2*ind;
	parentchr = 2*mother;
	tmp = drand48();
	if (tmp < 0.5)
		flag = 0;
	else
		flag = 1;
	for (pos = 0; pos < g_numofregions; pos++) {
		g_genotype[generation][chr][pos] = g_genotype[generation-1][parentchr+flag][pos];
		tmp = drand48();
		if (tmp < g_recombrate){
			flag = 1-flag;
		}
	}

	chr = 2*ind+1;
	parentchr = 2*father;
	tmp = drand48();
	if (tmp < 0.5)
		flag = 0;
	else
		flag = 1;
	for (pos = 0; pos < g_numofregions; pos++) {
		g_genotype[generation][chr][pos] = g_genotype[generation-1][parentchr+flag][pos];
		tmp = drand48();
		if (tmp < g_recombrate)
			flag = 1-flag;
	}
}


void buildgeneration(int generation) {

	int i,j;
	int mother, father;
	int second_wife;
	char *used;
	int totalused = 0;
	double tmp;
	int numofchildren;
	double sum;
	int far_child;
	int grand_id;
	map<unsigned int,unsigned int> father_NA;
	map<unsigned int, unsigned int> mother_NA;


	printf ("Building generation %d\n", generation);
	//  fflush(stdout);
	used = (char *) malloc (sizeof (char)* g_numofancestors*SAFENET);
	if (used == NULL) {
		printf ("Cannot allocate memory\n");
		exit(-1);
	}
	father_NA.clear();
	mother_NA.clear();

	for (i = 0; i < g_numofancestors; i++) {
		used[i] = 1-g_used[generation-1][i];
		grand_id=g_parents[1][generation-1][i];
		totalused += used[i];
		//cout<<""<<(char)used[i]<<"\t"<<totalused<<endl;;
		if(grand_id==-1)
		{	
			if(used[i]==0)
			{
				cout<<i<<"    -1"<<endl;
				father_NA[i]=1;
			}
		}
		else
		{
			map<unsigned int,unsigned int>::const_iterator mi = father_NA.find(grand_id);
			if(mi==father_NA.end());
			{
				if(used[i]==0)
				{
					cout<<i<<"   "<<grand_id<<endl;
					father_NA[grand_id]=1;
				}
			}
		}
		grand_id=g_parents[0][generation-1][i];
		if(grand_id==-1)
		{
			if(used[i]==0)
			{
				cout<<i<<"    -1"<<endl;
				mother_NA[i]=1;
			}
		}
		else
		{
			map<unsigned int,unsigned int>::const_iterator mi = mother_NA.find(grand_id);
			if(mi==mother_NA.end());
			{
				if(used[i]==0)
				{
					cout<<i<<"   "<<grand_id<<endl;
					mother_NA[grand_id]=1;
				}
			}
		}

		g_used[generation][i] = 0;
	}
	cout<<"initialized."<<endl;
	cout<<"fathers: "<<father_NA.size()<<endl;
	cout<<"mothers: "<<mother_NA.size()<<endl;
	i = 0;
	//printf("g_numofancestors %i\n", g_numofancestors);
	//WAHT if only sibling left, but we allow hs mating and not sb mating.
	while (totalused < g_numofancestors-1 && i < g_numofancestors &&(allow_hs_mate|| father_NA.size()>1)&&(allow_sb_mate || mother_NA.size()>1)) {
		//    printf ("total used = %d i = %d\n", totalused, i);

		father_NA.clear();
		mother_NA.clear();
		tmp = drand48();
		father = (int)(tmp * (double) g_numofancestors);	

		while (used[father] ) {
			tmp = drand48();
			father = (int)(tmp * (double) g_numofancestors);
		}
		used[father]=1;
		tmp = drand48();
		mother = (int)(tmp * (double) g_numofancestors);
		while (used[mother]||(allow_hs_mate==0 && g_parents[1][generation-1][mother]==g_parents[1][generation-1][father]&&g_parents[1][generation-1][mother]!=-1)||(allow_sb_mate==0 && g_parents[0][generation-1][mother]==g_parents[0][generation-1][father]&&g_parents[0][generation-1][mother]!=-1)) {
			tmp = drand48();
			mother = (int)(tmp * (double) g_numofancestors);      
		}

		//    printf ("mother = %d ", mother);
		//fflush(stdout);
		used[father] = used[mother] = 1;
		totalused += 2;
		cout<<"total used: "<<totalused<<endl;		
		/*	if(g_parent[1][generation-1][father]!=-1)
			{
			grand_NA[g_parent[1][generation-1][father]]=1;
			}
			if(g_parent[1][generation-1][mother]!=-1)
			{
			grand_NA[g_parent[1][generation-1][mother]]=1;		
			}*/

		cout<<"parent allocated"<<endl;
		for(int k=0;k<g_numofancestors;k++)
		{
			grand_id=g_parents[1][generation-1][k];
			if(grand_id==-1)
			{	
				if(used[k]==0)
				{
					father_NA[k]=1;
				}
			}
			else{
				map<unsigned int,unsigned int>::const_iterator mi = father_NA.find(grand_id);
				if(mi==father_NA.end())
				{
					if(used[k]==0)
					{
						father_NA[grand_id]=1;
					}
				}
			}
			grand_id=g_parents[0][generation-1][k];
			if(grand_id==-1)
			{
				if(used[k]==0)
				{
					mother_NA[k]=1;
				}
			}
			else{
				map<unsigned int,unsigned int>::const_iterator mi = mother_NA.find(grand_id);
				if(mi==mother_NA.end())
				{
					if(used[k]==0)
					{
						mother_NA[grand_id]=1;
					}
				}
			}

		}

		cout<<"count available"<<endl;
		//    printf ("father = %d\n", father);
		//fflush(stdout);

		numofchildren = 0;
		sum = exp(-g_avgnumofchildren);
		tmp = drand48();
		if (tmp < sum){
			numofchildren = 0;

		}
		else {
			j = 1;
			while (numofchildren == 0) {
				sum = sum + pow(g_avgnumofchildren,j)*exp(-g_avgnumofchildren)/factorial(j);
				if (tmp < sum) {
					numofchildren = j;
				}
				j++;
			}
		}
		for (j = 0; i+j < g_numofancestors && j < numofchildren; j++) {
			g_used[generation][i+j] = 1;
			//printf ("generation %d: father of %d is %d\n", generation, i+j+1, father+1);
			//printf ("generation %d: mother of %d is %d\n", generation, i+j+1, mother+1);
			g_parents[0][generation][i+j] = mother;
			g_parents[1][generation][i+j] = father;
			cout<<" -- "<<i+j<<"  "<<g_parents[0][generation][i+j]<<" "<<g_parents[1][generation][i+j]<<endl;

			constructindividual (generation, mother, father, i+j);
		}
		i=i+j;
		//		cout<<"Begin to generate half sibling"<<endl;
		//		cout<<"total used"<<totalused<<endl;
		double ttmp=drand48();
		//		cout<<"ttmp: "<<ttmp<<endl;
		//		cout<<"rate: "<<g_rate_of_half_sibling<<endl;
		//
		//two cases: if father_NA.size>1, we can select one of them; if father_NA.size=1, it should be a different one with current husband.

		if(ttmp<g_rate_of_half_sibling & totalused< g_numofancestors & i < g_numofancestors &&(allow_hs_mate|| father_NA.size()>1||(father_NA.size()==1 &&father_NA.find(g_parents[1][generation-1][father])==father_NA.end()))&&(allow_sb_mate|| mother_NA.size()>1||(mother_NA.size()==1 &&mother_NA.find(g_parents[0][generation-1][father])==mother_NA.end())))
		{
			cout<<"making sibling"<<endl;
			tmp = drand48();
			mother = (int)(tmp * (double) g_numofancestors);
			while (used[mother]||(allow_hs_mate==0 && g_parents[1][generation-1][mother]==g_parents[1][generation-1][father]&&g_parents[1][generation-1][mother]!=-1)||(allow_sb_mate==0 && g_parents[0][generation-1][mother]==g_parents[0][generation-1][father]&&g_parents[0][generation-1][mother]!=-1)) {
				tmp = drand48();
				mother = (int)(tmp * (double) g_numofancestors);
			}
			used[mother]=1;
			totalused+=1;
			cout<<"totalused:: "<<totalused<<endl;			
			cout<<"allocate second wife"<<endl;
			father_NA.clear();
			mother_NA.clear();
			for(int k=0;k<g_numofancestors;k++)
			{
				grand_id=g_parents[1][generation-1][k];
				if(grand_id==-1)
				{
					if(used[k]==0)
					{
						father_NA[k]=1;
					}
				}
				else{
					map<unsigned int,unsigned int>::const_iterator mi = father_NA.find(grand_id);
					if(mi==father_NA.end())
					{
						if(used[k]==0)
						{
							father_NA[grand_id]=1;
						}
					}
				}
				grand_id=g_parents[0][generation-1][k];
				if(grand_id==-1)
				{
					if(used[k]==0)
					{
						mother_NA[k]=1;
					}
				}
				else{
					map<unsigned int,unsigned int>::const_iterator mi = mother_NA.find(grand_id);
					if(mi==mother_NA.end())
					{
						if(used[k]==0)
						{
							mother_NA[grand_id]=1;
						}
					}
				}


			}
			cout<<"count new av"<<endl;

			numofchildren = 0;
			sum = exp(-g_avgnumofchildren);
			tmp = drand48();
			if (tmp < sum)
				numofchildren = 0;
			else {
				j = 1;
				while (numofchildren == 0) {
					sum = sum + pow(g_avgnumofchildren,j)*exp(-g_avgnumofchildren)/factorial(j);
					if (tmp < sum) {
						numofchildren = j;
					}
					j++;
				}
			}
			/*			cout<<"*****  i   "<<i<<endl;
						cout<<"***** mother  "<<mother<<endl;
						cout<<"***** father  "<<father<<endl;
						cout<<"num of children: "<<numofchildren<<endl;
						cout<<"num of ancestors: "<<g_numofancestors<<endl;
						*/

			for (j = 0; i+j < g_numofancestors && j < numofchildren; j++) {
				g_used[generation][i+j] = 1;
				//printf ("generation %d: father of %d is %d\n", generation, i+j+1, father+1);
				//printf ("generation %d: mother of %d is %d\n", generation, i+j+1, mother+1);
				g_parents[0][generation][i+j] = mother;
				g_parents[1][generation][i+j] = father;
				cout<<" -- "<<i+j<<"  "<<g_parents[0][generation][i+j]<<" "<<g_parents[1][generation][i+j]<<endl;			
				constructindividual (generation, mother, father, i+j);
			}
			i=i+j;
		}
	}

	//  printf ("total used = %d i = %d\n", totalused, i);
	//printf ("The number of individuals in generation %d is %d\n", generation, i);
	g_actualnumindividuals = i;
	cout<<"totalused: "<<totalused<<endl;
	cout<<"actualnum: "<<g_actualnumindividuals<<endl;
	cout<<"g_totalac: "<<g_numofancestors<<endl;
	//	cout<<"finished building generation"<<endl; 

	free (used);



}




void printgenotypes(int generation) {

	int i,j;

	for (i = 0; i < 2*g_numofancestors; i++) {
		for (j = 0; j < g_numofregions; j++) {
			printf ("%d ", g_genotype[generation][i][j]);
		}
		printf ("\n");
	}

	return;
}




// if empty extant_ptrs, 
//    then take all the actual individuals
// otherwise
//    take only extant_ptrs
void make_sim_pedigree(vector<Indiv>& sim_pedigree, 
		vector<int>& extant_ptrs)
{

	makeUntypedPerson(&sim_pedigree, 0); // dummy person

	// data stored as:
	//   g_parents[0][generation][i] = mother;
	//   g_parents[1][generation][i] = father;
	unsigned int eran_gen = g_numofgenerations-1;

	// SHARED PARENTS... only add them once
	//   key:   g_parents mother/father index
	//   value: sim_pedigree person index
	map<unsigned int,unsigned int> sim_to_ped_index;


	// First make all the people,
	// in generation order.
	// Then set parent-child pointers (after all the people are created).


	// queue for making individuals
	queue<pair<unsigned int,unsigned int> > ancestors; // indexes from g_parents

	// queue for setting parents (indexes from g_parents)
	//    (child, (f, m))
	queue<pair<unsigned int, pair<unsigned int, unsigned int> > > parents;

	unsigned int size = extant_ptrs.size();
	if (size == 0)
		size = g_actualnumindividuals;
	for (unsigned int c = 0; c < size; c++)
	{
		int i = -1;
		if (extant_ptrs.size() != 0)
			i = extant_ptrs[c];  
		else
			i = c;


		unsigned int i_index = eran_gen*g_numofancestors + i;
		unsigned int child = makeTypedPerson(&sim_pedigree, 1);
		sim_pedigree[child].SetGender(MALE);
		sim_to_ped_index[i_index] = child;
		//		printf("  made sim %i, as index %i (%i)\n", i, child, i_index);

		// Add parent to queue
		int m = g_parents[0][eran_gen][i];
		int f = g_parents[1][eran_gen][i];

		unsigned int m_index = (eran_gen-1)*g_numofancestors + m;
		unsigned int f_index = (eran_gen-1)*g_numofancestors + f;

		if (m < 0)  // founder
			continue;


		// if parents don't already exist, then add them to the ped
		// otherwise find their ped indexes
		/*
		   bool alreadyExists = 0;
		   map<unsigned int,unsigned int>::const_iterator mi = sim_to_ped_index.find(m_index);
		   if (mi != sim_to_ped_index.end()){
		   alreadyExists = 1;
		   }

		   unsigned int p1 = 0;
		   unsigned int p2 = 0;
		   if (alreadyExists){
		   p1 = sim_to_ped_index[f_index];
		   p2 = sim_to_ped_index[m_index];
		   } else {
		   sim_to_ped_index[f_index] = p1;
		   sim_to_ped_index[m_index] = p2;

		// only need to push the parents once
		ancestors.push(pair<unsigned int,unsigned int>(f,eran_gen-1));
		ancestors.push(pair<unsigned int,unsigned int>(m,eran_gen-1));
		}
		*/

		bool alreadyExists_f = 0;
		bool alreadyExists_m = 0;
		map<unsigned int,unsigned int>::const_iterator mi_m = sim_to_ped_index.find(m_index);
		if (mi_m != sim_to_ped_index.end()){
			alreadyExists_m = 1;
		}

		map<unsigned int,unsigned int>::const_iterator mi_f = sim_to_ped_index.find(f_index);
		if (mi_f != sim_to_ped_index.end()){
			alreadyExists_f = 1;
		}


		unsigned int p1 = 0;
		unsigned int p2 = 0;

		if (alreadyExists_m){
			p2 = sim_to_ped_index[m_index];
		} else {
			sim_to_ped_index[m_index] = p2;
			ancestors.push(pair<unsigned int,unsigned int>(m,eran_gen-1));
		}


		if (alreadyExists_f){
			p1 = sim_to_ped_index[f_index];
		} else {
			sim_to_ped_index[f_index] = p1;
			ancestors.push(pair<unsigned int,unsigned int>(f,eran_gen-1));
		}


		pair<unsigned int, unsigned int> p_indexes(f_index,m_index);
		parents.push(pair<unsigned int,pair<unsigned int, unsigned int> >(i_index, p_indexes));
	}



	pair<int,int> a;
	unsigned int dsplit_gen = 2;
	unsigned int prev_gen = eran_gen-1;
	while (!ancestors.empty())
	{
		// person is already in pedigree if they are on this list
		a = ancestors.front();
		ancestors.pop();


		unsigned int eran_gen = a.second;
		if (prev_gen != eran_gen)
		{
			dsplit_gen++;
			prev_gen = eran_gen;
		}
		unsigned int child = makeUntypedPerson(&sim_pedigree, dsplit_gen);

		unsigned int i_index = (a.second)*g_numofancestors + a.first;
		sim_to_ped_index[i_index] = child;

		//printf("    made sim ancestor %i (%i), as index %i, gen %i\n", a.first, i_index, child, dsplit_gen);


		// Add parents to queue
		int m = g_parents[0][a.second][a.first];
		int f = g_parents[1][a.second][a.first];

		unsigned int m_index = (a.second-1)*g_numofancestors + m;
		unsigned int f_index = (a.second-1)*g_numofancestors + f;

		if (m < 0) // founder
			continue;


		// if parents don't already exist, then add them to the ped
		// otherwise find their ped indexes
		bool alreadyExists_f = 0;
		bool alreadyExists_m = 0;
		map<unsigned int,unsigned int>::const_iterator mi_m = sim_to_ped_index.find(m_index);
		if (mi_m != sim_to_ped_index.end()){
			alreadyExists_m = 1;
		}

		map<unsigned int,unsigned int>::const_iterator mi_f = sim_to_ped_index.find(f_index);
		if (mi_f != sim_to_ped_index.end()){
			alreadyExists_f = 1;
		}


		unsigned int p1 = 0;
		unsigned int p2 = 0;

		if (alreadyExists_m){
			p2 = sim_to_ped_index[m_index];
		} else {
			sim_to_ped_index[m_index] = p2;

			if (a.second-1 >= 0)
			{
				ancestors.push(pair<unsigned int,unsigned int>(m, a.second-1));
			}
		}


		if (alreadyExists_f){
			p1 = sim_to_ped_index[f_index];
		} else {
			sim_to_ped_index[f_index] = p1;

			// only need to push the parents once
			if (a.second-1 >= 0)
			{
				ancestors.push(pair<unsigned int,unsigned int>(f, a.second-1));
			}
		}

		pair<unsigned int, unsigned int> p_indexes(f_index,m_index);
		parents.push(pair<unsigned int,pair<unsigned int, unsigned int> >(i_index, p_indexes));
	}



	// Then set all the parent pointers
	pair<unsigned int, pair<unsigned int, unsigned int> > p;
	while (!parents.empty())
	{
		// person is already in pedigree if they are on this list
		p = parents.front();
		parents.pop();

		unsigned int child = sim_to_ped_index[p.first];

		unsigned int p1 = sim_to_ped_index[(p.second).first];
		unsigned int p2 = sim_to_ped_index[(p.second).second];

		setParents(&sim_pedigree, child, p1,p2);

		sim_pedigree[p1].SetGender(MALE);
		sim_pedigree[p2].SetGender(FEMALE);

		//printf("    setParents(%i,f%i,m%i) = (%i,%i,%i)\n", child, p1,p2, p.first, (p.second).first, (p.second).second);
	}


}






void observed_sharing_unphased(UndirectedGraph& sharing, int num_indiv)
{
	// Genotypes of extant generation are in
	//   g_genotypes[g_numofgenerations-1][chr][pos]
	//      where chr = 2 * indiv + j, with j in {0,1} 
	//      and pos in [0,...,g_numofregions]

	for (int i = 0; i < num_indiv; i++)
	{
		for (int j=i;  j < num_indiv; j++)
		{
			printf("  %i,%i) ", i+1,j+1);

			// compute sharing for this pair of people
			vector<int> max_share;

			int length = 0;
			for (int pos = 0;  pos < g_numofregions; pos++)
			{
				bool isShared = 0;
				int a_i0 = g_genotype[g_numofgenerations-1][2*i][pos];
				int a_i1 = g_genotype[g_numofgenerations-1][2*i+1][pos];
				int a_j0 = g_genotype[g_numofgenerations-1][2*j][pos];
				int a_j1 = g_genotype[g_numofgenerations-1][2*j+1][pos];

				if (i == j){
					if(a_i0 == a_i1)
						isShared = 1;
				} else {
					if (a_i0 == a_j0)
						isShared = 1;
					else if (a_i0 == a_j1)
						isShared = 1;
					else if (a_i1 == a_j0)
						isShared = 1;
					else if (a_i1 == a_j1)
						isShared = 1;
				}

				//printf(" {%i,%i}=%i={%i,%i}", a_i0, a_i1, isShared, a_j0, a_j1);
				if (isShared) {
					length++;
				} else {
					if (length > 0)
						max_share.push_back(length);
					length = 0;
				}

			} // end for pos
			if (length > 0)
				max_share.push_back(length);
			//printf("\n");

			// average length of max-length sharing regions
			double avg_length = 0.0;
			printf("Avg Share: ");
			for (unsigned int k = 0;  k < max_share.size();  k++)
			{
				//printf(" %i +", max_share[k]);
				avg_length += max_share[k];
			}
			if (max_share.size() > 0)
				avg_length /= max_share.size();
			printf(" = %f\n", avg_length);
			sharing.SetEdge(i+1,j+1, avg_length);
		}
	}
}





void observed_sharing(UndirectedGraph& sharing, vector<vector<double> >& expected_sharing, int num_indiv)
{
	// ASSUME genotypes are PHASED

	// Genotypes of extant generation are in
	//   g_genotypes[g_numofgenerations-1][chr][pos]
	//      where chr = 2 * indiv + j, with j in {0,1} 
	//      and pos in [0,...,g_numofregions]

	vector<double> row(num_indiv+1, 0.0);
	expected_sharing.push_back(row);



	double avg_avg_length = 0.0;
	int avg_avg_number = 0;
	double avg_sq_diff = 0.0;
	for (int i = 0; i < num_indiv; i++)
	{
		vector<double> row(num_indiv+1, 0.0);
		expected_sharing.push_back(row);

		for (int j=i;  j < num_indiv; j++)
		{
			//printf("  %i,%i) ", i+1,j+1);

			// compute sharing for this pair of people
			vector<int> max_share;

			int length1 = 0;
			int length2 = 0;
			for (int pos = 0;  pos < g_actualnumofregions; pos++)
			{
				// indicates orientation of sharing (consider both pairings of haps)
				int isShared = 0;
				int a_i0 = g_genotype[g_numofgenerations-1][2*i][pos];
				int a_i1 = g_genotype[g_numofgenerations-1][2*i+1][pos];
				int a_j0 = g_genotype[g_numofgenerations-1][2*j][pos];
				int a_j1 = g_genotype[g_numofgenerations-1][2*j+1][pos];

				if (i == j){
					if(a_i0 == a_i1)
						isShared = 1;
				} else {
					if (a_i0 == a_j0 || a_i1 == a_j1)
						isShared = 1;
					else if (a_i0 == a_j1 || a_i1 == a_j0)
						isShared = 2;
				}

				//printf(" {%i,%i}=%i={%i,%i}", a_i0, a_i1, isShared, a_j0, a_j1);
				if (isShared == 1) {
					length1++;
				} else {
					if (length1 > 0)
						max_share.push_back(length1);
					length1 = 0;
				}
				if (isShared == 2) {
					length2++;
				} else {
					if (length2 > 0)
						max_share.push_back(length2);
					length2 = 0;
				}

			} // end for pos
			if (length1 > 0)
				max_share.push_back(length1);
			if (length2 > 0)
				max_share.push_back(length2);
			//printf("\n");

			// average length of max-length sharing regions
			double avg_length = 0.0;
			//printf("Avg Share: ");
			for (unsigned int k = 0;  k < max_share.size();  k++)
			{
				//printf(" %i +", max_share[k]);
				avg_length += max_share[k];
				avg_avg_length += max_share[k];
				avg_avg_number++;
				avg_sq_diff += pow(max_share[k]-2.690608, 2);
			}
			if (max_share.size() > 0)
				avg_length /= max_share.size();
			//printf(" = %f\n", avg_length);
			sharing.SetEdge(i+1,j+1, avg_length);


			expected_sharing[i+1][j+1] = avg_length / g_numofregions;
		}
	}



	//printf("avg of all lengths %f = %f/%i\n", (avg_avg_length / (double) avg_avg_number), avg_avg_length, avg_avg_number);
	//printf("var of all lengths %f\n", (avg_sq_diff / (double) avg_avg_number));
	//printf("\n");
}




// void populate_alleles(vector<Indiv>* pedigree, int eran_idx, int ped_idx)
// {
//   for (int pos = 0;  pos < g_numofregions; pos++)
//     {
//       (*pedigree)[ped_idx].distinct_alleles.push_back(set<int>());

//       int a0 = g_genotype[g_numofgenerations-1][2*eran_idx][pos];
//       int a1 = g_genotype[g_numofgenerations-1][2*eran_idx+1][pos];

//       (*pedigree)[ped_idx].distinct_alleles[pos].insert(a0);
//       (*pedigree)[ped_idx].distinct_alleles[pos].insert(a1);
//     }   
// }







double computePairWeights(vector<Indiv>& pedigree, 
		UndirectedGraph& observed_sharing,
		UndirectedGraph& w, // <-- weights to compute
		double lambda,
		unsigned int num_samples,
		unsigned int num_recent_gen,
		unsigned int oldest_gen, 
		int k, int l)
{
	double log_test = 0.0;

	set<int> dk;
	pedigree[k].GetDescentList(dk);

	set<int> dl;
	pedigree[l].GetDescentList(dl);



	for (unsigned int i = 1;  i <= num_recent_gen;  i++)
	{
		set<int>::iterator found = dk.find(i);
		if (found == dk.end())
			continue;

		for (unsigned int j = 1;  j <= num_recent_gen;  j++)
		{
			set<int>::iterator found = dl.find(j);
			if (found == dl.end())
				continue;

			//printf("\n (%i,%i) IBD\n-----------\n", i,j);


			//paths.clear();
			resetColor(pedigree);
			colorPedigree(pedigree, i,j, 999);

			vector<unsigned int> all_paths;
			vector<unsigned int> paths;
			//unsigned int m = getInheritance(paths, pedigree, i, j, 999);
			unsigned int m = getInheritance2(paths, pedigree, i, j, 999);

			// sample some of the paths
			// 	  unsigned int size = (unsigned int) floor(all_paths.size() * fraction);
			// 	  if (all_paths.size() < 1000)
			// 	    size = all_paths.size();
			// 	  else
			// 	    size = 1000;
			// 	  for (unsigned int k = 0; k < size;  k++)
			// 	    {
			// 	      paths.push_back(all_paths[k]);
			// 	    }


			//  	  if (paths.size() > 0)
			//  	    {
			//  	      printf("New Alg Paths: ");
			//  	      for (unsigned int p = 0;  p < paths.size(); p++)
			//  		printf("%i, ", paths[p]);
			//  	      printf("\n");
			//  	    }

			if (m > 0)
			{
#ifdef MODELS
				if (oldest_gen == 3)
					printf("  got inheritance with %i meioses and %i paths\n", m, paths.size());
				fflush(stdout);
#endif



				//printf("  num meioses: %i, num inheritance paths: %i\n", m, paths.size());

				// 	      vector<unsigned int> paths2;
				// 	      unsigned int m2 = getInheritance2(paths2, pedigree, i,j, 999);
				// 	      if (paths2.size() > 0)
				// 		{
				// 		  printf("Old Alg Paths: ");
				// 		  for (unsigned int p = 0;  p < paths2.size(); p++)
				// 		    printf("%i, ", paths2[p]);
				// 		  printf("\n");
				// 		}

				// 	      if (m != m2)
				// 		{
				// 		  printf("FATAL ERRPOR: incorrect # meiosis %i <> %i\n", m, m2);
				// 		  exit(-324);
				// 		}


				vector<int> sum(paths.size(), 0);
				vector<vector<int> > rates(paths.size(), vector<int>(paths.size()+1, 0));
				getTransitionRates(rates, sum, paths, m);

#ifdef MODELS
				if (oldest_gen == 3)
					printf("  got transition rates with %i rates\n", rates.size());
				fflush(stdout);
#endif


				//map<unsigned int, double> prob;
				vector<double> length;
				samplePoissonRandomWalk(length, paths, rates, lambda, m, num_samples);

				// compute w[i][j]
				//printf("  observed_sharing.GetEdge(%i,%i)=%f\n", i,j, observed_sharing.GetEdge(i,j));
				double weight = testCoefficient(length, observed_sharing.GetEdge(i,j));

#ifdef MODELS
				printf("  weight[%i][%i] = %f\n", i,j,weight);
				fflush(stdout);
#endif
				w.SetEdge(i,j, weight);
				log_test += weight / (dk.size() * dl.size());
			}
			else
			{
				w.SetEdge(i,j, default_test);
				log_test += default_test / (dk.size() * dl.size());
			}
		}
	}
	return log_test;
}








double weight = 1.0;
double ifSiblingsCoefficient(double observed_sharing, 
		int num_generation, 
		double lambda)

{
	//double x=2;
	//double p_same=1/pow(x,  2*num_generation);
	double expectLength=2/(2*num_generation*lambda);
	double variance=expectLength*expectLength/2;
	//cout<<"shared length"<<(observed_sharing)<<" "<<expectLength<<" "<<variance<<endl;
	printf("shared length %f, expectLength %f, variance %f\n", observed_sharing, expectLength, variance);
	double length_test = pow((observed_sharing-expectLength), 2)/variance;
	printf("shuai length_test %f\n", length_test);


	return length_test; 
}

vector<int>* temp_v1=new vector<int>();
vector<int>* temp_v2=new vector<int>();
	double
computePairWeight(UndirectedGraph& observed_sharing,
		int index1, int index2, int generation_no, double lambda)
{
	//printf("Shuai's computePairWeight(%i,%i,%i)\n",index1,index2,generation_no);


	if ((signed) (*mExtantChildren).size() <= generation_no-1)
		printf("WARNING: out of bound mExtandChildren gen. no. %i\n", generation_no-1);
	if ( (signed) (*((*mExtantChildren)[generation_no-1])).size() <= index1)
		printf("WARNING: out of bound mExtandChildren index1 %i (gen. no. %i)\n", index1, generation_no-1);

	vector<int>* extant1=(*((*mExtantChildren)[generation_no-1]))[index1];
	vector<int>* extant2=(*((*mExtantChildren)[generation_no-1]))[index2];
	if(extant1==extant2)
	{
		return 0;
	}
	int size1=extant1->size(); 
	int size2=extant2->size();
	int i1=0;
	int i2=0;
	double result=0;
	double counter=0;
	temp_v1->clear();
	temp_v2->clear();
	while(i1!=size1 || i2!=size2)
	{
		if(i1==size1)
		{
			temp_v2->push_back((*extant2)[i2]);
			i2++;
			continue;
		}
		if(i2==size2)
		{
			temp_v1->push_back((*extant1)[i1]);
			i1++;
			continue;
		}
		if((*extant1)[i1]<(*extant2)[i2])
		{
			for(int j=i2; j<size2; j++)
			{		 
				result+= ifSiblingsCoefficient(observed_sharing.GetEdge((*extant1)[i1],
							(*extant2)[j]),
						generation_no, lambda);
				counter+=1;
			}
			temp_v1->push_back((*extant1)[i1]);
			i1++;
		}
		else if((*extant1)[i1]==(*extant2)[i2])
		{
			i1++;
			i2++;

		}
		else
		{
			for(int i=i1; i<size1; i++)
			{		 
				result+= ifSiblingsCoefficient(observed_sharing.GetEdge((*extant1)[i],
							(*extant2)[i2]), 
						generation_no, lambda);
				counter+=1;
			}//for
			temp_v2->push_back((*extant2)[i2]);     
			i2++;
		}//else
	}

	int s1=temp_v1->size();
	int s2=temp_v2->size();
	result=0;
	counter=0;
	for(int i=0; i<s1; i++)
	{
		for(int j=0; j<s2; j++)
		{
			printf("[%i, %i] ", (*temp_v1)[i], (*temp_v2)[j]);
			result+=ifSiblingsCoefficient(observed_sharing.GetEdge((*temp_v1)[i],
						(*temp_v2)[j]),
					generation_no, lambda);
			counter++;
		}
	}
	//cout<<"+!++!+!+!+!+!+!+"<<counter<<endl;
	//printf("returning %f/%f = %f\n", result, counter, result/counter);
	if(counter==0)
	{
		return 0;
	}
	return (result/counter);


}







// Note that pedigree individuals are numbered starting at 1
// (i.e. there should be a dummy individual at zero
void makeEdges(vector<Indiv>& pedigree, 
		UndirectedGraph& graph,
		UndirectedGraph& observed_sharing,
		double lambda,
		unsigned int num_samples,
		unsigned int num_extant,
		// num of indiv in most recently constructed generation
		unsigned int num_oldest_gen, 
		unsigned int oldest_gen,
		double threshold,
		int g)
{

	// For existing pedigree, (i.e. no g'th generation relationships)
	// estimate the sharing
#ifdef MODELS
	printf("MODEL B (existing relationships)\n");
#endif
	UndirectedGraph w(num_extant+1);
	//computePairWeights(pedigree, observed_sharing,
	//		     w, // <-- weights to compute
	//	     lambda, num_samples, num_extant, oldest_gen);



	// For all pairs of people, k,l, in the oldest generation, 
	// compute test weights as if k,l are siblings
	// Added edges to graph when the test passes the threshold
	int first_of_last_gen = pedigree.size()-num_oldest_gen;
	int last_of_last_gen = pedigree.size()-1;

	int parent1 = makeUntypedPerson(&pedigree,oldest_gen+1);
	int parent2 = makeUntypedPerson(&pedigree,oldest_gen+1);
	for (int k = first_of_last_gen;  k <= last_of_last_gen;  k++)
	{
		set<int> dk;
		pedigree[k].GetDescentList(dk);
		for (int l = k+1;  l <= last_of_last_gen;  l++)
		{
			set<int> dl;
			pedigree[l].GetDescentList(dl);

			// make k & l siblings
#ifdef MODELS
			printf("MODEL A, gen %i, siblings %i,%i\n", oldest_gen, k,l);
#endif
			setParents(&pedigree, k, parent1,parent2);
			setParents(&pedigree, l, parent1,parent2);

			//printPedigree(pedigree);

			UndirectedGraph w_kl(num_extant+1);
			double log_test = computePairWeights(pedigree, observed_sharing,
					w_kl, // <-- weights to compute
					lambda, num_samples, 
					num_extant, oldest_gen,
					k, l);



#ifdef MODELS
			double log_test2=computePairWeight(observed_sharing, 
					k-first_of_last_gen, 
					l-first_of_last_gen, 
					g, lambda);
			printf("   log_test = %f, shuai's %f\n", log_test,log_test2);
#endif
			if (log_test < threshold)
			{
#ifdef MODELS
				printf("     added edge %i,%i, value %f + %f\n", k,l, log_test-threshold, threshold);
				fflush(stdout);
#endif
				//graph.SetEdge(k,l,log_test-threshold);
				graph.SetEdge(k,l,threshold-log_test);
			}


			// make k & l NOT siblings
			unsetParents(&pedigree, k, parent1,parent2);
			unsetParents(&pedigree, l, parent2,parent2);
		}
	}
	// delete the extra parents
	pedigree.pop_back();
	pedigree.pop_back();

}



unsigned int countMatchEdges(vector<Indiv>& sim_pedigree, 
		vector<Indiv>& pedigree,
		vector<unsigned int>& generations, // for ped.
		map<unsigned int, unsigned int>& matching)
{
	// num matched edges
	unsigned int match_edges = 0;

	//printf("Count matching edges:\n");

	// For each generation, compute the match count
	unsigned int est_first_in_gen = generations[1]; // estimated ped
	for (unsigned int g = 2;  g < generations.size();  g++) // generations[g] is the person to not include in this gen
	{

		// for each edge in pedigree, see whether there 
		// is a matching edge in sim_pedgiree
		for (unsigned int i = est_first_in_gen; i < generations[g]; i++)
		{
			unsigned int j = matching[i];

			// for each child we have an edge
			vector<int> children_i;
			pedigree[i].GetChildList(children_i);
			for (unsigned int ci = 0;  ci < children_i.size();  ci++)
			{
				unsigned int child_i = children_i[ci];

				//printf("ped edge (%i,%i) -> ", i, child_i);

				if (sim_pedigree[j].HasChild(matching[child_i]))
				{
					match_edges++;
					//printf("(%i,%i)", j, matching[child_i]);
				}
				//printf("\n");

			} // end for every child ci
		} // end for every parent i

		est_first_in_gen = generations[g]; // estimated ped
	} // end for each generation

	return match_edges;
}





unsigned int randomMatch(vector<Indiv>& sim_pedigree, 
		vector<Indiv>& pedigree,
		vector<unsigned int>& generations, // for ped.
		int num_extant)
{

	map<unsigned int, unsigned int> matching; // pedigree indiv, sim_ped indiv
	for (int i = 0;  i < num_extant; i++)
	{
		matching[i] = i;
	}


	// In the estimated pedigree, assign gender randomly.
	// Init gender to UNKNOWN
	for (unsigned int i = 0;  i < pedigree.size(); i++)
	{
		pedigree[i].gender = UNKNOWN;
	}


	// Each generation is randomly matched with probability proportional
	// to the number of same-labeled individuals in the descent set.


	// matching prob. for prev generation
	map<unsigned int, unsigned int> prev_idx_i;
	map<unsigned int, unsigned int> prev_idx_j;
	vector<vector<double> > first_gen_match_prob(num_extant, vector<double>(num_extant, 0.0));
	// init to first generation (where match forced to same label)
	for (int i = 0;  i < num_extant; i++)
	{
		first_gen_match_prob[i][i] = 1;
		prev_idx_i[i] = i;
		prev_idx_j[i] = i;
	}
	vector<vector<double> >* prev_gen_match_prob;
	prev_gen_match_prob = &first_gen_match_prob;



	// For each generation, compute the matching prob
	unsigned int sim_first_in_gen = generations[1]; // sim ped
	unsigned int est_first_in_gen = generations[1]; // estimated ped
	for (unsigned int g = 2;  g < generations.size();  g++) // generations[g] is the person to not include in this gen
	{
		//printf("Generation %i: persons [%i, %i)\n", g, est_first_in_gen, generations[g]);

		//if (est_first_in_gen > pedigree.size())
		//break;


		if (est_first_in_gen != generations[g-1] || 
				pedigree[est_first_in_gen].generation != sim_pedigree[sim_first_in_gen].generation){
			printf("ERROR: mismatch in generation for estimated pedigree\n");
			printf("   %i != %i   ||  %i != %i\n", est_first_in_gen, generations[g-1], 
					pedigree[est_first_in_gen].generation, sim_pedigree[sim_first_in_gen].generation);
			exit(-34);
		}
		unsigned int sim_next_gen = 0;


		map<unsigned int, unsigned int> idx_i;
		map<unsigned int, unsigned int> idx_j;

		map<unsigned int, bool> used_j;  // key j, value 0,1 for used or not


		//vector<vector<double> >* match_prob = new vector<vector<double> >(0, vector<double>(0, 0.0));
		vector<vector<double> > match_prob;
		for (unsigned int i = est_first_in_gen; i < generations[g]; i++)
		{
			vector<double> counts = vector<double>(0,0.0);
			//idx_i[i] = match_prob->size();
			idx_i[i] = match_prob.size();

			vector<int> D_i;
			pedigree[i].GetDescentList(D_i);

			unsigned int sum_male = 0;
			unsigned int sum_female = 0;

			unsigned int j = sim_first_in_gen;
			while(j < sim_pedigree.size() &&
					sim_pedigree[j].generation == (signed) g)
			{
				int common_descendant_count = 0;
				for (unsigned int d = 0;  d < D_i.size();  d++)
				{
					if (sim_pedigree[j].HasDescendant(D_i[d]))
						common_descendant_count++;
				}
				if (used_j[j] != 1)
				{
					if (sim_pedigree[j].gender == MALE)
						sum_male += common_descendant_count;
					else
						sum_female += common_descendant_count;
				}

				//if (common_descendant_count > 0)
				//printf("gen %i, est %i, sim %i have %i shared\n",g, i, j, common_descendant_count);

				if (i == est_first_in_gen)
					idx_j[j] = counts.size();

				counts.push_back(common_descendant_count);

				j++;
			} // end for each sim_pedigree person in gen
			sim_next_gen = j;
			//(*match_prob).push_back(counts);
			match_prob.push_back(counts);

			//printf("gen %i, est %i, sim %i have %i male, %i female\n",g, i, j, sum_male, sum_female);


			// Normalize
			j = sim_first_in_gen;
			while(j < sim_pedigree.size() &&
					sim_pedigree[j].generation == (signed) g)
			{
				if (sum_male+sum_female > 0)
				{
					if (sim_pedigree[j].gender == MALE)
						match_prob[idx_i[i]][idx_j[j]] /= (sum_male);
					else
						match_prob[idx_i[i]][idx_j[j]] /= (sum_female);
				}

				j++;
			} // end for each sim_pedigree person in gen


			// draw the gender for this person and their co-parent
			double u = drand48();
			if (pedigree[i].gender == UNKNOWN)
			{
				Gender other_gender = UNKNOWN;
				if (u < 0.5) {
					pedigree[i].gender = MALE;
					other_gender = FEMALE;
				} else {
					pedigree[i].gender = FEMALE;
					other_gender = MALE;
				}
				// set other parent
				unsigned int child = pedigree[i].children[0];
				unsigned int other_parent = pedigree[child].parents[0];
				if (other_parent == i)
					other_parent = pedigree[child].parents[1];
				pedigree[other_parent].gender = other_gender;

				u = drand48();
			}

			// draw the matching for this person
			j = sim_first_in_gen;
			while(j < sim_pedigree.size() &&
					sim_pedigree[j].generation == (signed) g)
			{
				if (sim_pedigree[j].gender == pedigree[i].gender &&
						used_j[j] != 1)
				{
					//u -= (*match_prob)[idx_i[i]][idx_j[j]];
					u -= match_prob[idx_i[i]][idx_j[j]];
					if (u <= 0.0)
					{
						matching[i] = j;
						used_j[j] = 1;
						break;
					}
				}

				j++;
			} // end for each sim_pedigree person in gen

			//printf("  match ped %i to sim %i\n", i, matching[i]);

		} // for each i in pedigree


		sim_first_in_gen = sim_next_gen; // sim ped
		est_first_in_gen = generations[g]; // estimated ped


	} // end for each generation






	return countMatchEdges(sim_pedigree, 
			pedigree,
			generations, // for ped.
			matching);
}



// Assume that TYPED individuals have same labels in both
// pedigrees.
double editDistance(vector<Indiv>& sim_pedigree, 
		vector<Indiv>& pedigree,
		vector<unsigned int>& generations, // for ped.
		int num_extant)
{
	//for (unsigned int g = 0;  g < generations.size();  g++)
	//  printf("generations[%i] = %i\n", g, generations[g]);


	// In the estimated pedigree, assign gender randomly.

	// Each generation is randomly matched with probability proportional
	// to the number of same-labeled individuals in the descent set.

	double total_match_exp = 0.0;

	// matching prob. for prev generation
	map<unsigned int, unsigned int> prev_idx_i;
	map<unsigned int, unsigned int> prev_idx_j;
	vector<vector<double> > first_gen_match_prob(num_extant, vector<double>(num_extant, 0.0));
	// init to first generation (where match forced to same label)
	for (int i = 0;  i < num_extant; i++)
	{
		first_gen_match_prob[i][i] = 1;
		prev_idx_i[i] = i;
		prev_idx_j[i] = i;
	}
	//vector<vector<double> >* prev_gen_match_prob;
	//prev_gen_match_prob = &first_gen_match_prob;
	vector<vector<double> > prev_gen_match_prob;
	prev_gen_match_prob = first_gen_match_prob;



	// For each generation, compute the matching prob
	unsigned int sim_first_in_gen = generations[1]; // sim ped
	unsigned int est_first_in_gen = generations[1]; // estimated ped
	for (unsigned int g = 2;  g < generations.size();  g++) // generations[g] is the person to not include in this gen
	{
		//printf("ITERATION %i: persons [%i, %i)\n", g, est_first_in_gen, generations[g]);

		//if (est_first_in_gen > pedigree.size())
		//break;

		if (est_first_in_gen != generations[g-1] || 
				pedigree[est_first_in_gen].generation != sim_pedigree[sim_first_in_gen].generation){
			printf("ERROR: mismatch in generation for estimated pedigree\n");
			printf("   %i != %i   ||  %i != %i\n", est_first_in_gen, generations[g-1], 
					pedigree[est_first_in_gen].generation, sim_pedigree[sim_first_in_gen].generation);
			exit(-34);
		}
		unsigned int sim_next_gen = 0;


		map<unsigned int, unsigned int> idx_i;
		map<unsigned int, unsigned int> idx_j;


		//vector<vector<double> >* match_prob = new vector<vector<double> >(0, vector<double>(0, 0.0));
		vector<vector<double> > match_prob;
		for (unsigned int i = est_first_in_gen; i < generations[g]; i++)
		{
			vector<double> counts = vector<double>(0,0.0);
			idx_i[i] = match_prob.size();

			vector<int> D_i;
			pedigree[i].GetDescentList(D_i);

			unsigned int sum_male = 0;
			unsigned int sum_female = 0;

			unsigned int j = sim_first_in_gen;
			//printf("Generations testing %i <> %i?\n", sim_pedigree[j].generation, g);
			while(j < sim_pedigree.size() &&
					sim_pedigree[j].generation == (signed) g)
			{

				int common_descendant_count = 0;
				for (unsigned int d = 0;  d < D_i.size();  d++)
				{
					if (sim_pedigree[j].HasDescendant(D_i[d]))
						common_descendant_count++;
				}
				if (sim_pedigree[j].gender == MALE)
					sum_male += common_descendant_count;
				else
					sum_female += common_descendant_count;

				//if (common_descendant_count > 0)
				//printf("gen %i, est %i, sim %i have %i shared\n",g, i, j, common_descendant_count);

				if (i == est_first_in_gen)
					idx_j[j] = counts.size();

				counts.push_back(common_descendant_count);

				j++;
			} // end for each sim_pedigree person in gen
			sim_next_gen = j;
			//(*match_prob).push_back(counts);
			match_prob.push_back(counts);

			//printf("gen %i, est %i, sim %i have %i male, %i female\n",g, i, j, sum_male, sum_female);


			// Normalize
			j = sim_first_in_gen;
			while(j < sim_pedigree.size() &&
					sim_pedigree[j].generation == (signed) g)
			{
				if (sum_male+sum_female > 0)
				{
					if (sim_pedigree[j].gender == MALE)
						match_prob[idx_i[i]][idx_j[j]] /= 2*(sum_male);
					//(*match_prob)[idx_i[i]][idx_j[j]] /= 2*(sum_male);
					else
						match_prob[idx_i[i]][idx_j[j]] /= 2*(sum_female);
					//(*match_prob)[idx_i[i]][idx_j[j]] /= 2*(sum_female);
				}

				j++;
			} // end for each sim_pedigree person in gen

		} // for each i in pedigree



		// for each pair of edges, compute the probability of matching
		double match_exp = 0.0;
		unsigned int num_edges = 0;
		for (unsigned int i = est_first_in_gen; i < generations[g]; i++)
		{
			//printf("EDGES for %i\n", i);

			// for each child we have an edge
			vector<int> children_i;
			pedigree[i].GetChildList(children_i);
			for (unsigned int ci = 0;  ci < children_i.size();  ci++)
			{
				num_edges++;
				unsigned int child_i = children_i[ci];

				// for each parent j in sim_pedigree
				unsigned int j = sim_first_in_gen;
				while(j < sim_pedigree.size() && 
						sim_pedigree[j].generation == (signed) g)
				{
					// for each child of j, we have another edge
					vector<int> children_j;
					sim_pedigree[j].GetChildList(children_j);

					for (unsigned int cj = 0;  cj < children_j.size();  cj++)
					{
						unsigned int child_j = children_j[cj];

						// Have two matched edges (i,child_i) and (j,child_j)
						// Expected match probability
						//double edge_prob = (*match_prob)[idx_i[i]][idx_j[j]] 
						//* (*prev_gen_match_prob)[prev_idx_i[child_i]][prev_idx_j[child_j]];
						double edge_prob = match_prob[idx_i[i]][idx_j[j]] 
							* prev_gen_match_prob[prev_idx_i[child_i]][prev_idx_j[child_j]];
						//if (edge_prob > 0)
						//printf("   edge match (%i,%i) =m= (%i,%i) hs prob %f\n", i,ci, j, cj, edge_prob);
						match_exp += edge_prob;
					}

					j++;
				} // end for every parent j

			} // end for every child ci
		} // end for every parent i
		//printf("  gen %i, has match_exp %f out of %i estimated edges\n", g, match_exp, num_edges);
		total_match_exp += match_exp;


		prev_gen_match_prob = match_prob;

		prev_idx_i.clear();
		prev_idx_i.insert(idx_i.begin(), idx_i.end());
		prev_idx_j.clear();
		prev_idx_j.insert(idx_j.begin(), idx_j.end());




		sim_first_in_gen = sim_next_gen; // sim ped
		est_first_in_gen = generations[g]; // estimated ped

	} // end of generation


	unsigned int max_matching = 0;
	for (int c = 0;  c < 100; c++)
	{
		unsigned int matching = randomMatch(sim_pedigree, 
				pedigree,
				generations, // for ped.
				num_extant);
		//printf("   match edges %i\n", matching);
		if (matching > max_matching)
			max_matching = matching;
	}

	//printf("Found Matching of %i edges <> exp matching of %f edges\n", max_matching, total_match_exp);
	//return total_match_exp;
	return max_matching;

}



int numMeioses(vector<Indiv>& pedigree)
{
	int meioses = 0;
	for (unsigned int i = 0;  i < pedigree.size();  i++)
	{
		if (pedigree[i].parents.size() > 0)
			meioses+=2;
	}
	return meioses;
}





double kinshipEmpiricalDistance(vector<Indiv>& pedigree, 
		vector<vector<double> >& expected_sharing, 
		vector<unsigned int>& generations, // for ped.
		int num_extant)
{
	// compare the kinship matrices of the two pedigrees
	vector<vector<double> > kinship_ped;
	kinship(pedigree, kinship_ped);

	// sum the absolute difference between the labeled people
	// in the two matrices
	double sum = 0;
	for (unsigned int i = 1;  i <= (unsigned) num_extant;  i++)
	{
		for (unsigned int j = i; j <= (unsigned) num_extant; j++)
		{
			//printf("%i,%i: |%f - %f|\n", i,j, kinship_est[i][j], kinship_sim[i][j]);
			double diff = kinship_ped[i][j] - expected_sharing[i][j];
			if (diff < 0)
				diff *= -1;
			sum += diff;
		}
	}

	return sum;
}


void perturbPedigree(vector<Indiv>& pedigree,
		vector<Indiv>& new_pedigree,
		vector<unsigned int>& generations,
		vector<unsigned int>& new_generations,
		double change)
{
	// make person for every one in pedigree
	vector<Indiv> tmp_pedigree;

	for (unsigned int j = 0;  j < pedigree.size(); j++)
	{
		int g = pedigree[j].generation;
		if (g == 1)
			makeTypedPerson(&tmp_pedigree,g);
		else
			makeUntypedPerson(&tmp_pedigree,g);
	}


	// randomly change some child-parent edges
	int num_changes = 0;
	for (unsigned int i = 1;  i < pedigree.size();  i++)
	{
		if (pedigree[i].parents.size() > 0)
		{
			double u = drand48();
			if (u < change)
			{
				num_changes++;
				// pick new monogomous parents
				unsigned int g = pedigree[i].generation;
				if (g+1 >= generations.size()){
			//		printPedigree(pedigree);
					printf("generations: ");
					for (unsigned int gi = 0;  gi < generations.size();  gi++)
						printf("%i, ", generations[gi]);
					printf("\npedigree person %i has %i parents, gen %i, while gensize %i\n", i, (int) pedigree[i].parents.size(), g, (int) generations.size());
					exit(-5346543);
				}
				int g_start = generations[g]; // generation-1+1
				int g_end = generations[g+1]-1;
				unsigned int d = g_end - g_start + 1;
				u = drand48();
				unsigned int idx = g_start + (int) floor(u*10000) % (int) d;
				if (pedigree[idx].children.size() <= 0){
					printf("ERROR: no children for selected parent\n");
					exit(-534);
				}
				unsigned int child = pedigree[idx].children[0];
				unsigned int other_parent = pedigree[child].parents[0];
				if (other_parent == idx)
					other_parent = pedigree[child].parents[1];
				setParents(&tmp_pedigree, i, idx, other_parent);
			}
			else
			{
				// add person without changing parents
				setParents(&tmp_pedigree, i, pedigree[i].parents[0], pedigree[i].parents[1]);
			}

		}
	}

	printf("Number of perturbations change: %i\n", num_changes);
	//printPedigree(tmp_pedigree);


	// copy into new_pedigree such that everyone has children (except leafs)
	map<unsigned int,unsigned int> tmp_to_new;
	queue<unsigned int> bfs_list;
	vector<bool> was_pushed(tmp_pedigree.size(), 0);
	for (unsigned int i = 1; i < generations[1];  i++)
	{
		bfs_list.push(i);
		was_pushed[i] = 1;
	}
	makeTypedPerson(&new_pedigree,0);
	new_generations.push_back(1);


	unsigned int prev_g = 1;
	while (!bfs_list.empty())
	{
		unsigned int j = bfs_list.front();
		bfs_list.pop();

		unsigned int g = tmp_pedigree[j].generation;
		if (g == 1){
			unsigned int c = makeTypedPerson(&new_pedigree,g);
			tmp_to_new[j] = c;
			if (g != prev_g){
				new_generations.push_back(new_pedigree.size()-1);
				prev_g = g;
			}
		} else {
			unsigned int c = makeUntypedPerson(&new_pedigree,g);
			tmp_to_new[j] = c;
			if (g != prev_g){
				new_generations.push_back(new_pedigree.size()-1);
				prev_g = g;
			}
		}

		for (unsigned int p = 0;  p < tmp_pedigree[j].parents.size();  p++)
		{
			unsigned int p_idx = tmp_pedigree[j].parents[p];
			if (was_pushed[p_idx] == 0){
				bfs_list.push(p_idx);
				was_pushed[p_idx] = 1;
			}
		}
	}

	// make parent-child edges
	for (unsigned int i = 1;  i < tmp_pedigree.size();  i++)
	{

		if (tmp_pedigree[i].parents.size() > 0)
		{
			unsigned int c = tmp_to_new[i];
			unsigned int p0 = tmp_to_new[tmp_pedigree[i].parents[0]];
			unsigned int p1 = tmp_to_new[tmp_pedigree[i].parents[1]];
			setParents(&new_pedigree, c, p0, p1);
		}
	}
	new_generations.push_back(new_pedigree.size());

	//printPedigree(new_pedigree);
	//fflush(stdout);


}




// Assume that TYPED individuals have same labels in both
// pedigrees.
double kinshipDistance(vector<Indiv>& sim_pedigree, 
		vector<Indiv>& pedigree,
		vector<unsigned int>& generations, // for ped.
		int num_extant)
{
	// compare the kinship matrices of the two pedigrees
	vector<vector<double> > kinship_sim;
	kinship(sim_pedigree, kinship_sim);

	vector<vector<double> > kinship_est;
	kinship(pedigree, kinship_est);

	// sum the absolute difference between the labeled people
	// in the two matrices
	double sum = 0;
	for (unsigned int i = 1;  i <= (unsigned) num_extant;  i++)
	{
		for (unsigned int j = i; j <= (unsigned) num_extant; j++)
		{
			//printf("%i,%i: |%f - %f|\n", i,j, kinship_est[i][j], kinship_sim[i][j]);
			double diff = kinship_est[i][j] - kinship_sim[i][j];
			if (diff < 0)
				diff *= -1;
			sum += diff;
		}
	}

	return sum;
}






// Assume that TYPED individuals have same labels in both
// pedigrees.
double comparePedigrees(vector<Indiv>& sim_pedigree, 
		vector<Indiv>& pedigree,
		vector<unsigned int>& generations, // for ped.
		int num_extant,
		double& nosibship_acc)
{
	// First Generation of relationships ONLY (i.e. second gen descent sets)
	// make upper triangular indicator matrix 
	//    where S(i,j)=1  if i,j are in same descent set for monogomous parents
	//          S(i,j)=0  otherwise



	UndirectedGraph coincidence(num_extant+1);

	// sim_pedigree first
	//printf("Sim_pedigree: ");
	unsigned int i = generations[1]; // first index of second gen.
	if (sim_pedigree.size() <= i)
		printf("WARNING: sim_pedigree out of bounds on attempt to read person %i has size %i.\n", i, (int) sim_pedigree.size());
	fflush(stdout);
	if (2 != sim_pedigree[i].generation)
		printf("WARNING: sim_pedigree and pedigree do not have same number of extant people.\n");
	while (generations.size() > 2 && 
			sim_pedigree.size() > i && 
			i < generations[2])
	{
		if (2 != sim_pedigree[i].generation)
			break;

		vector<int> D_i;
		sim_pedigree[i].GetDescentList(D_i);

		for (unsigned int j = 0;  j < D_i.size();  j++)
		{
			for (unsigned int k = j+1;  k < D_i.size();  k++)
			{
				// then people indexed by j,k appear in same descent set
				//printf(" (%i, %i)",D_i[j], D_i[k]);
				double cmp = coincidence.GetEdge(D_i[j], D_i[k]);
				cmp += 1.0;
				if (cmp > 2.0){
					printf("  FATAL ERROR: too many D-sets for pair %i,%i, index %i, gen %i\n", D_i[j],D_i[k], i, sim_pedigree[i].generation);
					exit(-423453);
				}
				coincidence.SetEdge(D_i[j], D_i[k], cmp);
			}
		}

		i++;
	}
	//printf("\n");


	// count the non-zero edges of the coincidence graph
	// for the number of mistakes if no sib-groups are created
	int sum_nonzero = 0;
	for (int j = 1;  j < num_extant+1;  j++)
		for (int k = j+1;  k < num_extant+1; k++)
		{
			double cmp = coincidence.GetEdge(j,k);
			if (cmp > 0)
				sum_nonzero++;
		}
	nosibship_acc = 1 - (double) sum_nonzero*2 / (num_extant*(num_extant-1));
	//printf("If no sibships, num mistakes would be: %i, frac %f\n", sum_nonzero, 1-nosibship_acc);


	// pedigree
	// for each parent
	//printf("Pedigree: ");
	for (unsigned int i = generations[1];  i < generations[2]; i++)
	{
		vector<int> D_i;
		pedigree[i].GetDescentList(D_i);

		for (unsigned int j = 0;  j < D_i.size();  j++)
		{
			for (unsigned int k = j+1;  k < D_i.size();  k++)
			{
				// then people indexed by j,k appear in same descent set
				double cmp = coincidence.GetEdge(D_i[j], D_i[k]);
				cmp -= 1.0;
				coincidence.SetEdge(D_i[j], D_i[k], cmp);
				//printf(" (%i, %i)=%f",D_i[j], D_i[k], cmp);
			}
		}
	}
	//printf("\n");


	// sum over the edge set of the coincidence graph
	double sum_difference = 0.0;
	for (int j = 1;  j < num_extant+1;  j++)
		for (int k = j+1;  k < num_extant+1; k++)
		{
			double cmp = coincidence.GetEdge(j,k);
			if (cmp < 0)
				cmp *= -1;
			sum_difference += cmp;
		}



	return sum_difference;
}












// Reconstruct appropriate sets for d_splits
// Given prev_d_splits, with a d_split vector for 
// each previously reconstructed generation, and
// the genotypes of the extant individuals.
//
void reconstruct_generation(vector<Indiv>& pedigree,
		vector<unsigned int>& generations,
		UndirectedGraph& sharing,
		double lambda,
		int num_extant,
		double threshold,
		int g)
{


	vector<set<int> > typed_to_founder;
	foundersOfTyped(pedigree, typed_to_founder);




	// num of individuals in most 
	// recently constructed generation
	unsigned int gen = generations.size()-1;
	unsigned int num_oldest_gen = generations[gen] - generations[gen-1];

	//printf("  Reconstruct gen %i, gen index %i, num_oldest_gen %i\n", gen, generations[gen], num_oldest_gen);

	// Compatibility graph
	UndirectedGraph graph(pedigree.size());

	printf("    makeEdges()...\n");
	makeEdges(pedigree, graph, 
			sharing, lambda, 10000, num_extant, num_oldest_gen, gen, threshold, g);
	printf("    max clique...\n");
	fflush(stdout);

	// Until everyone has two parents
	// greedily select the largest clique 
	// and asign them two parents
	unsigned int num_singletons = 0;
	map<int, bool> missingParent;
	for (unsigned int i = 0; i < num_oldest_gen;  i++)
		missingParent[generations[gen-1]+i] = 1;
	bool areMissingParents = 1;
	unsigned int lower_bound = generations[gen-1];
	unsigned int upper_bound = generations[gen]-1;
	graph.AdjList(lower_bound, upper_bound);


	double poisson_mean = g_avgnumofchildren;
	graph.SequentialColor(lower_bound, upper_bound, poisson_mean);

	vector<vector<int>*>* new_extant_list=new vector<vector<int> *>();
	mExtantChildren->push_back(new_extant_list);

	while (areMissingParents)
	{
		//printf("MaxClique\n");

		// UNCOMMENT DELETE _clique !!!!!!!!!!!!!!!!!!
		int* _clique = graph.MaxClique(lower_bound, upper_bound);
		vector<int> clique;
		if (_clique != NULL)
		{
			unsigned int num_vertices = upper_bound - lower_bound + 1;
			for (unsigned int i = 0; i < num_vertices; i++){
				if (_clique[i] == 1)
					clique.push_back(i+lower_bound);
			}
		}


		//      vector<int> clique = graph.NextPartitionSet(lower_bound, upper_bound);



		//printf("  areMissingParents, cliqueSize %i for nodes [%i,%i]\n", clique.size(), lower_bound, upper_bound);

		// totally cludgy
		if (clique.size() == 0){
			printf("FATAL ERROR: clique size is zero, but have missing parents.");
			exit(-4234);
		}
		else if (clique.size() == 1)
			num_singletons++;
		else
			printf("    sibship size %i\n", (int) clique.size());

		// make two parents for the clique individuals
		unsigned int p1 = makeUntypedPerson(&pedigree,gen+1);
		unsigned int p2 = makeUntypedPerson(&pedigree,gen+1);
		// set the parent's distinct alleles




		printf("clique: ");
		set<int>::iterator si;
		for (unsigned int c=0;  c < clique.size(); c++)
		{
			printf("%i, ", clique[c]);
			setParents(&pedigree, clique[c], p1,p2);
			missingParent[clique[c]] = 0;

			// remove person's edges from graph
			graph.DeleteEdgeSet(clique[c]);
		}

		//============================================
		// begin to update the mExtantChildren
		vector<int>* new_extant_list_iter=NULL;
		if(clique.size()==1)
		{
			new_extant_list_iter=(*((*mExtantChildren)[gen-1]))[clique[0]-lower_bound];        
		}//if
		else  if(clique.size()>1) 
		{
			//try to merge the childrens
			int c_size=clique.size();
			new_extant_list_iter=new vector<int>();
			int* indices=new int[c_size];
			vector<int>** lists=new vector<int> *[c_size];
			for(int k=0; k<c_size; k++)
			{
				indices[k]=0;
				int tt=clique[k]-lower_bound;//this need to be changed later
				lists[k]=(*((*mExtantChildren)[gen-1]))[tt];
			}
			bool all_merged=false;
			while(!all_merged)
			{
				all_merged=true;
				int min_index=-1000;
				int min_v=-1000;
				for(int k=0; k<c_size; k++)
				{

					if(indices[k] >= (signed) lists[k]->size()) continue;
					//cout<<indices[k]<<" test "<<lists[k]->size()<<endl;
					vector<int>* ll=lists[k];
					all_merged=false;
					if(min_v<(*ll)[indices[k]])
					{
						min_v=(*ll)[indices[k]];
						min_index=k;
					}//if
				}//for
				if(!all_merged)
				{
					new_extant_list_iter->push_back((*(lists[min_index]))[indices[min_index]]);
					indices[min_index]++;
				}
			}
			delete [] indices;
			delete [] lists;

		} 
		new_extant_list->push_back(new_extant_list_iter);
		new_extant_list->push_back(new_extant_list_iter);


		//
		//end of updating the mExtantChildren
		//============================================


		printf("\n");

		// see if there are people missing any parents
		//printf("missing parents: ");
		areMissingParents = 0;
		for (unsigned int i = 0; i < num_oldest_gen;  i++){
			if(missingParent[generations[gen-1]+i] == 1){
				//printf("%i, ", generations[gen-1]+i);
				areMissingParents = 1;
			}
		}
		//printf("\n");


		delete[] _clique;
	}

	if (num_singletons == num_oldest_gen)
		printf("WARNING: no %i-th cousins were created!!!!!!!!\n", gen-1);

	generations.push_back(pedigree.size());
}




// Reconstruct a pedigree with depth estimate_depth
// put reconstruction into pedigree which contains
// parent pointers, children pointers, and 
// a descent set.
//
//
//  TRACK start index for each generation (remove counting from makeEdges)
//
void reconstruct_pedigree(int estimate_depth, 
		int num_extant, 
		vector<double> thresholds,
		vector<vector<double> >& expected_sharing) 
{
	UndirectedGraph sharing(num_extant+1);

	observed_sharing(sharing, expected_sharing, num_extant);


	//double lambda = -(1.0 / 2.0) * log(1-2 * g_recombrate);
	double lambda = -log(1-g_recombrate);
	//printf("lambda = %f\n", lambda);


	// build the first generation of the pedigree
	makeUntypedPerson(&pedigree, 0); // dummy person
	generations.push_back(1);
	for (int i = 0;  i < num_extant;  i++)
	{
		makeTypedPerson(&pedigree, 1);
		//populate_alleles(&pedigree, i, pedigree.size()-1);
	}
	generations.push_back(pedigree.size());



	mExtantChildren  =new vector<vector<vector<int> *>* >();  
	vector<vector<int>* >* extant_list=new vector<vector<int> *>(num_extant+1);
	for(int i=0; i<num_extant; i++)
	{
		(*extant_list)[i]=new vector<int>();
		(*extant_list)[i]->push_back(i+1);
	}
	mExtantChildren->push_back(extant_list);



	int num_recent_gen = generations[1] - generations[0];
	if (num_recent_gen != num_extant){
		printf("WARNING: disagreement about num extant individuals\n");
		printf("num last gen %i,  num extant %i\n", num_recent_gen, num_extant);
	}


	for (int g = 0;  g < estimate_depth;  g++)
	{
		// Construct this generation's descent sets from the
		// decisions at the previous generations.
		printf("Constructing generation %i with threshold %f\n", g+2, thresholds[g]);

		reconstruct_generation(pedigree, generations, sharing, 
				lambda, num_extant,
				thresholds[g], g+1);

		//printPedigree(pedigree);
	}

	int num_gen=mExtantChildren->size();
	for(int i=0; i<num_gen; i++)
	{
		vector<vector<int>*>* extants_lists=(*mExtantChildren)[i];
		//int num_indiv=extants_lists->size();
		for(int j=0; j<num_gen; j++)
		{
			//delete (*extants_lists)[j];
		}
		delete extants_lists;
	}
	delete mExtantChildren;


}







void randIndiv(unsigned int num_choose, 
		vector<int>& extant_ptrs)
{
	set<int> chosen;
	for (unsigned int i = 0;  i < num_choose;  i++)
	{
		bool chosenAlready = true;
		while (chosenAlready)
		{
			double u = drand48()*100000;
			int idx = (int) u % g_numofancestors;
			idx++;

			set<int>::iterator si = chosen.find(idx);
			if (si != chosen.end()) {
				chosenAlready = true;
			} else {
				chosenAlready = false;
				chosen.insert(idx);
				extant_ptrs.push_back(idx);
			}
		}
	}
}









void connectedIndiv(vector<Indiv>& pedigree,
		unsigned int num_choose, 
		vector<int>& extant_ptrs)
{
	set<int> chosen;

	// choose starting vertex for BFS search
	double u = drand48()*100000;
	int idx = (int) u % g_numofancestors;
	/*	while(!pedigree[idx+1].gender==MALE)
		{
		u=drand48()*100000;
		idx=(int) u % g_numofancestors;
		}
		*/
	cout<<"starting point: "<<idx <<endl;

	idx++;

	//printf("index: %i, sizeped %i\n", idx, (int) pedigree.size());
	//fflush(stdout);

	queue<int> Q;
	Q.push(idx);
	while (chosen.size() < num_choose)
	{
		int idx = Q.front();
		Q.pop();

		if ((unsigned) idx < 0 || pedigree.size() <= (unsigned) idx)
		{
			continue;
		}

		// get the extant descendants
		vector<int> dsplit;
		if (pedigree.size() > (unsigned) idx)
		{
			//printf("idx %i\n", idx);
			fflush(stdout);
			//printf("   dsplits size %i\n", pedigree[idx].descendants.size());
			fflush(stdout);
			pedigree[idx].GetDescentList(dsplit);
			//printf("   got dsplit\n");
			fflush(stdout);
		}
		for (unsigned int i = 0;  i < dsplit.size();  i++)
		{
			set<int>::iterator si = chosen.find(dsplit[i]);
			if (chosen.size() < num_choose &&
					si == chosen.end()) {
				chosen.insert(dsplit[i]);
				extant_ptrs.push_back(dsplit[i]-1);
			}
		}

		for (unsigned int p = 0; p < pedigree[idx].parents.size(); p++)
		{
			Q.push(pedigree[idx].parents[p]);
		}

		if (Q.empty())
		{
			double u = drand48()*100000;
			int idx = (int) u % g_numofancestors;
			idx++;
			Q.push(idx);
			cout<<"new starting point: "<<idx<<endl;
		}
	}

	sort(extant_ptrs.begin(), extant_ptrs.end());

}




vector<int> drawRandPed(vector<Indiv>& sim_pedigree)
{

	// make random pedigree from WF model 
	for (int i = 0; i < 2; i++) {
		for (int j = 0; j < g_numofgenerations; j++) {
			for (int k = 0; k < g_numofancestors; k++) {
				g_parents[i][j][k] = -1;
			}
		}
	}
	for (int i = 0; i < g_numofgenerations; i++) {
		for (int j = 0; j < 2*g_numofancestors; j++) {
			for (int k = 0; k < g_numofregions; k++)
				g_genotype[i][j][k] = -1;
		}
	}
	printf ("  initializing generation 0\n");
	fflush(stdout);
	initializegeneration0();
	//printgenotypes(0);

	for (int i = 1; i < g_numofgenerations; i++) {
		buildgeneration(i);
		/*for(int j=0;j<g_numofancestors;j++)
		  {
		  cout<<g_numofancestors*(i)+j<<":   "<<g_numofancestors*(i-1)+g_parents[0][i][j]<<"    "<<g_numofancestors*(i-1)+g_parents[1][i][j]<<endl;
		  }
		  */
		//printgenotypes(i);
	}
	//**** Print out the whole WF model, the huge pedigree ************** 
	/*	cout<<"**********"<<endl;
		for(int i=g_numofgenerations-1;i>0;i--)
		{
		cout<<"---generation: "<<i<<endl;
		for(int j=0;j<g_numofancestors;j++)
		{
		cout<<g_numofancestors*(i)+j<<":   "<<g_numofancestors*(i-1)+g_parents[0][i][j]<<"    "<<g_numofancestors*(i-1)+g_parents[1][i][j]<<endl;
		}

		}
		*/
	//********************************************************************


	//printgenotypes(g_numofgenerations-1);

	// choose the individuals
	vector<int> extant_ptrs;
	if (g_outbred)
	{
		printf("Outbred sim\n");
		//randIndiv(tmp_ancestors, extant_ptrs); // sample disconnected people
		vector<Indiv> tmp_ped; // first make ped from whole simulation

		// whole simulation made into ped, since extant ptrs is empty
		make_sim_pedigree(tmp_ped, extant_ptrs);
		connectedIndiv(tmp_ped, tmp_ancestors, extant_ptrs); // sample connected people
	}


	printf("Num extant %i\n", (int) extant_ptrs.size());
	make_sim_pedigree(sim_pedigree, extant_ptrs);
	//printf("Simulated Pedigree with %i actual num people:\n", g_actualnumindividuals);
	printPedigree(sim_pedigree);
	cout<<"finished generating pedigree"<<endl;

	if (g_outbred)
	{
		// copy the genotypes into the first tmp individuals
		vector<vector<int> > tmp_geno(2*extant_ptrs.size(), vector<int>(g_numofregions, 0)); // people at this gen
		for (int i = 0; i < extant_ptrs.size(); i++) {
			for (int j = 0; j < g_numofregions; j++) {
				int idx = extant_ptrs[i]; 

				if (idx >= g_numofancestors){
					printf("WARNING: idx out of bounds\n");
					fflush(stdout);
				}
				tmp_geno[2*i][j] = g_genotype[g_numofgenerations-1][2*idx][j];
				tmp_geno[2*i+1][j] = g_genotype[g_numofgenerations-1][2*idx+1][j];
			}
		}
		for (int i = 0; i < extant_ptrs.size(); i++) {
			for (int j = 0; j < g_numofregions; j++) {
				g_genotype[g_numofgenerations-1][2*i][j] = tmp_geno[2*i][j];
				g_genotype[g_numofgenerations-1][2*i+1][j] = tmp_geno[2*i+1][j];
			}
		}
		printf("");

		g_actualnumindividuals = extant_ptrs.size();
	}




	return extant_ptrs;

}



void makeRandGenerations(vector<unsigned int>& rand_generations, vector<Indiv>& rand_pedigree)
{
	for (unsigned int i = 0; i < rand_pedigree.size(); i++)
	{
		unsigned int g = rand_pedigree[i].generation;
		while (rand_generations.size() <= g)
			rand_generations.push_back(0);
		rand_generations[g] = i+1;
	}
}





#endif
