#ifndef _TEST_ALG_4_H_
#define _TEST_ALG_4_H_

#include "test_alg.h"
#include "MatGraph.h"
#include "AdjGraph.h"
#include "ClusterGraph.h"
#include <iostream>
#include "Option.h"
using namespace std;


//vector<vector<vector<int> *> *>* mExtantChildren;

void observed_sharing(MatGraph* sharing, 
		      vector<vector<double> >& expected_len_sharing, 
		      int num_indiv)
{
  // ASSUME genotypes are PHASED

  // Genotypes of extant generation are in
  //   g_genotypes[g_numofgenerations-1][chr][pos]
  //      w/here chr = 2 * indiv + j, with j in {0,1} 
  //      and pos in [0,...,g_numofregions]

  vector<double> row(num_indiv+1, 0.0);
  expected_len_sharing.push_back(row);


  //printf("numofregions %i, numgen %i\n", g_actualnumofregions, g_numofgenerations);


  double avg_avg_length = 0.0;
  int avg_avg_number = 0;
  double avg_sq_diff = 0.0;
  for (int i = 0; i < num_indiv; i++)
    {
      vector<double> row(num_indiv+1, 0.0);
      expected_len_sharing.push_back(row);

      for (int j=i;  j < num_indiv; j++)
	{
	  //printf("  %i,%i) ", i+1,j+1);

	  // compute sharing for this pair of people
	  vector<int> max_share;

	  int length1 = 0;
	  int length2 = 0;
	  for (int pos = 0;  pos < g_actualnumofregions; pos++)
	    {
	      // indicates orientation of sharing (consider both pairings of haps)
	      int isShared = 0;
	      int a_i0 = g_genotype[g_numofgenerations-1][2*i][pos];
	      int a_i1 = g_genotype[g_numofgenerations-1][2*i+1][pos];
	      int a_j0 = g_genotype[g_numofgenerations-1][2*j][pos];
	      int a_j1 = g_genotype[g_numofgenerations-1][2*j+1][pos];

	      if (i == j){
		if(a_i0 == a_i1)
		  isShared = 1;
	      } else {
		if (a_i0 == a_j0 || a_i1 == a_j1)
		  isShared = 1;
		else if (a_i0 == a_j1 || a_i1 == a_j0)
		  isShared = 2;
	      }

	      //printf(" {%i,%i}=%i={%i,%i}", a_i0, a_i1, isShared, a_j0, a_j1);
	      if (isShared == 1) {
		length1++;
	      } else {
		if (length1 > 0)
		  max_share.push_back(length1);
		length1 = 0;
	      }
	      if (isShared == 2) {
		length2++;
	      } else {
		if (length2 > 0)
		  max_share.push_back(length2);
		length2 = 0;
	      }

	    } // end for pos
	  if (length1 > 0)
	    max_share.push_back(length1);
	  if (length2 > 0)
	    max_share.push_back(length2);
	  //printf("\n");

	  // average length of max-length sharing regions
	  double avg_length = 0.0;
	  //printf("Avg Share: ");
	  for (unsigned int k = 0;  k < max_share.size();  k++)
	    {
	      //printf(" %i +", max_share[k]);
	      avg_length += max_share[k];
	      avg_avg_length += max_share[k];
	      avg_avg_number++;
	      avg_sq_diff += pow(max_share[k]-2.690608, 2);
	    }
	  if (max_share.size() > 0)
	    avg_length /= max_share.size();
	  //printf(" = %f\n", avg_length);
	  //printf("sharing->setWeight(%i,%i) = %f\n",i+1,j+1,avg_length);
	  sharing->setWeight(i+1,j+1, avg_length);
	  expected_len_sharing[i+1][j+1] = avg_length / g_numofregions;
	}
    }



  //printf("avg of all lengths %f = %f/%i\n", (avg_avg_length / (double) avg_avg_number), avg_avg_length,  avg_avg_number);
  //printf("var of all lengths %f\n", (avg_sq_diff / (double) avg_avg_number));
  //printf("\n");
}

double ifSiblingsCoefficient4(double observed_sharing, 
			     int num_generation, 
			     double lambda)

{
  double x=2;
  double p_same=1/pow(x,  2*num_generation);
  double expectLength=2/(2*num_generation*lambda);
  double variance=expectLength*expectLength/2;
  //cout<<"shared length"<<(observed_sharing)<<" "<<expectLength<<" "<<variance<<endl;
  double length_test = pow((observed_sharing-expectLength), 2)/variance;
  //printf("length_test %f\n", length_test);

  double L = g_numofregions; 
  double k = 2*num_generation;
  double expectedNumber = k*lambda*L / pow(2, k-1);

  return length_test; // + weight * num_test;
}

double ifNotSiblingsCoefficient(double observed_sharing,  int num_generation, double total_lens)

{
  return ifSiblingsCoefficient4(observed_sharing, num_generation+1, total_lens);
}



//vector<int>* temp_v1=new vector<int>();
//vector<int>* temp_v2=new vector<int>();
double
computePairWeight(MatGraph* observed_sharing, 
		  int index1, int index2, int generation_no, double lambda)
{
   vector<int>* extant1=(*((*mExtantChildren)[generation_no-1]))[index1];
   vector<int>* extant2=(*((*mExtantChildren)[generation_no-1]))[index2];
   int size1=extant1->size(); 
   int size2=extant2->size();
   int i1=0;
   int i2=0;
   double result=0;
   double counter=0;
   temp_v1->clear();
   temp_v2->clear();
   while(i1!=size1 || i2!=size2)
   {
      if(i1==size1)
      {
         temp_v2->push_back((*extant2)[i2]);
	 i2++;
	 continue;
      }
      if(i2==size2)
      {
         temp_v1->push_back((*extant1)[i1]);
         i1++;
	 continue;
      }
      if((*extant1)[i1]<(*extant2)[i2])
      {
	 for(int j=i2; j<size2; j++)
         {		 
	      result+= ifSiblingsCoefficient4(observed_sharing->getWeight((*extant1)[i1],
									 (*extant2)[j]),
					     generation_no, lambda);
	      counter+=1;
         }
	 temp_v1->push_back((*extant1)[i1]);
        i1++;
      }
      else if((*extant1)[i1]==(*extant2)[i2])
      {
	 i1++;
	 i2++;

      }
      else
      {
	for(int i=i1; i<size1; i++)
         {		 
	      result+= ifSiblingsCoefficient4(observed_sharing->getWeight((*extant1)[i],
									 (*extant2)[i2]), 
					     generation_no, lambda);
	      counter+=1;
         }//for
	 temp_v2->push_back((*extant2)[i2]);     
	 i2++;
      }//else
   }

   int s1=temp_v1->size();
   int s2=temp_v2->size();
   result=0;
   counter=0;
   for(int i=0; i<s1; i++)
   {
      for(int j=0; j<s2; j++)
      {
         result+=ifSiblingsCoefficient4(observed_sharing->getWeight((*temp_v1)[i],
								   (*temp_v2)[j]),
	                               generation_no, lambda);
	 counter++;
      }
   }
   //cout<<"+!++!+!+!+!+!+!+"<<counter<<endl;
   if(counter==0)
   {
     return 0;
   }
   return (result/counter);

   
}





void makeEdges(vector<Indiv>& pedigree, 
	       AdjGraph* graph,
	       MatGraph* observed_sharing,
	       double lambda,
	       unsigned int num_samples,
	       unsigned int num_extant,
	       // num of indiv in most recently constructed generation
	       unsigned int num_oldest_gen, 
	       unsigned int oldest_gen,
	       double threshold, 
	       int g)
{

  //printf("COP test\n");

  int first_of_last_gen = pedigree.size()-num_oldest_gen;
  int last_of_last_gen = pedigree.size()-1;
  bool anyEdges=false;
  int size=last_of_last_gen-first_of_last_gen+1;
  for (int k = 0;  k <size;  k++)
  {
      for (int l = k+1;  l <size;  l++)
      {
	   double log_test=computePairWeight(observed_sharing, 
					     k, l, g, lambda);
           if(log_test==0) log_test=20;
	   if (log_test < threshold)
	   {
	      graph->insertEdge(k,l, threshold-log_test);
	      anyEdges=true;
	   }//if
      }
   }
  // delete the extra parents
   if(!anyEdges)  cout<<"construction should stop at generation"<<(g+1)<<endl;
}






double computePairWeightsCIP(vector<Indiv>& pedigree, 
			     UndirectedGraph& observed_sharing,
			     UndirectedGraph& w, // <-- weights to compute
			     double lambda,
			     unsigned int num_samples,
			     unsigned int num_recent_gen,
			     unsigned int oldest_gen, 
			     int k, int l)
{
  double log_test = 0.0;

  set<int> dk;
  pedigree[k].GetDescentList(dk);

  set<int> dl;
  pedigree[l].GetDescentList(dl);
  //if(dk==dl) return 10000;

  for (unsigned int i = 1;  i <= num_recent_gen;  i++)
    {
      set<int>::iterator found = dk.find(i);
      if (found == dk.end())
	continue;

      for (unsigned int j = 1;  j <= num_recent_gen;  j++)
	{
	  set<int>::iterator found = dl.find(j);
	  if (found == dl.end())
	    continue;

	  //printf("\n (%i,%i) IBD\n-----------\n", i,j);


	  //paths.clear();
	  resetColor(pedigree);
	  colorPedigree(pedigree, i,j, 999);

	  vector<unsigned int> all_paths;
	  vector<unsigned int> paths;
	  //unsigned int m = getInheritance(paths, pedigree, i, j, 999);
	  unsigned int m = getInheritance2(paths, pedigree, i, j, 999);


	  if (m > 0)
	    {
#ifdef MODELS
	      if (oldest_gen == 3)
		printf("  got inheritance with %i meioses and %i paths\n", m, paths.size());
	      fflush(stdout);
#endif


	      //printf("  num meioses: %i, num inheritance paths: %i\n", m, paths.size());


	      vector<int> sum(paths.size(), 0);
	      vector<vector<int> > rates(paths.size(), vector<int>(paths.size()+1, 0));
	      getTransitionRates(rates, sum, paths, m);
	      
#ifdef MODELS
	      if (oldest_gen == 3)
		printf("  got transition rates with %i rates\n", rates.size());
	      fflush(stdout);
#endif


	      //map<unsigned int, double> prob;
	      vector<double> length;
	      samplePoissonRandomWalk(length, paths, rates, lambda, m, num_samples);

	      // compute w[i][j]
	      //printf("  observed_sharing.GetEdge(%i,%i)=%f\n", i,j, observed_sharing->getWeight(i,j));
	      double weight = testCoefficient(length, 
					      observed_sharing.GetEdge(i,j)
					      );

#ifdef MODELS
	      printf("  weight[%i][%i] = %f\n", i,j,weight);
	      fflush(stdout);
#endif
	      w.SetEdge(i,j, weight);
	      log_test += weight / (dk.size() * dl.size());
	    }
	  else
	    {
	      w.SetEdge(i,j, default_test);
	      log_test += default_test / (dk.size() * dl.size());
	    }
	}
    }
  return log_test;
}


// Note that pedigree individuals are numbered starting at 1
// (i.e. there should be a dummy individual at zero
void makeEdgesCIP(vector<Indiv>& pedigree, 
		  AdjGraph* graph,
	//	  UndirectedGraph& graph2,
                  UndirectedGraph& observed_sharing,
		  double lambda,
		  unsigned int num_samples,
		  unsigned int num_extant,
		  // num of indiv in most recently constructed generation
		  unsigned int num_oldest_gen, 
		  unsigned int oldest_gen,
		  double threshold,
		  int g)
{
  //printf("CIP test\n");


  // For existing pedigree, (i.e. no g'th generation relationships)
  // estimate the sharing

  // For all pairs of people, k,l, in the oldest generation, 
  // compute test weights as if k,l are siblings
  // Added edges to graph when the test passes the threshold
  int first_of_last_gen = pedigree.size()-num_oldest_gen;
  int last_of_last_gen = pedigree.size()-1;

  int parent1 = makeUntypedPerson(&pedigree,oldest_gen+1);
  int parent2 = makeUntypedPerson(&pedigree,oldest_gen+1);
  for (int k = first_of_last_gen;  k <= last_of_last_gen;  k++)
    {
      set<int> dk;
      pedigree[k].GetDescentList(dk);
      for (int l = k+1;  l <= last_of_last_gen;  l++)
	{
	  set<int> dl;
	  pedigree[l].GetDescentList(dl);

	  // make k & l siblings
	  setParents(&pedigree, k, parent1,parent2);
	  setParents(&pedigree, l, parent1,parent2);

	  //printPedigree(pedigree);

	  //printf("##me## observed_sharing->getWeight(1,2) = %f\n", observed_sharing->getWeight(1,2));

	  UndirectedGraph w_kl(num_extant+1);
	  double log_test = computePairWeightsCIP(pedigree, observed_sharing,
					       w_kl, // <-- weights to compute
					       lambda, num_samples, 
					       num_extant, oldest_gen,
					       k, l);
	  //printf("log_test %f\n", log_test);

	  if (log_test < threshold)
	    {
	      //printf("     added edge %i,%i, value %f + %f\n", k,l, log_test-threshold, threshold);

	      graph->insertEdge(k-first_of_last_gen,l-first_of_last_gen, threshold-log_test);
             //graph2.SetEdge(k,l,threshold-log_test);
	    }


	  // make k & l NOT siblings
	  unsetParents(&pedigree, k, parent1,parent2);
	  unsetParents(&pedigree, l, parent2,parent2);
	}
    }
  // delete the extra parents
  pedigree.pop_back();
  pedigree.pop_back();

}




// Reconstruct appropriate sets for d_splits
// Given prev_d_splits, with a d_split vector for 
// each previously reconstructed generation, and
// the genotypes of the extant individuals.
//
void reconstruct_generation(vector<Indiv>& pedigree,
			    vector<unsigned int>& generations,
			    MatGraph* sharing,
			    double lambda,
			    int num_extant,
			    double threshold,  int g)
{


  vector<set<int> > typed_to_founder;
  foundersOfTyped(pedigree, typed_to_founder);



  // num of individuals in most 
  // recently constructed generation
  unsigned int gen = generations.size()-1;
  unsigned int num_oldest_gen = generations[gen] - generations[gen-1];

  //printf("  Reconstruct gen %i, gen index %i, num_oldest_gen %i\n", gen, generations[gen], num_oldest_gen);

  // Compatibility graph
  AdjGraph* graph=new AdjGraph(0.9, num_oldest_gen);
 // UndirectedGraph graph2(pedigree.size());
  printf("    makeEdges()...\n");
  makeEdges(pedigree, graph, 
	    sharing, 
	    lambda, 100, 
	    num_extant, num_oldest_gen, 
	    gen, threshold,  g);
 // graph->print();
  ClusterGraph* cg=new ClusterGraph(graph, Option::a_value);
  printf("    clustering...\n");
  fflush(stdout);
  cg->PerformClustering();
  int offset= generations[gen-1]; 
  // Until everyone has two parents
  // greedily select the largest clique 
  // and asign them two parents
  //================================test code begins====================
  unsigned int num_singletons = 0;
  map<int, bool> missingParent;
  for (unsigned int i = 0; i < num_oldest_gen;  i++)
    missingParent[generations[gen-1]+i] = 1;
  bool areMissingParents = 1;
  unsigned int lower_bound = generations[gen-1];
  unsigned int upper_bound = generations[gen]-1;
  double poisson_mean = g_avgnumofchildren;


  num_singletons = 0;
  //map<int, bool> missingParent;
  for (unsigned int i = 0; i < num_oldest_gen;  i++)
    missingParent[generations[gen-1]+i] = 1;
   areMissingParents = 1;
   lower_bound = generations[gen-1];
   upper_bound = generations[gen]-1;


  //cg->print();

  poisson_mean = g_avgnumofchildren;
  vector<vector<int>*>* new_extant_list=new vector<vector<int> *>();
  mExtantChildren->push_back(new_extant_list);

  vector<Cluster *>* clusters=cg->getClusterCopy();
  int num_cluster=clusters->size();
  for(int i=0; i<num_cluster; i++)
  {
     Cluster* cluster=(*clusters)[i];
  
     if (cluster->getSize() == 1)
	num_singletons++;
     else printf("    sibship size %i\n", cluster->getSize());
      
      unsigned int p1 = makeUntypedPerson(&pedigree,gen+1);
      unsigned int p2 = makeUntypedPerson(&pedigree,gen+1);
      vector<int>* new_extant_list_iter=NULL; 
      int c_size=cluster->getSize();
      vector<ClusterVertex* >* vertices=cluster->getVertices()->getCopy();
      if(c_size==1)
      {
	 new_extant_list_iter=(*((*mExtantChildren)[gen-1]))[(*vertices)[0]->getVertex()->getNumber()];
      }
      else
      {
        new_extant_list_iter=new vector<int>();
        int* indices=new int[c_size];
	vector<int>** lists=new vector<int> *[c_size];
	for(int k=0; k<c_size; k++)
	{
	   indices[k]=0;
	   int tt=((*vertices)[k])->getVertex()->getNumber();//this need to be changed later
           lists[k]=(*((*mExtantChildren)[gen-1]))[tt]; 
        }	
	bool all_merged=false;
	while(!all_merged)
	{
	  all_merged=true;
	  int min_index=-1000;
	  int min_v=-1000;
	  for(int k=0; k<c_size; k++)
	  {
	    
	    if(indices[k]>=lists[k]->size()) continue;
	    //cout<<indices[k]<<" test "<<lists[k]->size()<<endl;
	    vector<int>* ll=lists[k];
	    all_merged=false;
	    if(min_v<(*ll)[indices[k]])
	    {
	       min_v=(*ll)[indices[k]];
	       min_index=k;
	    }//if
	  }//for
	  if(!all_merged)
	  {
	    new_extant_list_iter->push_back((*(lists[min_index]))[indices[min_index]]);
	    indices[min_index]++;
	  }  
	}
	delete [] indices;
	delete [] lists;
      }
      int size=new_extant_list_iter->size();
      bool lower=false;
      bool upper=false;
      for(int i=0; i<size; i++)
      {
         if((*new_extant_list_iter)[i]<=88) lower=true;
         else upper=true;  
      }
      if(lower && upper)
      {
         for(int i=0; i<size; i++)
         {
            for(int j=i+1; j<size; j++)
            {
             
              cout<<"mixed "<<((*new_extant_list_iter)[i])<<" "<<((*new_extant_list_iter)[j])<<" ";
              cout<<sharing->getWeight((*new_extant_list_iter)[i], (*new_extant_list_iter)[j])<<endl; 
            }
          }
	  pedigree[p1].setAffect(0);
          pedigree[p2].setAffect(0);
      }
      else if(lower)
      {
          pedigree[p1].setAffect(1);
          pedigree[p2].setAffect(1);
      }
      else
      {
          pedigree[p1].setAffect(2);
          pedigree[p2].setAffect(2);
      }


      new_extant_list->push_back(new_extant_list_iter);
      new_extant_list->push_back(new_extant_list_iter);

      // set the parent's distinct alleles

      for (unsigned int c=0;  c < c_size; c++)
	{
	  if(c_size>1) printf("%i, ", ((*vertices)[c]->getVertex()->getNumber()+offset));
	  //it is slow now, but it can be faster;
	
	  Indiv* c_d=&(pedigree[(*vertices)[c]->getVertex()->getNumber()+offset]);
	  setParents(&pedigree, (*vertices)[c]->getVertex()->getNumber()+offset, p1, p2);

	  // remove person's edges from graph
	}

	  if(c_size>1) {
		  //
	  } else if (c_size == 1) {
		  printf("%i, ", ((*vertices)[0]->getVertex()->getNumber()+offset));
	  }
	  printf("%d-%d\n",p1,p2);

	  delete vertices;
	  // see if there are people missing any parents
	  areMissingParents = 0;
	  for (unsigned int i = 0; i < num_oldest_gen;  i++){
		  if(missingParent[generations[gen-1]+i] == 1){
			  areMissingParents = 1;
		  }
	  }


      //delete[] _clique;
    }

  if (num_singletons == num_oldest_gen)
    printf("WARNING: no %i-th cousins were created!!!!!!!!\n", gen-1);
  generations.push_back(pedigree.size());
}



void reconstruct_generationCIP(vector<Indiv>& pedigree,
                            vector<unsigned int>& generations,
                            UndirectedGraph& sharing,
                            double lambda,
                            int num_extant,
                            double threshold,
                            int g)
{


  vector<set<int> > typed_to_founder;
  foundersOfTyped(pedigree, typed_to_founder);



  // num of individuals in most 
  // recently constructed generation
  unsigned int gen = generations.size()-1;
  unsigned int num_oldest_gen = generations[gen] - generations[gen-1];

  //printf("  Reconstruct gen %i, gen index %i, num_oldest_gen %i\n", gen, generations[gen], num_oldest_gen);

  // Compatibility graph
  AdjGraph* graph=new AdjGraph(0.01, num_oldest_gen);
  //UndirectedGraph graph2(pedigree.size());
  printf("    makeEdges()...\n");
  makeEdgesCIP(pedigree, graph, 
    //        graph2, 
	    sharing, 
	    lambda, 1000, 
	    num_extant, num_oldest_gen, 
	    gen, threshold, g);
  printf("    clustering...\n");
  ClusterGraph* cg;
  if(g==0)
     cg=new ClusterGraph(graph, Option::a_value);
  else cg=new ClusterGraph(graph, Option::a_value);
  

  fflush(stdout);
  cg->PerformClustering();
  int offset= generations[gen-1]; 
  // Until everyone has two parents
  // greedily select the largest clique 
  // and asign them two parents
  //================================test code begins====================
  unsigned int num_singletons = 0;
  map<int, bool> missingParent;
  for (unsigned int i = 0; i < num_oldest_gen;  i++)
    missingParent[generations[gen-1]+i] = 1;
  bool areMissingParents = 1;
  unsigned int lower_bound = generations[gen-1];
  unsigned int upper_bound = generations[gen]-1;
  double poisson_mean = g_avgnumofchildren;

  //cg->print();

  poisson_mean = g_avgnumofchildren;

  vector<Cluster *>* clusters=cg->getClusterCopy();
  int num_cluster=clusters->size();
  for(int i=0; i<num_cluster; i++)
  {
     Cluster* cluster=(*clusters)[i];
  
     if (cluster->getSize() == 1)
	num_singletons++;
     else printf("    sibship size %i\n", cluster->getSize());
      
      unsigned int p1 = makeUntypedPerson(&pedigree,gen+1);
      unsigned int p2 = makeUntypedPerson(&pedigree,gen+1);
      // set the parent's distinct alleles
      int c_size=cluster->getSize();
      vector<ClusterVertex* >* vertices=cluster->getVertices()->getCopy();
       for (unsigned int c=0;  c < c_size; c++)
	{
	  if(c_size>1) printf("%i, ", ((*vertices)[c]->getVertex()->getNumber()+offset));
	  //it is slow now, but it can be faster;
	
	  Indiv* c_d=&(pedigree[(*vertices)[c]->getVertex()->getNumber()+offset]);
	  setParents(&pedigree, (*vertices)[c]->getVertex()->getNumber()+offset, p1, p2);
	}

	   if(c_size>1) {
		   //
	   } else if (c_size == 1) {
		   printf("%i, ", ((*vertices)[0]->getVertex()->getNumber()+offset));
	   }
	   printf("%d-%d\n",p1,p2);
	   delete vertices;
  }

  if (num_singletons == num_oldest_gen)
    printf("WARNING: no %i-th cousins were created!!!!!!!!\n", gen-1);
  generations.push_back(pedigree.size());
}
			       



// Reconstruct a pedigree with depth estimate_depth
// put reconstruction into pedigree which contains
// parent pointers, children pointers, and 
// a descent set.
//
//
//  TRACK start index for each generation (remove counting from makeEdges)
//
void reconstruct_pedigree(int estimate_depth, 
			  int num_extant, 
			  vector<double> thresholds,
                          vector<vector<double> >& expected_len_sharing,
                          MatGraph* sharing)
{
  //double lambda = -(1.0 / 2.0) * log(1-2 * g_recombrate);
  double lambda = -log(1.0-g_recombrate);
  //printf("lambda = %f\n", lambda);


  // build the first generation of the pedigree
  makeUntypedPerson(&pedigree, 0); // dummy person
  generations.push_back(1);
  for (int i = 0;  i < num_extant;  i++)
  {
      makeTypedPerson(&pedigree, 1);
      //populate_alleles(&pedigree, i, pedigree.size()-1);
  }
  generations.push_back(pedigree.size());
  
  mExtantChildren  =new vector<vector<vector<int> *>* >();  
  vector<vector<int>* >* extant_list=new vector<vector<int> *>(num_extant+1);
  for(int i=0; i<num_extant; i++)
  {
     (*extant_list)[i]=new vector<int>();
     (*extant_list)[i]->push_back(i+1);
  }
  mExtantChildren->push_back(extant_list);
  int num_recent_gen = generations[1] - generations[0];
  if (num_recent_gen != num_extant){
    printf("WARNING: disagreement about num extant individuals\n");
    printf("num last gen %i,  num extant %i\n", num_recent_gen, num_extant);
  }


  for (int g = 0;  g < estimate_depth;  g++)
    {
      // Construct this generation's descent sets from the
      // decisions at the previous generations.
      printf("Constructing generation %i with threshold %f\n", g+2, thresholds[g]);

      reconstruct_generation(pedigree, generations, sharing,
			     lambda, num_extant,
			     thresholds[g],  g+1);

    }
   int num_gen=mExtantChildren->size();
   for(int i=0; i<num_gen; i++)
   {
      vector<vector<int>*>* extants_lists=(*mExtantChildren)[i];
      int num_indiv=extants_lists->size();
      for(int j=0; j<num_gen; j++)
      {
         //delete (*extants_lists)[j];
      }
      delete extants_lists;
   }
   delete mExtantChildren;
   delete sharing;
}





void reconstruct_pedigreeCIP(int estimate_depth,
                          int num_extant,
                          vector<double> thresholds,
                          vector<vector<double> >& expected_sharing, UndirectedGraph & sharing)
{
  //double lambda = -(1.0 / 2.0) * log(1-2 * g_recombrate);
  double lambda = -log(1-g_recombrate);
  //printf("lambda = %f\n", lambda);

  // build the first generation of the pedigree
  makeUntypedPerson(&pedigree, 0); // dummy person
  generations.push_back(1);
  for (int i = 0;  i < num_extant;  i++)
    {
      makeTypedPerson(&pedigree, 1);
      //populate_alleles(&pedigree, i, pedigree.size()-1);
    }
  generations.push_back(pedigree.size());



  mExtantChildren  =new vector<vector<vector<int> *>* >();
  vector<vector<int>* >* extant_list=new vector<vector<int> *>(num_extant+1);
  for(int i=0; i<num_extant; i++)
  {
     (*extant_list)[i]=new vector<int>();
     (*extant_list)[i]->push_back(i+1);
  }
  mExtantChildren->push_back(extant_list);



  int num_recent_gen = generations[1] - generations[0];
  if (num_recent_gen != num_extant){
    printf("WARNING: disagreement about num extant individuals\n");
    printf("num last gen %i,  num extant %i\n", num_recent_gen, num_extant);
  }


  for (int g = 0;  g < estimate_depth;  g++)
    {
      // Construct this generation's descent sets from the
      // decisions at the previous generations.
      printf("Constructing generation %i with threshold %f\n", g+2, thresholds[g]);

      reconstruct_generationCIP(pedigree, generations, sharing,
                             lambda, num_extant,
                             thresholds[g], g+1);

      //printPedigree(pedigree);
    }

   int num_gen=mExtantChildren->size();
   for(int i=0; i<num_gen; i++)
   {
      vector<vector<int>*>* extants_lists=(*mExtantChildren)[i];
      //int num_indiv=extants_lists->size();
      for(int j=0; j<num_gen; j++)
      {
         //delete (*extants_lists)[j];
      }
      delete extants_lists;
   }
   delete mExtantChildren;

}

#endif
